@@ -54,16 +54,13 @@ function create_trove_accounts {
 
         create_service_user "trove"
 
-        if [[ "$KEYSTONE_CATALOG_BACKEND" = 'sql' ]]; then
-
-            local trove_service=$(get_or_create_service "trove" \
-                "database" "Trove Service")
-            get_or_create_endpoint $trove_service \
-                "$REGION_NAME" \
-                "http://$SERVICE_HOST:8779/v1.0/\$(tenant_id)s" \
-                "http://$SERVICE_HOST:8779/v1.0/\$(tenant_id)s" \
-                "http://$SERVICE_HOST:8779/v1.0/\$(tenant_id)s"
-        fi
+        local trove_service=$(get_or_create_service "trove" \
+            "database" "Trove Service")
+        get_or_create_endpoint $trove_service \
+            "$REGION_NAME" \
+            "http://$SERVICE_HOST:8779/v1.0/\$(tenant_id)s" \
+            "http://$SERVICE_HOST:8779/v1.0/\$(tenant_id)s" \
+            "http://$SERVICE_HOST:8779/v1.0/\$(tenant_id)s"
     fi
 }
 
@@ -103,16 +100,52 @@ function iniset_conditional {
 }
 
 
+# configure_nova_kvm() - update the nova hypervisor configuration if possible
+function configure_nova_kvm {
+    cpu="unknown"
+
+    if [ -e /sys/module/kvm_*/parameters/nested ]; then
+        reconfigure_nova="F"
+
+        if [ -e /sys/module/kvm_intel/parameters/nested ]; then
+            cpu="Intel"
+            if [[ "$(cat /sys/module/kvm_*/parameters/nested)" == "Y" ]]; then
+                reconfigure_nova="Y"
+            fi
+        elif [ -e /sys/module/kvm_amd/parameters/nested ]; then
+            cpu="AMD"
+            if [[ "$(cat /sys/module/kvm_*/parameters/nested)" == "1" ]]; then
+                reconfigure_nova="Y"
+            fi
+        fi
+
+        if [ "${reconfigure_nova}" == "Y" ]; then
+            NOVA_CONF_DIR=${NOVA_CONF_DIR:-/etc/nova}
+            NOVA_CONF=${NOVA_CONF:-${NOVA_CONF_DIR}/nova.conf}
+            iniset $NOVA_CONF libvirt cpu_mode "none"
+            iniset $NOVA_CONF libvirt virt_type "kvm"
+        fi
+    fi
+
+    virt_type=$(iniget $NOVA_CONF libvirt virt_type)
+    echo "configure_nova_kvm: using virt_type: ${virt_type} for cpu: ${cpu}."
+}
+
 # configure_trove() - Set config files, create data dirs, etc
 function configure_trove {
     setup_develop $TROVE_DIR
 
+    configure_nova_kvm
+
     # Create the trove conf dir and cache dirs if they don't exist
     sudo install -d -o $STACK_USER ${TROVE_CONF_DIR} ${TROVE_AUTH_CACHE_DIR}
 
     # Copy api-paste file over to the trove conf dir
     cp $TROVE_LOCAL_API_PASTE_INI $TROVE_API_PASTE_INI
 
+    # Copy the default policy file over to the trove conf dir
+    cp $TROVE_LOCAL_POLICY_JSON $TROVE_POLICY_JSON
+
     # (Re)create trove conf files
     rm -f $TROVE_CONF
     rm -f $TROVE_TASKMANAGER_CONF
@@ -126,6 +159,7 @@ function configure_trove {
         iniset_conditional $TROVE_CONF DEFAULT max_volumes_per_tenant $TROVE_MAX_VOLUMES_PER_TENANT
 
         iniset $TROVE_CONF DEFAULT rpc_backend "rabbit"
+        iniset $TROVE_CONF DEFAULT control_exchange trove
         iniset $TROVE_CONF oslo_messaging_rabbit rabbit_hosts $RABBIT_HOST
         iniset $TROVE_CONF oslo_messaging_rabbit rabbit_password $RABBIT_PASSWORD
         iniset $TROVE_CONF oslo_messaging_rabbit rabbit_userid $RABBIT_USERID
@@ -151,6 +185,7 @@ function configure_trove {
         iniset_conditional $TROVE_TASKMANAGER_CONF DEFAULT state_change_wait_time $TROVE_STATE_CHANGE_WAIT_TIME
 
         iniset $TROVE_TASKMANAGER_CONF DEFAULT rpc_backend "rabbit"
+        iniset $TROVE_TASKMANAGER_CONF DEFAULT control_exchange trove
         iniset $TROVE_TASKMANAGER_CONF oslo_messaging_rabbit rabbit_hosts $RABBIT_HOST
         iniset $TROVE_TASKMANAGER_CONF oslo_messaging_rabbit rabbit_password $RABBIT_PASSWORD
         iniset $TROVE_TASKMANAGER_CONF oslo_messaging_rabbit rabbit_userid $RABBIT_USERID
@@ -161,6 +196,20 @@ function configure_trove {
         iniset $TROVE_TASKMANAGER_CONF DEFAULT nova_proxy_admin_tenant_name trove
         iniset $TROVE_TASKMANAGER_CONF DEFAULT nova_proxy_admin_pass $RADMIN_USER_PASS
         iniset $TROVE_TASKMANAGER_CONF DEFAULT trove_auth_url $TROVE_AUTH_ENDPOINT
+
+        iniset $TROVE_TASKMANAGER_CONF cassandra tcp_ports 22,7000,7001,7199,9042,9160
+        iniset $TROVE_TASKMANAGER_CONF couchbase tcp_ports 22,8091,8092,4369,11209-11211,21100-21199
+        iniset $TROVE_TASKMANAGER_CONF couchdb tcp_ports 22,5984
+        iniset $TROVE_TASKMANAGER_CONF db2 tcp_ports 22,50000
+        iniset $TROVE_TASKMANAGER_CONF mariadb tcp_ports 22,3306,4444,4567,4568
+        iniset $TROVE_TASKMANAGER_CONF mongodb tcp_ports 22,2500,27017,27019
+        iniset $TROVE_TASKMANAGER_CONF mysql tcp_ports 22,3306
+        iniset $TROVE_TASKMANAGER_CONF percona tcp_ports 22,3306
+        iniset $TROVE_TASKMANAGER_CONF postgresql tcp_ports 22,5432
+        iniset $TROVE_TASKMANAGER_CONF pxc tcp_ports 22,3306,4444,4567,4568
+        iniset $TROVE_TASKMANAGER_CONF redis tcp_ports 22,6379,16379
+        iniset $TROVE_TASKMANAGER_CONF vertica tcp_ports 22,5433,5434,5444,5450,4803
+
         setup_trove_logging $TROVE_TASKMANAGER_CONF
     fi
 
@@ -177,6 +226,7 @@ function configure_trove {
         iniset $TROVE_CONDUCTOR_CONF DEFAULT nova_proxy_admin_pass $RADMIN_USER_PASS
         iniset $TROVE_CONDUCTOR_CONF DEFAULT trove_auth_url $TROVE_AUTH_ENDPOINT
         iniset $TROVE_CONDUCTOR_CONF DEFAULT control_exchange trove
+
         setup_trove_logging $TROVE_CONDUCTOR_CONF
     fi
 
@@ -185,7 +235,6 @@ function configure_trove {
 
     # Set up Guest Agent conf
     iniset $TROVE_GUESTAGENT_CONF DEFAULT rpc_backend "rabbit"
-
     iniset $TROVE_GUESTAGENT_CONF oslo_messaging_rabbit rabbit_password $RABBIT_PASSWORD
     iniset $TROVE_GUESTAGENT_CONF oslo_messaging_rabbit rabbit_userid $RABBIT_USERID
     iniset $TROVE_GUESTAGENT_CONF oslo_messaging_rabbit rabbit_hosts $TROVE_HOST_GATEWAY
@@ -198,6 +247,7 @@ function configure_trove {
     iniset $TROVE_GUESTAGENT_CONF DEFAULT ignore_users os_admin
     iniset $TROVE_GUESTAGENT_CONF DEFAULT log_dir /var/log/trove/
     iniset $TROVE_GUESTAGENT_CONF DEFAULT log_file trove-guestagent.log
+
     setup_trove_logging $TROVE_GUESTAGENT_CONF
 }
 
@@ -238,7 +288,7 @@ function init_trove {
     # The first part mimics the tempest setup, so make sure we have that.
     ALT_USERNAME=${ALT_USERNAME:-alt_demo}
     ALT_TENANT_NAME=${ALT_TENANT_NAME:-alt_demo}
-    get_or_create_project ${ALT_TENANT_NAME} default
+    ALT_TENANT_ID=$(get_or_create_project ${ALT_TENANT_NAME} default)
     get_or_create_user ${ALT_USERNAME} "$ADMIN_PASSWORD" "default" "alt_demo@example.com"
     get_or_add_user_project_role Member ${ALT_USERNAME} ${ALT_TENANT_NAME}
 
@@ -246,6 +296,19 @@ function init_trove {
     ADMIN_ALT_USERNAME=${ADMIN_ALT_USERNAME:-admin_${ALT_USERNAME}}
     get_or_create_user ${ADMIN_ALT_USERNAME} "$ADMIN_PASSWORD" "default" "admin_alt_demo@example.com"
     get_or_add_user_project_role admin ${ADMIN_ALT_USERNAME} ${ALT_TENANT_NAME}
+    # Now add these credentials to the clouds.yaml file
+    ADMIN_ALT_DEMO_CLOUD=devstack-alt-admin
+    CLOUDS_YAML=${CLOUDS_YAML:-/etc/openstack/clouds.yaml}
+    $TOP_DIR/tools/update_clouds_yaml.py \
+        --file ${CLOUDS_YAML} \
+        --os-cloud ${ADMIN_ALT_DEMO_CLOUD} \
+        --os-region-name ${REGION_NAME} \
+        --os-identity-api-version 3 \
+        ${CA_CERT_ARG} \
+        --os-auth-url ${KEYSTONE_AUTH_URI} \
+        --os-username ${ADMIN_ALT_USERNAME} \
+        --os-password ${ADMIN_PASSWORD} \
+        --os-project-name ${ALT_TENANT_NAME}
 
     # If no guest image is specified, skip remaining setup
     [ -z "$TROVE_GUEST_IMAGE_URL" ] && return 0
@@ -281,31 +344,139 @@ function init_trove {
     fi
 }
 
+# Create private IPv4 subnet
+# Note: This was taken from devstack:lib/neutron_plugins/services/l3 and will need to be maintained
+function _create_private_subnet_v4 {
+    local project_id=$1
+    local net_id=$2
+    local name=${3:-$PRIVATE_SUBNET_NAME}
+    local os_cloud=${4:-devstack-admin}
+
+    local subnet_params="--project $project_id "
+    subnet_params+="--ip-version 4 "
+    if [[ -n "$NETWORK_GATEWAY" ]]; then
+        subnet_params+="--gateway $NETWORK_GATEWAY "
+    fi
+    if [ -n $SUBNETPOOL_V4_ID ]; then
+        subnet_params+="--subnet-pool $SUBNETPOOL_V4_ID "
+    else
+        subnet_params+="--subnet-range $FIXED_RANGE "
+    fi
+    subnet_params+="--network $net_id $name"
+    local subnet_id
+    subnet_id=$(openstack --os-cloud $os_cloud --os-region "$REGION_NAME" subnet create $subnet_params | grep ' id ' | get_field 2)
+    die_if_not_set $LINENO subnet_id "Failure creating private IPv4 subnet for $project_id"
+    echo $subnet_id
+}
+
+# Create private IPv6 subnet
+# Note: This was taken from devstack:lib/neutron_plugins/services/l3 and will need to be maintained
+function _create_private_subnet_v6 {
+    local project_id=$1
+    local net_id=$2
+    local name=${3:-$IPV6_PRIVATE_SUBNET_NAME}
+    local os_cloud=${4:-devstack-admin}
+
+    die_if_not_set $LINENO IPV6_RA_MODE "IPV6 RA Mode not set"
+    die_if_not_set $LINENO IPV6_ADDRESS_MODE "IPV6 Address Mode not set"
+    local ipv6_modes="--ipv6-ra-mode $IPV6_RA_MODE --ipv6-address-mode $IPV6_ADDRESS_MODE"
+    local subnet_params="--project $project_id "
+    subnet_params+="--ip-version 6 "
+    if [[ -n "$IPV6_PRIVATE_NETWORK_GATEWAY" ]]; then
+        subnet_params+="--gateway $IPV6_PRIVATE_NETWORK_GATEWAY "
+    fi
+    if [ -n $SUBNETPOOL_V6_ID ]; then
+        subnet_params+="--subnet-pool $SUBNETPOOL_V6_ID "
+    else
+        subnet_params+="--subnet-range $FIXED_RANGE_V6 $ipv6_modes} "
+    fi
+    subnet_params+="--network $net_id $name "
+    local ipv6_subnet_id
+    ipv6_subnet_id=$(openstack --os-cloud $os_cloud --os-region "$REGION_NAME" subnet create $subnet_params | grep ' id ' | get_field 2)
+    die_if_not_set $LINENO ipv6_subnet_id "Failure creating private IPv6 subnet for $project_id"
+    echo $ipv6_subnet_id
+}
+
+# Set up a network on the alt_demo tenant.  Requires ROUTER_ID, REGION_NAME and IP_VERSION to be set
+function set_up_network() {
+    local CLOUD_USER=$1
+    local PROJECT_ID=$2
+    local NET_NAME=$3
+    local SUBNET_NAME=$4
+    local IPV6_SUBNET_NAME=$5
+
+    NEW_NET_ID=$(openstack --os-cloud ${CLOUD_USER} --os-region "$REGION_NAME" network create --project ${PROJECT_ID} "$NET_NAME" | grep ' id ' | get_field 2)
+    if [[ "$IP_VERSION" =~ 4.* ]]; then
+        NEW_SUBNET_ID=$(_create_private_subnet_v4 ${PROJECT_ID} ${NEW_NET_ID} ${SUBNET_NAME} ${CLOUD_USER})
+        openstack --os-cloud ${CLOUD_USER} --os-region "$REGION_NAME" router add subnet $ROUTER_ID $NEW_SUBNET_ID
+    fi
+    if [[ "$IP_VERSION" =~ .*6 ]]; then
+        NEW_IPV6_SUBNET_ID=$(_create_private_subnet_v6 ${PROJECT_ID} ${NEW_NET_ID} ${IPV6_SUBNET_NAME} ${CLOUD_USER})
+        openstack --os-cloud ${CLOUD_USER} --os-region "$REGION_NAME" router add subnet $ROUTER_ID $NEW_IPV6_SUBNET_ID
+    fi
+
+    echo $NEW_NET_ID
+}
+
 # finalize_trove_network() - do the last thing(s) before starting Trove
 function finalize_trove_network {
-    management_network_id=$(neutron --os-tenant-name admin --os-username admin --os-password $ADMIN_PASSWORD net-list | grep $PRIVATE_NETWORK_NAME | awk '{print $2}')
 
-    echo "finalize_trove_network: found network id $management_network_id"
+    echo "Finalizing Neutron networking for Trove"
+    echo "Dumping current network parameters:"
+    echo "  SERVICE_HOST: $SERVICE_HOST"
+    echo "  BRIDGE_IP: $BRIDGE_IP"
+    echo "  PUBLIC_NETWORK_GATEWAY: $PUBLIC_NETWORK_GATEWAY"
+    echo "  NETWORK_GATEWAY: $NETWORK_GATEWAY"
+    echo "  IPV4_ADDRS_SAFE_TO_USE: $IPV4_ADDRS_SAFE_TO_USE"
+    echo "  IPV6_ADDRS_SAFE_TO_USE: $IPV6_ADDRS_SAFE_TO_USE"
+    echo "  FIXED_RANGE: $FIXED_RANGE"
+    echo "  FLOATING_RANGE: $FLOATING_RANGE"
+    echo "  SUBNETPOOL_PREFIX_V4: $SUBNETPOOL_PREFIX_V4"
+    echo "  SUBNETPOOL_SIZE_V4: $SUBNETPOOL_SIZE_V4"
+    echo "  SUBNETPOOL_V4_ID: $SUBNETPOOL_V4_ID"
+    echo "  ROUTER_GW_IP: $ROUTER_GW_IP"
 
-    iniset $TROVE_CONF DEFAULT network_label_regex .*
-    iniset $TROVE_CONF DEFAULT ip_regex .*
-    iniset $TROVE_CONF DEFAULT black_list_regex ^10.0.1.*
-    iniset $TROVE_CONF DEFAULT default_neutron_networks $management_network_id
+    # Create the net/subnet for the alt_demo tenant so the int-tests have a proper network
+    echo "Creating network/subnets for ${ALT_TENANT_NAME} project"
+    ALT_PRIVATE_NETWORK_NAME=${TROVE_PRIVATE_NETWORK_NAME}
+    ALT_PRIVATE_SUBNET_NAME=${TROVE_PRIVATE_SUBNET_NAME}
+    ALT_PRIVATE_IPV6_SUBNET_NAME=ipv6-${ALT_PRIVATE_SUBNET_NAME}
+    ALT_NET_ID=$(set_up_network $ADMIN_ALT_DEMO_CLOUD $ALT_TENANT_ID $ALT_PRIVATE_NETWORK_NAME $ALT_PRIVATE_SUBNET_NAME $ALT_PRIVATE_IPV6_SUBNET_NAME)
+    echo "Created network ${ALT_PRIVATE_NETWORK_NAME} (${ALT_NET_ID})"
+
+    # Set up a management network to test that functionality
+    ALT_MGMT_NETWORK_NAME=trove-mgmt
+    ALT_MGMT_SUBNET_NAME=${ALT_MGMT_NETWORK_NAME}-subnet
+    ALT_MGMT_IPV6_SUBNET_NAME=ipv6-${ALT_MGMT_SUBNET_NAME}
+    ALT_MGMT_ID=$(set_up_network $ADMIN_ALT_DEMO_CLOUD $ALT_TENANT_ID $ALT_MGMT_NETWORK_NAME $ALT_MGMT_SUBNET_NAME $ALT_MGMT_IPV6_SUBNET_NAME)
+    echo "Created network ${ALT_MGMT_NETWORK_NAME} (${ALT_MGMT_ID})"
+
+    # Make sure we can reach the VMs
+    local replace_range=${SUBNETPOOL_PREFIX_V4}
+    if [[ -z "${SUBNETPOOL_V4_ID}" ]]; then
+        replace_range=${FIXED_RANGE}
+    fi
+    sudo ip route replace $replace_range via $ROUTER_GW_IP
+
+    echo "Neutron network list:"
+    openstack --os-cloud devstack-admin --os-region "$REGION_NAME" network list
+
+    # Now make sure the conf settings are right
+    iniset $TROVE_CONF DEFAULT network_label_regex "${ALT_PRIVATE_NETWORK_NAME}"
+    iniset $TROVE_CONF DEFAULT ip_regex ""
+    iniset $TROVE_CONF DEFAULT black_list_regex ""
+    # Don't use a default network for now, until the neutron issues are figured out
+    #iniset $TROVE_CONF DEFAULT default_neutron_networks "${ALT_MGMT_ID}"
+    iniset $TROVE_CONF DEFAULT default_neutron_networks ""
     iniset $TROVE_CONF DEFAULT network_driver trove.network.neutron.NeutronDriver
 
+    iniset $TROVE_TASKMANAGER_CONF DEFAULT network_label_regex "${ALT_PRIVATE_NETWORK_NAME}"
+    iniset $TROVE_TASKMANAGER_CONF DEFAULT ip_regex ""
+    iniset $TROVE_TASKMANAGER_CONF DEFAULT black_list_regex ""
+    # Don't use a default network for now, until the neutron issues are figured out
+    #iniset $TROVE_TASKMANAGER_CONF DEFAULT default_neutron_networks "${ALT_MGMT_ID}"
+    iniset $TROVE_CONF DEFAULT default_neutron_networks ""
     iniset $TROVE_TASKMANAGER_CONF DEFAULT network_driver trove.network.neutron.NeutronDriver
-    iniset $TROVE_TASKMANAGER_CONF cassandra tcp_ports 22,7000,7001,7199,9042,9160
-    iniset $TROVE_TASKMANAGER_CONF couchbase tcp_ports 22,8091,8092,4369,11209-11211,21100-21199
-    iniset $TROVE_TASKMANAGER_CONF couchdb tcp_ports 22,5984
-    iniset $TROVE_TASKMANAGER_CONF db2 tcp_ports 22,50000
-    iniset $TROVE_TASKMANAGER_CONF mariadb tcp_ports 22,3306,4444,4567,4568
-    iniset $TROVE_TASKMANAGER_CONF mongodb tcp_ports 22,2500,27017,27019
-    iniset $TROVE_TASKMANAGER_CONF mysql tcp_ports 22,3306
-    iniset $TROVE_TASKMANAGER_CONF percona tcp_ports 22,3306
-    iniset $TROVE_TASKMANAGER_CONF postgresql tcp_ports 22,5432
-    iniset $TROVE_TASKMANAGER_CONF pxc tcp_ports 22,3306,4444,4567,4568
-    iniset $TROVE_TASKMANAGER_CONF redis tcp_ports 22,6379,16379
-    iniset $TROVE_TASKMANAGER_CONF vertica tcp_ports 22,5433,5434,5444,5450,4803
 }
 
 # start_trove() - Start running processes, including screen
