@@ -22,7 +22,6 @@ gettext.install('trove', unicode=1)
 
 
 from oslo_log import log as logging
-from oslo_log.versionutils import deprecated
 
 from trove.common import cfg
 from trove.common import exception
@@ -47,9 +46,8 @@ class Commands(object):
     def db_upgrade(self, version=None, repo_path=None):
         self.db_api.db_upgrade(CONF, version, repo_path=repo_path)
 
-    @deprecated(as_of=deprecated.MITAKA)
     def db_downgrade(self, version, repo_path=None):
-        self.db_api.db_downgrade(CONF, version, repo_path=repo_path)
+        raise SystemExit(_("Database downgrade is no longer supported."))
 
     def execute(self):
         exec_method = getattr(self, CONF.action.name)
@@ -140,12 +138,6 @@ def main():
             'latest version.')
         parser.add_argument('--repo_path', help=repo_path_help)
 
-        parser = subparser.add_parser(
-            'db_downgrade', description='Downgrade the database to the '
-            'specified version.')
-        parser.add_argument('version', help='Target version.')
-        parser.add_argument('--repo_path', help=repo_path_help)
-
         parser = subparser.add_parser(
             'datastore_update', description='Add or update a datastore. '
             'If the datastore already exists, the default version will be '
