@@ -23,10 +23,17 @@ from oslo_log import log as logging
 from oslo_middleware import cors
 from osprofiler import opts as profiler
 
+from trove.common.i18n import _
 from trove.version import version_info as version
 
 
+LOG = logging.getLogger(__name__)
 UNKNOWN_SERVICE_ID = 'unknown-service-id-error'
+HEAT_REMOVAL_DEPRECATION_WARNING = _('Support for heat templates in Trove is '
+                                     'scheduled for removal. You will no '
+                                     'longer be able to provide a heat '
+                                     'template to Trove for the provisioning '
+                                     'of resources.')
 
 path_opts = [
     cfg.StrOpt('pybasedir',
@@ -52,35 +59,51 @@ common_opts = [
                      'configured usage_timeout.'),
     cfg.StrOpt('os_region_name', default='RegionOne',
                help='Region name of this node. Used when searching catalog.'),
-    cfg.StrOpt('nova_compute_url', help='URL without the tenant segment.'),
+    cfg.URIOpt('nova_compute_url', help='URL without the tenant segment.'),
     cfg.StrOpt('nova_compute_service_type', default='compute',
                help='Service type to use when searching catalog.'),
     cfg.StrOpt('nova_compute_endpoint_type', default='publicURL',
                help='Service endpoint type to use when searching catalog.'),
     cfg.StrOpt('nova_client_version', default='2.12',
-               help="The version of of the compute service client."),
-    cfg.StrOpt('neutron_url', help='URL without the tenant segment.'),
+               help="The version of the compute service client."),
+    cfg.URIOpt('neutron_url', help='URL without the tenant segment.'),
     cfg.StrOpt('neutron_service_type', default='network',
                help='Service type to use when searching catalog.'),
     cfg.StrOpt('neutron_endpoint_type', default='publicURL',
                help='Service endpoint type to use when searching catalog.'),
-    cfg.StrOpt('cinder_url', help='URL without the tenant segment.'),
+    cfg.URIOpt('cinder_url', help='URL without the tenant segment.'),
     cfg.StrOpt('cinder_service_type', default='volumev2',
                help='Service type to use when searching catalog.'),
     cfg.StrOpt('cinder_endpoint_type', default='publicURL',
                help='Service endpoint type to use when searching catalog.'),
-    cfg.StrOpt('heat_url', help='URL without the tenant segment.'),
+    cfg.URIOpt('heat_url', deprecated_for_removal=True,
+               deprecated_reason=HEAT_REMOVAL_DEPRECATION_WARNING,
+               help='URL without the tenant segment.'),
     cfg.StrOpt('heat_service_type', default='orchestration',
+               deprecated_for_removal=True,
+               deprecated_reason=HEAT_REMOVAL_DEPRECATION_WARNING,
                help='Service type to use when searching catalog.'),
     cfg.StrOpt('heat_endpoint_type', default='publicURL',
+               deprecated_for_removal=True,
+               deprecated_reason=HEAT_REMOVAL_DEPRECATION_WARNING,
                help='Service endpoint type to use when searching catalog.'),
-    cfg.StrOpt('swift_url', help='URL ending in AUTH_.'),
+    cfg.URIOpt('swift_url', help='URL ending in ``AUTH_``.'),
     cfg.StrOpt('swift_service_type', default='object-store',
                help='Service type to use when searching catalog.'),
     cfg.StrOpt('swift_endpoint_type', default='publicURL',
                help='Service endpoint type to use when searching catalog.'),
-    cfg.StrOpt('trove_auth_url', default='http://0.0.0.0:5000/v2.0',
+    cfg.URIOpt('glance_url', help='URL ending in ``AUTH_``.'),
+    cfg.StrOpt('glance_service_type', default='image',
+               help='Service type to use when searching catalog.'),
+    cfg.StrOpt('glance_endpoint_type', default='publicURL',
+               help='Service endpoint type to use when searching catalog.'),
+    cfg.URIOpt('trove_auth_url', default='http://0.0.0.0:5000/v2.0',
                help='Trove authentication URL.'),
+    cfg.StrOpt('trove_url', help='URL without the tenant segment.'),
+    cfg.StrOpt('trove_service_type', default='database',
+               help='Service type to use when searching catalog.'),
+    cfg.StrOpt('trove_endpoint_type', default='publicURL',
+               help='Service endpoint type to use when searching catalog.'),
     cfg.IPOpt('host', default='0.0.0.0',
               help='Host to listen for RPC messages.'),
     cfg.IntOpt('report_interval', default=30,
@@ -95,17 +118,17 @@ common_opts = [
     cfg.StrOpt('dns_instance_entry_factory',
                default='trove.dns.driver.DnsInstanceEntryFactory',
                help='Factory for adding DNS entries.'),
-    cfg.StrOpt('dns_hostname', default="",
-               help='Hostname used for adding DNS entries.'),
+    cfg.HostnameOpt('dns_hostname', default="localhost",
+                    help='Hostname used for adding DNS entries.'),
     cfg.StrOpt('dns_account_id', default="",
                help='Tenant ID for DNSaaS.'),
-    cfg.StrOpt('dns_endpoint_url', default="0.0.0.0",
+    cfg.URIOpt('dns_endpoint_url', default="http://0.0.0.0",
                help='Endpoint URL for DNSaaS.'),
     cfg.StrOpt('dns_service_type', default="",
                help='Service Type for DNSaaS.'),
     cfg.StrOpt('dns_region', default="",
                help='Region name for DNSaaS.'),
-    cfg.StrOpt('dns_auth_url', default="",
+    cfg.URIOpt('dns_auth_url', default="http://0.0.0.0",
                help='Authentication URL for DNSaaS.'),
     cfg.StrOpt('dns_domain_name', default="",
                help='Domain name used for adding DNS entries.'),
@@ -113,7 +136,7 @@ common_opts = [
                help='Username for DNSaaS.'),
     cfg.StrOpt('dns_passkey', default="", secret=True,
                help='Passkey for DNSaaS.'),
-    cfg.StrOpt('dns_management_base_url', default="",
+    cfg.URIOpt('dns_management_base_url', default="http://0.0.0.0",
                help='Management URL for DNSaaS.'),
     cfg.IntOpt('dns_ttl', default=300,
                help='Time (in seconds) before a refresh of DNS information '
@@ -134,11 +157,11 @@ common_opts = [
                help='Page size for listing configurations.'),
     cfg.IntOpt('modules_page_size', default=20,
                help='Page size for listing modules.'),
-    cfg.IntOpt('agent_call_low_timeout', default=5,
+    cfg.IntOpt('agent_call_low_timeout', default=15,
                help="Maximum time (in seconds) to wait for Guest Agent 'quick'"
                     "requests (such as retrieving a list of users or "
                     "databases)."),
-    cfg.IntOpt('agent_call_high_timeout', default=60,
+    cfg.IntOpt('agent_call_high_timeout', default=60 * 10,
                help="Maximum time (in seconds) to wait for Guest Agent 'slow' "
                     "requests (such as restarting the database)."),
     cfg.IntOpt('agent_replication_snapshot_timeout', default=36000,
@@ -146,7 +169,7 @@ common_opts = [
                     'Agent replication snapshot.'),
     # The guest_id opt definition must match the one in cmd/guest.py
     cfg.StrOpt('guest_id', default=None, help="ID of the Guest Instance."),
-    cfg.IntOpt('state_change_wait_time', default=3 * 60,
+    cfg.IntOpt('state_change_wait_time', default=60 * 10,
                help='Maximum time (in seconds) to wait for a state change.'),
     cfg.IntOpt('state_change_poll_time', default=3,
                help='Interval between state change poll requests (seconds).'),
@@ -169,12 +192,12 @@ common_opts = [
     cfg.StrOpt('mount_options', default='defaults,noatime',
                help='Options to use when mounting a volume.'),
     cfg.IntOpt('max_instances_per_tenant',
-               default=5,
+               default=10,
                help='Default maximum number of instances per tenant.',
                deprecated_name='max_instances_per_user'),
-    cfg.IntOpt('max_accepted_volume_size', default=5,
+    cfg.IntOpt('max_accepted_volume_size', default=10,
                help='Default maximum volume size (in GB) for an instance.'),
-    cfg.IntOpt('max_volumes_per_tenant', default=20,
+    cfg.IntOpt('max_volumes_per_tenant', default=40,
                help='Default maximum volume capacity (in GB) spanning across '
                     'all Trove volumes per tenant.',
                deprecated_name='max_volumes_per_user'),
@@ -196,7 +219,8 @@ common_opts = [
     cfg.BoolOpt('use_nova_server_volume', default=False,
                 help='Whether to provision a Cinder volume for the '
                      'Nova instance.'),
-    cfg.BoolOpt('use_heat', default=False,
+    cfg.BoolOpt('use_heat', default=False, deprecated_for_removal=True,
+                deprecated_reason=HEAT_REMOVAL_DEPRECATION_WARNING,
                 help='Use Heat for provisioning.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
@@ -213,14 +237,15 @@ common_opts = [
                help='Maximum time (in seconds) to wait for a server delete.'),
     cfg.IntOpt('volume_time_out', default=60,
                help='Maximum time (in seconds) to wait for a volume attach.'),
-    cfg.IntOpt('heat_time_out', default=60,
+    cfg.IntOpt('heat_time_out', default=60, deprecated_for_removal=True,
+               deprecated_reason=HEAT_REMOVAL_DEPRECATION_WARNING,
                help='Maximum time (in seconds) to wait for a Heat request to '
                     'complete.'),
     cfg.IntOpt('reboot_time_out', default=60 * 2,
                help='Maximum time (in seconds) to wait for a server reboot.'),
     cfg.IntOpt('dns_time_out', default=60 * 2,
                help='Maximum time (in seconds) to wait for a DNS entry add.'),
-    cfg.IntOpt('resize_time_out', default=60 * 10,
+    cfg.IntOpt('resize_time_out', default=60 * 15,
                help='Maximum time (in seconds) to wait for a server resize.'),
     cfg.IntOpt('revert_time_out', default=60 * 10,
                help='Maximum time (in seconds) to wait for a server resize '
@@ -231,8 +256,6 @@ common_opts = [
                 help="Permissions to grant to the 'root' user."),
     cfg.BoolOpt('root_grant_option', default=True,
                 help="Assign the 'root' user GRANT permissions."),
-    cfg.IntOpt('default_password_length', default=36,
-               help='Character length of generated passwords.'),
     cfg.IntOpt('http_get_rate', default=200,
                help="Maximum number of HTTP 'GET' requests (per minute)."),
     cfg.IntOpt('http_post_rate', default=200,
@@ -308,17 +331,24 @@ common_opts = [
     cfg.StrOpt('remote_cinder_client',
                default='trove.common.remote.cinder_client',
                help='Client to send Cinder calls to.'),
-    cfg.StrOpt('remote_heat_client',
+    cfg.StrOpt('remote_heat_client', deprecated_for_removal=True,
+               deprecated_reason=HEAT_REMOVAL_DEPRECATION_WARNING,
                default='trove.common.remote.heat_client',
                help='Client to send Heat calls to.'),
     cfg.StrOpt('remote_swift_client',
                default='trove.common.remote.swift_client',
                help='Client to send Swift calls to.'),
+    cfg.StrOpt('remote_trove_client',
+               default='trove.common.trove_remote.trove_client',
+               help='Client to send Trove calls to.'),
+    cfg.StrOpt('remote_glance_client',
+               default='trove.common.glance_remote.glance_client',
+               help='Client to send Glance calls to.'),
     cfg.StrOpt('exists_notification_transformer',
                help='Transformer for exists notifications.'),
     cfg.IntOpt('exists_notification_interval', default=3600,
                help='Seconds to wait between pushing events.'),
-    cfg.IntOpt('quota_notification_interval', default=3600,
+    cfg.IntOpt('quota_notification_interval',
                help='Seconds to wait between pushing events.'),
     cfg.DictOpt('notification_service_id',
                 default={'mysql': '2f3ff068-2bfb-4f70-9a9d-a6bb65bc084b',
@@ -390,7 +420,7 @@ common_opts = [
                help="Describes the actual network manager used for "
                     "the management of network attributes "
                     "(security groups, floating IPs, etc.)."),
-    cfg.IntOpt('usage_timeout', default=900,
+    cfg.IntOpt('usage_timeout', default=60 * 30,
                help='Maximum time (in seconds) to wait for a Guest to become '
                     'active.'),
     cfg.IntOpt('restore_usage_timeout', default=36000,
@@ -404,7 +434,7 @@ common_opts = [
                     'become alive.'),
     cfg.StrOpt('module_aes_cbc_key', default='module_aes_cbc_key',
                help='OpenSSL aes_cbc key for module encryption.'),
-    cfg.ListOpt('module_types', default=['ping'],
+    cfg.ListOpt('module_types', default=['ping', 'new_relic_license'],
                 help='A list of module types supported. A module type '
                      'corresponds to the name of a ModuleDriver.'),
     cfg.StrOpt('guest_log_container_name',
@@ -414,6 +444,16 @@ common_opts = [
                help='Maximum size of a chunk saved in guest log container.'),
     cfg.IntOpt('guest_log_expiry', default=2592000,
                help='Expiry (in seconds) of objects in guest log container.'),
+    cfg.BoolOpt('enable_secure_rpc_messaging', default=True,
+                help='Should RPC messaging traffic be secured by encryption.'),
+    cfg.StrOpt('taskmanager_rpc_encr_key',
+               default='bzH6y0SGmjuoY0FNSTptrhgieGXNDX6PIhvz',
+               help='Key (OpenSSL aes_cbc) for taskmanager RPC encryption.'),
+    cfg.StrOpt('inst_rpc_key_encr_key',
+               default='emYjgHFqfXNB1NGehAFIUeoyw4V4XwWHEaKP',
+               help='Key (OpenSSL aes_cbc) to encrypt instance keys in DB.'),
+    cfg.StrOpt('instance_rpc_encr_key',
+               help='Key (OpenSSL aes_cbc) for instance RPC encryption.'),
 ]
 
 
@@ -481,6 +521,8 @@ mysql_group = cfg.OptGroup(
     'mysql', title='MySQL options',
     help="Oslo option group designed for MySQL datastore")
 mysql_opts = [
+    cfg.BoolOpt('icmp', default=False,
+                help='Whether to permit ICMP.'),
     cfg.ListOpt('tcp_ports', default=["3306"],
                 help='List of TCP ports and/or port ranges to open '
                      'in the security group (only applicable '
@@ -547,7 +589,14 @@ mysql_opts = [
                help='List of Guest Logs to expose for publishing.'),
     cfg.IntOpt('guest_log_long_query_time', default=1000,
                help='The time in milliseconds that a statement must take in '
-                    'in order to be logged in the slow_query log.'),
+                    'in order to be logged in the slow_query log.',
+               deprecated_for_removal=True,
+               deprecated_reason='Will be replaced by a configuration group '
+               'option: long_query_time'),
+    cfg.IntOpt('default_password_length', default=36,
+               help='Character length of generated passwords.',
+               deprecated_name='default_password_length',
+               deprecated_group='DEFAULT'),
 ]
 
 # Percona
@@ -555,6 +604,8 @@ percona_group = cfg.OptGroup(
     'percona', title='Percona options',
     help="Oslo option group designed for Percona datastore")
 percona_opts = [
+    cfg.BoolOpt('icmp', default=False,
+                help='Whether to permit ICMP.'),
     cfg.ListOpt('tcp_ports', default=["3306"],
                 help='List of TCP ports and/or port ranges to open '
                      'in the security group (only applicable '
@@ -625,7 +676,15 @@ percona_opts = [
                help='List of Guest Logs to expose for publishing.'),
     cfg.IntOpt('guest_log_long_query_time', default=1000,
                help='The time in milliseconds that a statement must take in '
-                    'in order to be logged in the slow_query log.'),
+                    'in order to be logged in the slow_query log.',
+               deprecated_for_removal=True,
+               deprecated_reason='Will be replaced by a configuration group '
+               'option: long_query_time'),
+    cfg.IntOpt('default_password_length',
+               default='${mysql.default_password_length}',
+               help='Character length of generated passwords.',
+               deprecated_name='default_password_length',
+               deprecated_group='DEFAULT'),
 ]
 
 # Percona XtraDB Cluster
@@ -633,6 +692,8 @@ pxc_group = cfg.OptGroup(
     'pxc', title='Percona XtraDB Cluster options',
     help="Oslo option group designed for Percona XtraDB Cluster datastore")
 pxc_opts = [
+    cfg.BoolOpt('icmp', default=False,
+                help='Whether to permit ICMP.'),
     cfg.ListOpt('tcp_ports', default=["3306", "4444", "4567", "4568"],
                 help='List of TCP ports and/or port ranges to open '
                      'in the security group (only applicable '
@@ -707,7 +768,15 @@ pxc_opts = [
                help='List of Guest Logs to expose for publishing.'),
     cfg.IntOpt('guest_log_long_query_time', default=1000,
                help='The time in milliseconds that a statement must take in '
-                    'in order to be logged in the slow_query log.'),
+                    'in order to be logged in the slow_query log.',
+               deprecated_for_removal=True,
+               deprecated_reason='Will be replaced by a configuration group '
+               'option: long_query_time'),
+    cfg.IntOpt('default_password_length',
+               default='${mysql.default_password_length}',
+               help='Character length of generated passwords.',
+               deprecated_name='default_password_length',
+               deprecated_group='DEFAULT'),
 ]
 
 
@@ -716,6 +785,8 @@ redis_group = cfg.OptGroup(
     'redis', title='Redis options',
     help="Oslo option group designed for Redis datastore")
 redis_opts = [
+    cfg.BoolOpt('icmp', default=False,
+                help='Whether to permit ICMP.'),
     cfg.ListOpt('tcp_ports', default=["6379", "16379"],
                 help='List of TCP ports and/or port ranges to open '
                      'in the security group (only applicable '
@@ -780,6 +851,10 @@ redis_opts = [
                help='Root controller implementation for redis.'),
     cfg.StrOpt('guest_log_exposed_logs', default='',
                help='List of Guest Logs to expose for publishing.'),
+    cfg.IntOpt('default_password_length', default=36,
+               help='Character length of generated passwords.',
+               deprecated_name='default_password_length',
+               deprecated_group='DEFAULT'),
 ]
 
 # Cassandra
@@ -787,6 +862,8 @@ cassandra_group = cfg.OptGroup(
     'cassandra', title='Cassandra options',
     help="Oslo option group designed for Cassandra datastore")
 cassandra_opts = [
+    cfg.BoolOpt('icmp', default=False,
+                help='Whether to permit ICMP.'),
     cfg.ListOpt('tcp_ports', default=["7000", "7001", "7199", "9042", "9160"],
                 help='List of TCP ports and/or port ranges to open '
                      'in the security group (only applicable '
@@ -835,8 +912,12 @@ cassandra_opts = [
     cfg.ListOpt('ignore_dbs', default=['system', 'system_auth',
                                        'system_traces'],
                 help='Databases to exclude when listing databases.'),
-    cfg.StrOpt('guest_log_exposed_logs', default='',
+    cfg.StrOpt('guest_log_exposed_logs', default='system',
                help='List of Guest Logs to expose for publishing.'),
+    cfg.StrOpt('system_log_level',
+               choices=['ALL', 'TRACE', 'DEBUG', 'INFO', 'WARN', 'ERROR'],
+               default='INFO',
+               help='Cassandra log verbosity.'),
     cfg.BoolOpt('cluster_support', default=True,
                 help='Enable clusters to be created and managed.'),
     cfg.StrOpt('api_strategy',
@@ -853,6 +934,30 @@ cassandra_opts = [
                '.cassandra.guestagent.CassandraGuestAgentStrategy',
                help='Class that implements datastore-specific Guest Agent API '
                     'logic.'),
+    cfg.IntOpt('default_password_length', default=36,
+               help='Character length of generated passwords.',
+               deprecated_name='default_password_length',
+               deprecated_group='DEFAULT'),
+    cfg.BoolOpt('enable_cluster_instance_backup',
+                default=False,
+                help='Allows backup of single instance in the cluster.'),
+    cfg.BoolOpt('enable_saslauthd', default=False,
+                help='Enable the saslauth daemon.'),
+    cfg.StrOpt('user_controller',
+               default='trove.extensions.cassandra.service.'
+               'CassandraUserController',
+               help='User controller implementation.'),
+    cfg.StrOpt('database_controller',
+               default='trove.extensions.cassandra.service.'
+               'CassandraDatabaseController',
+               help='Database controller implementation.'),
+    cfg.StrOpt('user_access_controller',
+               default='trove.extensions.cassandra.service.'
+               'CassandraUserAccessController',
+               help='User access controller implementation.'),
+    cfg.IntOpt('node_sync_time', default=60,
+               help='Time (in seconds) given to a node after a state change '
+               'to finish rejoining the cluster.'),
 ]
 
 # Couchbase
@@ -860,6 +965,8 @@ couchbase_group = cfg.OptGroup(
     'couchbase', title='Couchbase options',
     help="Oslo option group designed for Couchbase datastore")
 couchbase_opts = [
+    cfg.BoolOpt('icmp', default=False,
+                help='Whether to permit ICMP.'),
     cfg.ListOpt('tcp_ports',
                 default=["8091", "8092", "4369", "11209-11211",
                          "21100-21199"],
@@ -911,6 +1018,10 @@ couchbase_opts = [
                help='Root controller implementation for couchbase.'),
     cfg.StrOpt('guest_log_exposed_logs', default='',
                help='List of Guest Logs to expose for publishing.'),
+    cfg.IntOpt('default_password_length', default=24, min=6, max=24,
+               help='Character length of generated passwords.',
+               deprecated_name='default_password_length',
+               deprecated_group='DEFAULT'),
 ]
 
 # MongoDB
@@ -918,7 +1029,9 @@ mongodb_group = cfg.OptGroup(
     'mongodb', title='MongoDB options',
     help="Oslo option group designed for MongoDB datastore")
 mongodb_opts = [
-    cfg.ListOpt('tcp_ports', default=["2500", "27017"],
+    cfg.BoolOpt('icmp', default=False,
+                help='Whether to permit ICMP.'),
+    cfg.ListOpt('tcp_ports', default=["2500", "27017", "27019"],
                 help='List of TCP ports and/or port ranges to open '
                      'in the security group (only applicable '
                      'if trove_security_groups_support is True).'),
@@ -993,10 +1106,15 @@ mongodb_opts = [
                help='Maximum time to wait (in seconds) for a replica set '
                     'initialization process to complete.'),
     cfg.StrOpt('root_controller',
-               default='trove.extensions.common.service.DefaultRootController',
+               default='trove.extensions.mongodb.service.'
+                       'MongoDBRootController',
                help='Root controller implementation for mongodb.'),
     cfg.StrOpt('guest_log_exposed_logs', default='',
                help='List of Guest Logs to expose for publishing.'),
+    cfg.IntOpt('default_password_length', default=36,
+               help='Character length of generated passwords.',
+               deprecated_name='default_password_length',
+               deprecated_group='DEFAULT'),
 ]
 
 # PostgreSQL
@@ -1004,6 +1122,8 @@ postgresql_group = cfg.OptGroup(
     'postgresql', title='PostgreSQL options',
     help="Oslo option group for the PostgreSQL datastore.")
 postgresql_opts = [
+    cfg.BoolOpt('icmp', default=False,
+                help='Whether to permit ICMP.'),
     cfg.ListOpt('tcp_ports', default=["5432"],
                 help='List of TCP ports and/or port ranges to open '
                      'in the security group (only applicable '
@@ -1014,15 +1134,27 @@ postgresql_opts = [
                      'if trove_security_groups_support is True).'),
     cfg.PortOpt('postgresql_port', default=5432,
                 help='The TCP port the server listens on.'),
-    cfg.StrOpt('backup_strategy', default='PgDump',
+    cfg.StrOpt('backup_strategy', default='PgBaseBackup',
                help='Default strategy to perform backups.'),
-    cfg.DictOpt('backup_incremental_strategy', default={},
+    cfg.DictOpt('backup_incremental_strategy',
+                default={'PgBaseBackup': 'PgBaseBackupIncremental'},
                 help='Incremental Backup Runner based on the default '
                 'strategy. For strategies that do not implement an '
                 'incremental, the runner will use the default full backup.'),
+    cfg.StrOpt('replication_strategy',
+               default='PostgresqlReplicationStreaming',
+               help='Default strategy for replication.'),
+    cfg.StrOpt('replication_namespace',
+               default='trove.guestagent.strategies.replication.experimental.'
+                       'postgresql_impl',
+               help='Namespace to load replication strategies from.'),
     cfg.StrOpt('mount_point', default='/var/lib/postgresql',
                help="Filesystem path for mounting "
                "volumes if volume support is enabled."),
+    cfg.StrOpt('wal_archive_location', default='/mnt/wal_archive',
+               help="Filesystem path storing WAL archive files when "
+                    "WAL-shipping based backups or replication "
+                    "is enabled."),
     cfg.BoolOpt('root_on_create', default=False,
                 help='Enable the automatic creation of the root user for the '
                 'service during instance-create. The generated password for '
@@ -1040,9 +1172,10 @@ postgresql_opts = [
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb'),
     cfg.ListOpt('ignore_users', default=['os_admin', 'postgres', 'root']),
-    cfg.ListOpt('ignore_dbs', default=['postgres']),
+    cfg.ListOpt('ignore_dbs', default=['os_admin', 'postgres']),
     cfg.StrOpt('root_controller',
-               default='trove.extensions.common.service.DefaultRootController',
+               default='trove.extensions.postgresql.service'
+               '.PostgreSQLRootController',
                help='Root controller implementation for postgresql.'),
     cfg.StrOpt('guest_log_exposed_logs', default='general',
                help='List of Guest Logs to expose for publishing.'),
@@ -1050,7 +1183,14 @@ postgresql_opts = [
                help="The time in milliseconds that a statement must take in "
                     "in order to be logged in the 'general' log.  A value of "
                     "'0' logs all statements, while '-1' turns off "
-                    "statement logging."),
+                    "statement logging.",
+               deprecated_for_removal=True,
+               deprecated_reason='Will be replaced by configuration group '
+               'option: log_min_duration_statement'),
+    cfg.IntOpt('default_password_length', default=36,
+               help='Character length of generated passwords.',
+               deprecated_name='default_password_length',
+               deprecated_group='DEFAULT'),
 ]
 
 # Apache CouchDB
@@ -1058,6 +1198,8 @@ couchdb_group = cfg.OptGroup(
     'couchdb', title='CouchDB options',
     help="Oslo option group designed for CouchDB datastore")
 couchdb_opts = [
+    cfg.BoolOpt('icmp', default=False,
+                help='Whether to permit ICMP.'),
     cfg.ListOpt('tcp_ports',
                 default=["5984"],
                 help='List of TCP ports and/or port ranges to open '
@@ -1107,7 +1249,10 @@ couchdb_opts = [
                 help='Databases to exclude when listing databases.',
                 deprecated_name='ignore_dbs',
                 deprecated_group='DEFAULT'),
-
+    cfg.IntOpt('default_password_length', default=36,
+               help='Character length of generated passwords.',
+               deprecated_name='default_password_length',
+               deprecated_group='DEFAULT'),
 ]
 
 # Vertica
@@ -1115,6 +1260,8 @@ vertica_group = cfg.OptGroup(
     'vertica', title='Vertica options',
     help="Oslo option group designed for Vertica datastore")
 vertica_opts = [
+    cfg.BoolOpt('icmp', default=False,
+                help='Whether to permit ICMP.'),
     cfg.ListOpt('tcp_ports',
                 default=["5433", "5434", "22", "5444", "5450", "4803"],
                 help='List of TCP ports and/or port ranges to open '
@@ -1172,6 +1319,10 @@ vertica_opts = [
                help='List of Guest Logs to expose for publishing.'),
     cfg.IntOpt('min_ksafety', default=0,
                help='Minimum k-safety setting permitted for vertica clusters'),
+    cfg.IntOpt('default_password_length', default=36,
+               help='Character length of generated passwords.',
+               deprecated_name='default_password_length',
+               deprecated_group='DEFAULT'),
 ]
 
 # DB2
@@ -1179,6 +1330,8 @@ db2_group = cfg.OptGroup(
     'db2', title='DB2 options',
     help="Oslo option group designed for DB2 datastore")
 db2_opts = [
+    cfg.BoolOpt('icmp', default=False,
+                help='Whether to permit ICMP.'),
     cfg.ListOpt('tcp_ports',
                 default=["50000"],
                 help='List of TCP ports and/or port ranges to open '
@@ -1195,7 +1348,7 @@ db2_opts = [
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
-    cfg.StrOpt('backup_strategy', default='DB2Backup',
+    cfg.StrOpt('backup_strategy', default='DB2OfflineBackup',
                help='Default strategy to perform backups.'),
     cfg.StrOpt('replication_strategy', default=None,
                help='Default strategy for replication.'),
@@ -1226,6 +1379,10 @@ db2_opts = [
                help='Root controller implementation for db2.'),
     cfg.StrOpt('guest_log_exposed_logs', default='',
                help='List of Guest Logs to expose for publishing.'),
+    cfg.IntOpt('default_password_length', default=36,
+               help='Character length of generated passwords.',
+               deprecated_name='default_password_length',
+               deprecated_group='DEFAULT'),
 ]
 
 # MariaDB
@@ -1233,6 +1390,8 @@ mariadb_group = cfg.OptGroup(
     'mariadb', title='MariaDB options',
     help="Oslo option group designed for MariaDB datastore")
 mariadb_opts = [
+    cfg.BoolOpt('icmp', default=False,
+                help='Whether to permit ICMP.'),
     cfg.ListOpt('tcp_ports', default=["3306", "4444", "4567", "4568"],
                 help='List of TCP ports and/or port ranges to open '
                      'in the security group (only applicable '
@@ -1241,7 +1400,7 @@ mariadb_opts = [
                 help='List of UDP ports and/or port ranges to open '
                      'in the security group (only applicable '
                      'if trove_security_groups_support is True).'),
-    cfg.StrOpt('backup_strategy', default='InnoBackupEx',
+    cfg.StrOpt('backup_strategy', default='MariaDBInnoBackupEx',
                help='Default strategy to perform backups.',
                deprecated_name='backup_strategy',
                deprecated_group='DEFAULT'),
@@ -1263,12 +1422,14 @@ mariadb_opts = [
                help='Maximum time (in seconds) to wait for a Guest to become '
                     'active.'),
     cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.mysql_impl',
+               default='trove.guestagent.strategies.backup.experimental'
+                       '.mariadb_impl',
                help='Namespace to load backup strategies from.',
                deprecated_name='backup_namespace',
                deprecated_group='DEFAULT'),
     cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.mysql_impl',
+               default='trove.guestagent.strategies.restore.experimental'
+                       '.mariadb_impl',
                help='Namespace to load restore strategies from.',
                deprecated_name='restore_namespace',
                deprecated_group='DEFAULT'),
@@ -1277,7 +1438,8 @@ mariadb_opts = [
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
     cfg.DictOpt('backup_incremental_strategy',
-                default={'InnoBackupEx': 'InnoBackupExIncremental'},
+                default={'MariaDBInnoBackupEx':
+                         'MariaDBInnoBackupExIncremental'},
                 help='Incremental Backup Runner based on the default '
                 'strategy. For strategies that do not implement an '
                 'incremental backup, the runner will use the default full '
@@ -1300,7 +1462,10 @@ mariadb_opts = [
                help='List of Guest Logs to expose for publishing.'),
     cfg.IntOpt('guest_log_long_query_time', default=1000,
                help='The time in milliseconds that a statement must take in '
-                    'in order to be logged in the slow_query log.'),
+                    'in order to be logged in the slow_query log.',
+               deprecated_for_removal=True,
+               deprecated_reason='Will be replaced by a configuration group '
+               'option: long_query_time'),
     cfg.BoolOpt('cluster_support', default=True,
                 help='Enable clusters to be created and managed.'),
     cfg.IntOpt('min_cluster_member_count', default=3,
@@ -1319,24 +1484,30 @@ mariadb_opts = [
                'galera_common.guestagent.GaleraCommonGuestAgentStrategy',
                help='Class that implements datastore-specific Guest Agent API '
                     'logic.'),
+    cfg.IntOpt('default_password_length',
+               default='${mysql.default_password_length}',
+               help='Character length of generated passwords.',
+               deprecated_name='default_password_length',
+               deprecated_group='DEFAULT'),
 ]
 
 # RPC version groups
 upgrade_levels = cfg.OptGroup(
     'upgrade_levels',
     title='RPC upgrade levels group for handling versions',
-    help='Contains the support version caps for each RPC API')
+    help='Contains the support version caps (Openstack Release) for '
+    'each RPC API')
 
 rpcapi_cap_opts = [
     cfg.StrOpt(
-        'taskmanager', default="icehouse",
+        'taskmanager', default='latest',
         help='Set a version cap for messages sent to taskmanager services'),
     cfg.StrOpt(
-        'guestagent', default="icehouse",
+        'guestagent', default='latest',
         help='Set a version cap for messages sent to guestagent services'),
     cfg.StrOpt(
-        'conductor', default="icehouse",
-        help='Set a version cap for messages sent to conductor services'),
+        'conductor', default='latest',
+        help='Set Openstack Release compatibility for conductor services'),
 ]
 
 CONF = cfg.CONF
@@ -1389,35 +1560,35 @@ def parse_args(argv, default_config_files=None):
              default_config_files=default_config_files)
 
 
-def get_ignored_dbs(manager=None):
+def get_ignored_dbs():
     try:
-        return get_configuration_property('ignore_dbs', manager=manager)
+        return get_configuration_property('ignore_dbs')
     except NoSuchOptError:
         return []
 
 
-def get_ignored_users(manager=None):
+def get_ignored_users():
     try:
-        return get_configuration_property('ignore_users', manager=manager)
+        return get_configuration_property('ignore_users')
     except NoSuchOptError:
         return []
 
 
-def get_configuration_property(property_name, manager=None):
+def get_configuration_property(property_name):
     """
     Get a configuration property.
     Try to get it from the datastore-specific section first.
     If it is not available, retrieve it from the DEFAULT section.
     """
 
-    # TODO(pmalik): Note that the unit and fake-integration tests
-    # do not define 'CONF.datastore_manager'. *MySQL* options will
-    # be loaded unless the caller passes a manager name explicitly.
-    #
-    # Once the tests are fixed this conditional expression should be removed
-    # and the proper value should always be either loaded from
-    # 'CONF.datastore_manager' or passed-in by the caller.
-    datastore_manager = manager or CONF.datastore_manager or 'mysql'
+    # Fake-integration tests do not define 'CONF.datastore_manager'.
+    # *MySQL* options will
+    # be loaded. This should never occur in a production environment.
+    datastore_manager = CONF.datastore_manager
+    if not datastore_manager:
+        datastore_manager = 'mysql'
+        LOG.warning(_("Manager name ('datastore_manager') not defined, "
+                      "using '%s' options instead.") % datastore_manager)
 
     try:
         return CONF.get(datastore_manager).get(property_name)
@@ -1428,23 +1599,21 @@ def get_configuration_property(property_name, manager=None):
 def set_api_config_defaults():
     """This method updates all configuration default values."""
 
-    # CORS Middleware Defaults
-    # TODO(krotscheck): Update with https://review.openstack.org/#/c/285368/
-    cfg.set_defaults(cors.CORS_OPTS,
-                     allow_headers=['X-Auth-Token',
-                                    'X-Identity-Status',
-                                    'X-Roles',
-                                    'X-Service-Catalog',
-                                    'X-User-Id',
-                                    'X-Tenant-Id',
-                                    'X-OpenStack-Request-ID'],
-                     expose_headers=['X-Auth-Token',
-                                     'X-Subject-Token',
-                                     'X-Service-Token',
-                                     'X-OpenStack-Request-ID'],
-                     allow_methods=['GET',
-                                    'PUT',
-                                    'POST',
-                                    'DELETE',
-                                    'PATCH']
-                     )
+    cors.set_defaults(
+        allow_headers=['X-Auth-Token',
+                       'X-Identity-Status',
+                       'X-Roles',
+                       'X-Service-Catalog',
+                       'X-User-Id',
+                       'X-Tenant-Id',
+                       'X-OpenStack-Request-ID'],
+        expose_headers=['X-Auth-Token',
+                        'X-Subject-Token',
+                        'X-Service-Token',
+                        'X-OpenStack-Request-ID'],
+        allow_methods=['GET',
+                       'PUT',
+                       'POST',
+                       'DELETE',
+                       'PATCH']
+    )
