@@ -144,7 +144,13 @@ class DatastoreVersionInactive(TroveError):
 class DatastoreDefaultDatastoreNotFound(TroveError):
 
     message = _("Please specify datastore. Default datastore "
-                "cannot be found.")
+                "'%(datastore)s' cannot be found.")
+
+
+class DatastoreDefaultDatastoreNotDefined(TroveError):
+
+    message = _("Please specify datastore. No default datastore "
+                "is defined.")
 
 
 class DatastoreDefaultVersionNotFound(TroveError):
@@ -230,9 +236,9 @@ class UnprocessableEntity(TroveError):
     message = _("Unable to process the contained request.")
 
 
-class UnauthorizedRequest(TroveError):
+class ConfigurationNotSupported(UnprocessableEntity):
 
-    message = _("Unauthorized request.")
+    message = _("Configuration groups not supported by the datastore.")
 
 
 class CannotResizeToSameSize(TroveError):
@@ -303,6 +309,11 @@ class Forbidden(TroveError):
     message = _("User does not have admin privileges.")
 
 
+class PolicyNotAuthorized(Forbidden):
+
+    message = _("Policy doesn't allow %(action)s to be performed.")
+
+
 class InvalidModelError(TroveError):
 
     message = _("The following values are invalid: %(errors)s.")
@@ -491,6 +502,11 @@ class InsufficientSpaceForReplica(TroveError):
                 "of data.")
 
 
+class InsufficientSpaceForBackup(TroveError):
+    message = _("The instance has only %(free)sG free while the estimated "
+                "backup size is %(backup_size)sG.")
+
+
 class ReplicaSourceDeleteForbidden(Forbidden):
     message = _("The replica source cannot be deleted without detaching the "
                 "replicas.")
@@ -522,6 +538,15 @@ class ModuleAccessForbidden(Forbidden):
                 "options. %(options)s")
 
 
+class ModuleInvalid(Forbidden):
+
+    message = _("The module is invalid: %(reason)s")
+
+
+class InstanceNotFound(NotFound):
+    message = _("Instance '%(instance)s' cannot be found.")
+
+
 class ClusterNotFound(NotFound):
     message = _("Cluster '%(cluster)s' cannot be found.")
 
@@ -606,3 +631,8 @@ class ImageNotFound(NotFound):
 class DatastoreVersionAlreadyExists(BadRequest):
 
     message = _("A datastore version with the name '%(name)s' already exists.")
+
+
+class LogAccessForbidden(Forbidden):
+
+    message = _("You must be admin to %(action)s log '%(log)s'.")
