@@ -138,7 +138,8 @@ class TroveBaseTraits(object):
         self.context = ctxt
         return self
 
-    def notify(self, event_type, publisher_id=CONF.host):
+    def notify(self, event_type, publisher_id=None):
+        publisher_id = publisher_id or CONF.host
         event_type = self.event_type_format % event_type
         event_payload = self.serialize(self.context)
         LOG.debug('Sending event: %(event_type)s, %(payload)s' %
@@ -294,6 +295,15 @@ class DBaaSAPINotification(object):
     '''
 
     event_type_format = 'dbaas.%s.%s'
+    notify_callback = None
+
+    @classmethod
+    def register_notify_callback(cls, callback):
+        """A callback registered here will be fired whenever
+        a notification is sent out. The callback should
+        take a notification object, and event_qualifier.
+        """
+        cls.notify_callback = callback
 
     @abc.abstractmethod
     def event_type(self):
@@ -323,7 +333,7 @@ class DBaaSAPINotification(object):
 
     def optional_error_traits(self):
         'Returns list of optional traits for error notification'
-        return []
+        return ['instance_id']
 
     def required_base_traits(self):
         return ['tenant_id', 'client_ip', 'server_ip', 'server_type',
@@ -337,6 +347,10 @@ class DBaaSAPINotification(object):
     def server_type(self, server_type):
         self.payload['server_type'] = server_type
 
+    @property
+    def request_id(self):
+        return self.payload['request_id']
+
     def __init__(self, context, **kwargs):
         self.context = context
         self.needs_end_notification = True
@@ -354,7 +368,7 @@ class DBaaSAPINotification(object):
                                 })
         elif 'request_id' not in kwargs:
             raise TroveError(_("Notification %s must include 'request'"
-                             " property") % self.__class__.__name__)
+                               " property") % self.__class__.__name__)
 
         self.payload.update(kwargs)
 
@@ -371,7 +385,7 @@ class DBaaSAPINotification(object):
                                 'keys': list(required_keys - provided_keys)})
         if 'server_type' not in self.payload:
             raise TroveError(_("Notification %s must include a"
-                             " 'server_type' for correct routing")
+                               " 'server_type' for correct routing")
                              % self.__class__.__name__)
 
     def _notify(self, event_qualifier, required_traits, optional_traits,
@@ -394,6 +408,8 @@ class DBaaSAPINotification(object):
         del context.notification
         notifier = rpc.get_notifier(service=self.payload['server_type'])
         notifier.info(context, qualified_event_type, self.payload)
+        if self.notify_callback:
+            self.notify_callback(event_qualifier)
 
     def notify_start(self, **kwargs):
         self._notify('start', self.required_start_traits(),
@@ -420,7 +436,7 @@ class DBaaSInstanceCreate(DBaaSAPINotification):
 
     def required_start_traits(self):
         return ['name', 'flavor_id', 'datastore', 'datastore_version',
-                'image_id', 'availability_zone']
+                'image_id', 'availability_zone', 'region_name']
 
     def optional_start_traits(self):
         return ['databases', 'users', 'volume_size', 'restore_point',
@@ -490,6 +506,16 @@ class DBaaSInstanceDelete(DBaaSAPINotification):
         return ['instance_id']
 
 
+class DBaaSInstanceResetStatus(DBaaSAPINotification):
+
+    def event_type(self):
+        return 'instance_reset_status'
+
+    @abc.abstractmethod
+    def required_start_traits(self):
+        return ['instance_id']
+
+
 class DBaaSInstanceDetach(DBaaSAPINotification):
 
     @abc.abstractmethod
@@ -523,6 +549,28 @@ class DBaaSInstanceDetachConfiguration(DBaaSAPINotification):
         return ['instance_id']
 
 
+class DBaaSClusterAttachConfiguration(DBaaSAPINotification):
+
+    @abc.abstractmethod
+    def event_type(self):
+        return 'cluster_attach_configuration'
+
+    @abc.abstractmethod
+    def required_start_traits(self):
+        return ['cluster_id', 'configuration_id']
+
+
+class DBaaSClusterDetachConfiguration(DBaaSAPINotification):
+
+    @abc.abstractmethod
+    def event_type(self):
+        return 'cluster_detach_configuration'
+
+    @abc.abstractmethod
+    def required_start_traits(self):
+        return ['cluster_id']
+
+
 class DBaaSClusterCreate(DBaaSAPINotification):
 
     @abc.abstractmethod
@@ -538,6 +586,26 @@ class DBaaSClusterCreate(DBaaSAPINotification):
         return ['cluster_id']
 
 
+class DBaaSClusterRestart(DBaaSAPINotification):
+
+    def event_type(self):
+        return 'cluster_restart'
+
+    def required_start_traits(self):
+        return ['cluster_id']
+
+
+class DBaaSClusterUpgrade(DBaaSAPINotification):
+
+    @abc.abstractmethod
+    def event_type(self):
+        return 'cluster_upgrade'
+
+    @abc.abstractmethod
+    def required_start_traits(self):
+        return ['cluster_id', 'datastore_version']
+
+
 class DBaaSClusterDelete(DBaaSAPINotification):
 
     @abc.abstractmethod
@@ -549,6 +617,17 @@ class DBaaSClusterDelete(DBaaSAPINotification):
         return ['cluster_id']
 
 
+class DBaaSClusterResetStatus(DBaaSAPINotification):
+
+    @abc.abstractmethod
+    def event_type(self):
+        return 'cluster_reset_status'
+
+    @abc.abstractmethod
+    def required_start_traits(self):
+        return ['cluster_id']
+
+
 class DBaaSClusterAddShard(DBaaSAPINotification):
 
     @abc.abstractmethod
@@ -741,3 +820,25 @@ class DBaaSConfigurationEdit(DBaaSAPINotification):
     @abc.abstractmethod
     def required_start_traits(self):
         return ['configuration_id']
+
+
+class DBaaSInstanceUpgrade(DBaaSAPINotification):
+
+    @abc.abstractmethod
+    def event_type(self):
+        return 'upgrade'
+
+    @abc.abstractmethod
+    def required_start_traits(self):
+        return ['instance_id', 'datastore_version_id']
+
+
+class DBaaSInstanceMigrate(DBaaSAPINotification):
+
+    @abc.abstractmethod
+    def event_type(self):
+        return 'migrate'
+
+    @abc.abstractmethod
+    def required_start_traits(self):
+        return ['host']
