@@ -11,7 +11,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from novaclient import exceptions as nova_exceptions
 from oslo_log import log as logging
 
 from trove.cluster import models
@@ -19,7 +18,7 @@ from trove.cluster.tasks import ClusterTasks
 from trove.cluster.views import ClusterView
 from trove.common import cfg
 from trove.common import exception
-from trove.common import remote
+from trove.common import server_group as srv_grp
 from trove.common.strategies.cluster import base
 from trove.common import utils
 from trove.extensions.mgmt.clusters.views import MgmtClusterView
@@ -70,7 +69,8 @@ class VerticaCluster(models.Cluster):
 
     @staticmethod
     def _create_instances(context, db_info, datastore, datastore_version,
-                          instances, new_cluster):
+                          instances, extended_properties, locality,
+                          new_cluster=True):
         vertica_conf = CONF.get(datastore_version.manager)
         num_instances = len(instances)
 
@@ -78,49 +78,34 @@ class VerticaCluster(models.Cluster):
         num_existing = len(existing)
 
         # Matching number of instances with configured cluster_member_count
-        if new_cluster \
-                and num_instances != vertica_conf.cluster_member_count:
+        if (new_cluster and
+                num_instances != vertica_conf.cluster_member_count):
             raise exception.ClusterNumInstancesNotSupported(
                 num_instances=vertica_conf.cluster_member_count)
 
-        # Checking flavors
-        flavor_ids = [instance['flavor_id'] for instance in instances]
-        if len(set(flavor_ids)) != 1:
-            raise exception.ClusterFlavorsNotEqual()
-        flavor_id = flavor_ids[0]
-        nova_client = remote.create_nova_client(context)
-        try:
-            flavor = nova_client.flavors.get(flavor_id)
-        except nova_exceptions.NotFound:
-            raise exception.FlavorNotFound(uuid=flavor_id)
-        deltas = {'instances': num_instances}
+        models.validate_instance_flavors(
+            context, instances, vertica_conf.volume_support,
+            vertica_conf.device_path)
 
-        # Checking volumes
-        volume_sizes = [instance['volume_size'] for instance in instances
-                        if instance.get('volume_size', None)]
-        volume_size = None
-        if vertica_conf.volume_support:
-            if len(volume_sizes) != num_instances:
-                raise exception.ClusterVolumeSizeRequired()
-            if len(set(volume_sizes)) != 1:
-                raise exception.ClusterVolumeSizesNotEqual()
-            volume_size = volume_sizes[0]
-            models.validate_volume_size(volume_size)
-            deltas['volumes'] = volume_size * num_instances
-        else:
-            if len(volume_sizes) > 0:
-                raise exception.VolumeNotSupported()
-            ephemeral_support = vertica_conf.device_path
-            if ephemeral_support and flavor.ephemeral == 0:
-                raise exception.LocalStorageNotSpecified(flavor=flavor_id)
+        req_volume_size = models.get_required_volume_size(
+            instances, vertica_conf.volume_support)
+        models.assert_homogeneous_cluster(instances)
+
+        deltas = {'instances': num_instances, 'volumes': req_volume_size}
 
         check_quotas(context.tenant, deltas)
 
+        flavor_id = instances[0]['flavor_id']
+        volume_size = instances[0].get('volume_size', None)
+
         nics = [instance.get('nics', None) for instance in instances]
 
         azs = [instance.get('availability_zone', None)
                for instance in instances]
 
+        regions = [instance.get('region_name', None)
+                   for instance in instances]
+
         # Creating member instances
         minstances = []
         for i in range(0, num_instances):
@@ -131,24 +116,25 @@ class VerticaCluster(models.Cluster):
             instance_name = "%s-member-%s" % (db_info.name,
                                               str(i + num_existing + 1))
             minstances.append(
-                inst_models.Instance.create(context, instance_name,
-                                            flavor_id,
-                                            datastore_version.image_id,
-                                            [], [], datastore,
-                                            datastore_version,
-                                            volume_size, None,
-                                            nics=nics[i],
-                                            availability_zone=azs[i],
-                                            configuration_id=None,
-                                            cluster_config=member_config)
+                inst_models.Instance.create(
+                    context, instance_name, flavor_id,
+                    datastore_version.image_id, [], [], datastore,
+                    datastore_version, volume_size, None,
+                    nics=nics[i], availability_zone=azs[i],
+                    configuration_id=None, cluster_config=member_config,
+                    modules=instances[i].get('modules'), locality=locality,
+                    region_name=regions[i])
             )
         return minstances
 
     @classmethod
     def create(cls, context, name, datastore, datastore_version,
-               instances, extended_properties):
+               instances, extended_properties, locality, configuration):
         LOG.debug("Initiating cluster creation.")
 
+        if configuration:
+            raise exception.ConfigurationNotSupported()
+
         vertica_conf = CONF.get(datastore_version.manager)
         num_instances = len(instances)
 
@@ -163,7 +149,8 @@ class VerticaCluster(models.Cluster):
             task_status=ClusterTasks.BUILDING_INITIAL)
 
         cls._create_instances(context, db_info, datastore, datastore_version,
-                              instances, new_cluster=True)
+                              instances, extended_properties, locality,
+                              new_cluster=True)
         # Calling taskmanager to further proceed for cluster-configuration
         task_api.load(context, datastore_version.manager).create_cluster(
             db_info.id)
@@ -196,8 +183,10 @@ class VerticaCluster(models.Cluster):
 
         db_info.update(task_status=ClusterTasks.GROWING_CLUSTER)
 
+        locality = srv_grp.ServerGroup.convert_to_hint(self.server_group)
         new_instances = self._create_instances(context, db_info, datastore,
                                                datastore_version, instances,
+                                               None, locality,
                                                new_cluster=False)
 
         task_api.load(context, datastore_version.manager).grow_cluster(
