@@ -25,6 +25,7 @@ import jsonschema
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_service import service
+from oslo_utils import encodeutils
 import paste.urlmap
 import webob
 import webob.dec
@@ -321,6 +322,8 @@ class Controller(object):
             exception.BackupTooLarge,
             exception.ModuleAccessForbidden,
             exception.ModuleAppliedToInstance,
+            exception.PolicyNotAuthorized,
+            exception.LogAccessForbidden,
         ],
         webob.exc.HTTPBadRequest: [
             exception.InvalidModelError,
@@ -547,7 +550,8 @@ class ContextMiddleware(base_wsgi.Middleware):
                                           is_admin=is_admin,
                                           limit=limits.get('limit'),
                                           marker=limits.get('marker'),
-                                          service_catalog=service_catalog)
+                                          service_catalog=service_catalog,
+                                          roles=roles)
         request.environ[CONTEXT_KEY] = context
 
     @classmethod
@@ -575,7 +579,8 @@ class FaultWrapper(base_wsgi.Middleware):
                 return resp
             return resp
         except Exception as ex:
-            LOG.exception(_("Caught error: %s."), unicode(ex))
+            LOG.exception(_("Caught error: %s."),
+                          encodeutils.exception_to_unicode(ex))
             exc = webob.exc.HTTPInternalServerError()
             return Fault(exc)
 
@@ -657,4 +662,4 @@ class JSONDictSerializer(DictSerializer):
     """Default JSON request body serialization."""
 
     def default(self, data):
-        return jsonutils.dumps(data)
+        return jsonutils.dump_as_bytes(data)
