@@ -16,12 +16,17 @@
 from datetime import datetime
 
 from oslo_log import log as logging
+import six
 
+from trove.cluster import models as cluster_models
 import trove.common.apischema as apischema
 from trove.common import cfg
 from trove.common import exception
 from trove.common.i18n import _
+from trove.common import notification
+from trove.common.notification import StartNotification, EndNotification
 from trove.common import pagination
+from trove.common import policy
 from trove.common import wsgi
 from trove.configuration import models
 from trove.configuration.models import DBConfigurationParameter
@@ -38,9 +43,16 @@ class ConfigurationsController(wsgi.Controller):
 
     schemas = apischema.configuration
 
+    @classmethod
+    def authorize_config_action(cls, context, config_rule_name, config):
+        policy.authorize_on_target(
+            context, 'configuration:%s' % config_rule_name,
+            {'tenant': config.tenant_id})
+
     def index(self, req, tenant_id):
         context = req.environ[wsgi.CONTEXT_KEY]
         configs, marker = models.Configurations.load(context)
+        policy.authorize_on_tenant(context, 'configuration:index')
         view = views.ConfigurationsView(configs)
         paged = pagination.SimplePaginatedDataView(req.url, 'configurations',
                                                    view, marker)
@@ -51,6 +63,7 @@ class ConfigurationsController(wsgi.Controller):
                   % {"tenant": tenant_id, "id": id})
         context = req.environ[wsgi.CONTEXT_KEY]
         configuration = models.Configuration.load(context, id)
+        self.authorize_config_action(context, 'show', configuration)
         configuration_items = models.Configuration.load_items(context, id)
 
         configuration.instance_count = instances_models.DBInstance.find_all(
@@ -65,6 +78,7 @@ class ConfigurationsController(wsgi.Controller):
     def instances(self, req, tenant_id, id):
         context = req.environ[wsgi.CONTEXT_KEY]
         configuration = models.Configuration.load(context, id)
+        self.authorize_config_action(context, 'instances', configuration)
         instances = instances_models.DBInstance.find_all(
             tenant_id=context.tenant,
             configuration_id=configuration.id,
@@ -85,6 +99,10 @@ class ConfigurationsController(wsgi.Controller):
         LOG.debug("req : '%s'\n\n" % req)
         LOG.debug("body : '%s'\n\n" % req)
 
+        context = req.environ[wsgi.CONTEXT_KEY]
+        policy.authorize_on_tenant(context, 'configuration:create')
+        context.notification = notification.DBaaSConfigurationCreate(
+            context, request=req)
         name = body['configuration']['name']
         description = body['configuration'].get('description')
         values = body['configuration']['values']
@@ -97,25 +115,30 @@ class ConfigurationsController(wsgi.Controller):
         datastore, datastore_version = (
             ds_models.get_datastore_version(**datastore_args))
 
-        configItems = []
-        if values:
-            # validate that the values passed in are permitted by the operator.
-            ConfigurationsController._validate_configuration(
-                body['configuration']['values'],
-                datastore_version,
-                models.DatastoreConfigurationParameters.load_parameters(
-                    datastore_version.id))
+        with StartNotification(context, name=name, datastore=datastore.name,
+                               datastore_version=datastore_version.name):
+            configItems = []
+            if values:
+                # validate that the values passed in are permitted by the
+                # operator.
+                ConfigurationsController._validate_configuration(
+                    body['configuration']['values'],
+                    datastore_version,
+                    models.DatastoreConfigurationParameters.load_parameters(
+                        datastore_version.id))
 
-            for k, v in values.iteritems():
-                configItems.append(DBConfigurationParameter(
-                    configuration_key=k,
-                    configuration_value=v))
+                for k, v in values.items():
+                    configItems.append(DBConfigurationParameter(
+                        configuration_key=k,
+                        configuration_value=v))
+
+            cfg_group = models.Configuration.create(name, description,
+                                                    tenant_id, datastore.id,
+                                                    datastore_version.id)
+            with EndNotification(context, configuration_id=cfg_group.id):
+                cfg_group_items = models.Configuration.create_items(
+                    cfg_group.id, values)
 
-        cfg_group = models.Configuration.create(name, description, tenant_id,
-                                                datastore.id,
-                                                datastore_version.id)
-        cfg_group_items = models.Configuration.create_items(cfg_group.id,
-                                                            values)
         view_data = views.DetailedConfigurationView(cfg_group,
                                                     cfg_group_items)
         return wsgi.Result(view_data.data(), 200)
@@ -127,13 +150,17 @@ class ConfigurationsController(wsgi.Controller):
 
         context = req.environ[wsgi.CONTEXT_KEY]
         group = models.Configuration.load(context, id)
-        instances = instances_models.DBInstance.find_all(
-            tenant_id=context.tenant,
-            configuration_id=id,
-            deleted=False).all()
-        if instances:
-            raise exception.InstanceAssignedToConfiguration()
-        models.Configuration.delete(context, group)
+        self.authorize_config_action(context, 'delete', group)
+        context.notification = notification.DBaaSConfigurationDelete(
+            context, request=req)
+        with StartNotification(context, configuration_id=id):
+            instances = instances_models.DBInstance.find_all(
+                tenant_id=context.tenant,
+                configuration_id=id,
+                deleted=False).all()
+            if instances:
+                raise exception.InstanceAssignedToConfiguration()
+            models.Configuration.delete(context, group)
         return wsgi.Result(None, 202)
 
     def update(self, req, body, tenant_id, id):
@@ -143,6 +170,15 @@ class ConfigurationsController(wsgi.Controller):
 
         context = req.environ[wsgi.CONTEXT_KEY]
         group = models.Configuration.load(context, id)
+        # Note that changing the configuration group will also
+        # indirectly affect all the instances which attach it.
+        #
+        # The Trove instance itself won't be changed (the same group is still
+        # attached) but the configuration values will.
+        #
+        # The operator needs to keep this in mind when defining the related
+        # policies.
+        self.authorize_config_action(context, 'update', group)
 
         # if name/description are provided in the request body, update the
         # model with these values as well.
@@ -152,38 +188,67 @@ class ConfigurationsController(wsgi.Controller):
         if 'description' in body['configuration']:
             group.description = body['configuration']['description']
 
-        items = self._configuration_items_list(group, body['configuration'])
-        deleted_at = datetime.utcnow()
-        models.Configuration.remove_all_items(context, group.id, deleted_at)
-        models.Configuration.save(group, items)
-        self._refresh_on_all_instances(context, id)
+        context.notification = notification.DBaaSConfigurationUpdate(
+            context, request=req)
+        with StartNotification(context, configuration_id=id,
+                               name=group.name, description=group.description):
+            items = self._configuration_items_list(group,
+                                                   body['configuration'])
+            deleted_at = datetime.utcnow()
+            models.Configuration.remove_all_items(context, group.id,
+                                                  deleted_at)
+            models.Configuration.save(group, items)
+            self._refresh_on_all_instances(context, id)
+            self._refresh_on_all_clusters(context, id)
+
         return wsgi.Result(None, 202)
 
     def edit(self, req, body, tenant_id, id):
         context = req.environ[wsgi.CONTEXT_KEY]
         group = models.Configuration.load(context, id)
-        items = self._configuration_items_list(group, body['configuration'])
-        models.Configuration.save(group, items)
-        self._refresh_on_all_instances(context, id)
+        self.authorize_config_action(context, 'edit', group)
+        context.notification = notification.DBaaSConfigurationEdit(
+            context, request=req)
+        with StartNotification(context, configuration_id=id):
+            items = self._configuration_items_list(group,
+                                                   body['configuration'])
+            models.Configuration.save(group, items)
+            self._refresh_on_all_instances(context, id)
+            self._refresh_on_all_clusters(context, id)
 
     def _refresh_on_all_instances(self, context, configuration_id):
-        """Refresh a configuration group on all its instances.
+        """Refresh a configuration group on all single instances.
         """
-        dbinstances = instances_models.DBInstance.find_all(
+        LOG.debug("Re-applying configuration group '%s' to all instances."
+                  % configuration_id)
+        single_instances = instances_models.DBInstance.find_all(
+            tenant_id=context.tenant,
+            configuration_id=configuration_id,
+            cluster_id=None,
+            deleted=False).all()
+
+        config = models.Configuration(context, configuration_id)
+        for dbinstance in single_instances:
+            LOG.debug("Re-applying configuration to instance: %s"
+                      % dbinstance.id)
+            instance = instances_models.Instance.load(context, dbinstance.id)
+            instance.update_configuration(config)
+
+    def _refresh_on_all_clusters(self, context, configuration_id):
+        """Refresh a configuration group on all clusters.
+        """
+        LOG.debug("Re-applying configuration group '%s' to all clusters."
+                  % configuration_id)
+        clusters = cluster_models.DBCluster.find_all(
             tenant_id=context.tenant,
             configuration_id=configuration_id,
             deleted=False).all()
 
-        LOG.debug(
-            "All instances with configuration group '%s' on tenant '%s': %s"
-            % (configuration_id, context.tenant, dbinstances))
-
-        config = models.Configuration(context, configuration_id)
-        for dbinstance in dbinstances:
-            LOG.debug("Applying configuration group '%s' to instance: %s"
-                      % (configuration_id, dbinstance.id))
-            instance = instances_models.Instance.load(context, dbinstance.id)
-            instance.update_overrides(config)
+        for dbcluster in clusters:
+            LOG.debug("Re-applying configuration to cluster: %s"
+                      % dbcluster.id)
+            cluster = cluster_models.Cluster.load(context, dbcluster.id)
+            cluster.configuration_attach(configuration_id)
 
     def _configuration_items_list(self, group, configuration):
         ds_version_id = group.datastore_version_id
@@ -196,7 +261,7 @@ class ConfigurationsController(wsgi.Controller):
                 ds_version,
                 models.DatastoreConfigurationParameters.load_parameters(
                     ds_version.id))
-            for k, v in configuration['values'].iteritems():
+            for k, v in configuration['values'].items():
                 items.append(DBConfigurationParameter(
                     configuration_id=group.id,
                     configuration_key=k,
@@ -221,7 +286,7 @@ class ConfigurationsController(wsgi.Controller):
                     "datastore: %(name)s %(version)s") % output
             raise exception.UnprocessableEntity(message=msg)
 
-        for k, v in values.iteritems():
+        for k, v in values.items():
             key = k.lower()
             # parameter name validation
             if key not in rules_lookup:
@@ -246,7 +311,7 @@ class ConfigurationsController(wsgi.Controller):
                 raise exception.UnprocessableEntity(message=msg)
 
             # integer min/max checking
-            if isinstance(v, (int, long)) and not isinstance(v, bool):
+            if isinstance(v, six.integer_types) and not isinstance(v, bool):
                 if rule.min_size is not None:
                     try:
                         min_value = int(rule.min_size)
@@ -284,9 +349,11 @@ class ConfigurationsController(wsgi.Controller):
         if value_type == "boolean":
             return bool
         elif value_type == "string":
-            return basestring
+            return six.string_types
         elif value_type == "integer":
-            return (int, long)
+            return six.integer_types
+        elif value_type == "float":
+            return float
         else:
             raise exception.TroveError(_(
                 "Invalid or unsupported type defined in the "
@@ -303,7 +370,18 @@ class ConfigurationsController(wsgi.Controller):
 
 class ParametersController(wsgi.Controller):
 
+    @classmethod
+    def authorize_request(cls, req, rule_name):
+        """Parameters (configuration templates) bind to a datastore.
+        Datastores are not owned by any particular tenant so we only check
+        the current tenant is allowed to perform the action.
+        """
+        context = req.environ[wsgi.CONTEXT_KEY]
+        policy.authorize_on_tenant(context, 'configuration-parameter:%s'
+                                   % rule_name)
+
     def index(self, req, tenant_id, datastore, id):
+        self.authorize_request(req, 'index')
         ds, ds_version = ds_models.get_datastore_version(
             type=datastore, version=id)
         rules = models.DatastoreConfigurationParameters.load_parameters(
@@ -312,6 +390,7 @@ class ParametersController(wsgi.Controller):
                            200)
 
     def show(self, req, tenant_id, datastore, id, name):
+        self.authorize_request(req, 'show')
         ds, ds_version = ds_models.get_datastore_version(
             type=datastore, version=id)
         rule = models.DatastoreConfigurationParameters.load_parameter_by_name(
@@ -319,6 +398,7 @@ class ParametersController(wsgi.Controller):
         return wsgi.Result(views.ConfigurationParameterView(rule).data(), 200)
 
     def index_by_version(self, req, tenant_id, version):
+        self.authorize_request(req, 'index_by_version')
         ds_version = ds_models.DatastoreVersion.load_by_uuid(version)
         rules = models.DatastoreConfigurationParameters.load_parameters(
             ds_version.id)
@@ -326,6 +406,7 @@ class ParametersController(wsgi.Controller):
                            200)
 
     def show_by_version(self, req, tenant_id, version, name):
+        self.authorize_request(req, 'show_by_version')
         ds_models.DatastoreVersion.load_by_uuid(version)
         rule = models.DatastoreConfigurationParameters.load_parameter_by_name(
             version, name)
