@@ -33,7 +33,7 @@ CONF = cfg.CONF
 def load_mgmt_instances(context, deleted=None, client=None,
                         include_clustered=None):
     if not client:
-        client = remote.create_nova_client(context)
+        client = remote.create_nova_client(context, CONF.os_region_name)
     try:
         mgmt_servers = client.rdservers.list()
     except AttributeError:
@@ -56,7 +56,7 @@ def load_mgmt_instance(cls, context, id, include_deleted):
     try:
         instance = load_instance(cls, context, id, needs_server=True,
                                  include_deleted=include_deleted)
-        client = remote.create_nova_client(context)
+        client = remote.create_nova_client(context, CONF.os_region_name)
         try:
             server = client.rdservers.get(instance.server_id)
         except AttributeError:
@@ -151,7 +151,7 @@ class MgmtInstances(imodels.Instances):
             return SimpleMgmtInstance(context, db, server, status)
 
         if context is None:
-            raise TypeError("Argument context not defined.")
+            raise TypeError(_("Argument context not defined."))
         find_server = imodels.create_server_list_matcher(servers)
         instances = imodels.Instances._load_servers_status(load_instance,
                                                            context,
@@ -169,7 +169,7 @@ def _load_servers(instances, find_server):
             server = find_server(db.id, db.compute_instance_id)
             instance.server = server
         except Exception as ex:
-            LOG.error(ex)
+            LOG.exception(ex)
     return instances
 
 
