@@ -21,6 +21,7 @@ import webob.exc
 
 import trove.common.apischema as apischema
 from trove.common import cfg
+from trove.common.db.mysql import models as guest_models
 from trove.common import exception
 from trove.common.i18n import _
 from trove.common import notification
@@ -29,12 +30,12 @@ from trove.common import pagination
 from trove.common.utils import correct_id_with_req
 from trove.common import wsgi
 from trove.extensions.common.service import DefaultRootController
+from trove.extensions.common.service import ExtensionController
 from trove.extensions.mysql.common import populate_users
 from trove.extensions.mysql.common import populate_validated_databases
 from trove.extensions.mysql.common import unquote_user_host
 from trove.extensions.mysql import models
 from trove.extensions.mysql import views
-from trove.guestagent.db import models as guest_models
 
 
 LOG = logging.getLogger(__name__)
@@ -42,7 +43,7 @@ import_class = importutils.import_class
 CONF = cfg.CONF
 
 
-class UserController(wsgi.Controller):
+class UserController(ExtensionController):
     """Controller for instance functionality."""
     schemas = apischema.user
 
@@ -60,6 +61,7 @@ class UserController(wsgi.Controller):
                    "req : '%(req)s'\n\n") %
                  {"id": instance_id, "req": req})
         context = req.environ[wsgi.CONTEXT_KEY]
+        self.authorize_target_action(context, 'user:index', instance_id)
         users, next_marker = models.Users.load(context, instance_id)
         view = views.UsersView(users)
         paged = pagination.SimplePaginatedDataView(req.url, 'users', view,
@@ -75,6 +77,7 @@ class UserController(wsgi.Controller):
                   "req": strutils.mask_password(req),
                   "body": strutils.mask_password(body)})
         context = req.environ[wsgi.CONTEXT_KEY]
+        self.authorize_target_action(context, 'user:create', instance_id)
         context.notification = notification.DBaaSUserCreate(context,
                                                             request=req)
         users = body['users']
@@ -85,7 +88,8 @@ class UserController(wsgi.Controller):
                 model_users = populate_users(users)
                 models.User.create(context, instance_id, model_users)
             except (ValueError, AttributeError) as e:
-                raise exception.BadRequest(msg=str(e))
+                raise exception.BadRequest(_("User create error: %(e)s")
+                                           % {'e': e})
         return wsgi.Result(None, 202)
 
     def delete(self, req, tenant_id, instance_id, id):
@@ -93,23 +97,24 @@ class UserController(wsgi.Controller):
                    "req : '%(req)s'\n\n") %
                  {"id": instance_id, "req": req})
         context = req.environ[wsgi.CONTEXT_KEY]
+        self.authorize_target_action(context, 'user:delete', instance_id)
         id = correct_id_with_req(id, req)
         username, host = unquote_user_host(id)
+        user = None
         context.notification = notification.DBaaSUserDelete(context,
                                                             request=req)
         with StartNotification(context, instance_id=instance_id,
                                username=username):
-            user = None
             try:
-                user = guest_models.MySQLUser()
-                user.name = username
-                user.host = host
+                user = guest_models.MySQLUser(name=username,
+                                              host=host)
                 found_user = models.User.load(context, instance_id, username,
                                               host)
                 if not found_user:
                     user = None
             except (ValueError, AttributeError) as e:
-                raise exception.BadRequest(msg=str(e))
+                raise exception.BadRequest(_("User delete error: %(e)s")
+                                           % {'e': e})
             if not user:
                 raise exception.UserNotFound(uuid=id)
             models.User.delete(context, instance_id, user.serialize())
@@ -121,13 +126,15 @@ class UserController(wsgi.Controller):
                    "req : '%(req)s'\n\n") %
                  {"id": instance_id, "req": req})
         context = req.environ[wsgi.CONTEXT_KEY]
+        self.authorize_target_action(context, 'user:show', instance_id)
         id = correct_id_with_req(id, req)
         username, host = unquote_user_host(id)
         user = None
         try:
             user = models.User.load(context, instance_id, username, host)
         except (ValueError, AttributeError) as e:
-            raise exception.BadRequest(msg=str(e))
+            raise exception.BadRequest(_("User show error: %(e)s")
+                                       % {'e': e})
         if not user:
             raise exception.UserNotFound(uuid=id)
         view = views.UserView(user)
@@ -139,6 +146,7 @@ class UserController(wsgi.Controller):
                    "req : '%(req)s'\n\n") %
                  {"id": instance_id, "req": strutils.mask_password(req)})
         context = req.environ[wsgi.CONTEXT_KEY]
+        self.authorize_target_action(context, 'user:update', instance_id)
         id = correct_id_with_req(id, req)
         username, hostname = unquote_user_host(id)
         user = None
@@ -151,14 +159,16 @@ class UserController(wsgi.Controller):
                 user = models.User.load(context, instance_id, username,
                                         hostname)
             except (ValueError, AttributeError) as e:
-                raise exception.BadRequest(msg=str(e))
+                raise exception.BadRequest(_("Error loading user: %(e)s")
+                                           % {'e': e})
             if not user:
                 raise exception.UserNotFound(uuid=id)
             try:
                 models.User.update_attributes(context, instance_id, username,
                                               hostname, user_attrs)
             except (ValueError, AttributeError) as e:
-                raise exception.BadRequest(msg=str(e))
+                raise exception.BadRequest(_("User update error: %(e)s")
+                                           % {'e': e})
         return wsgi.Result(None, 202)
 
     def update_all(self, req, body, tenant_id, instance_id):
@@ -167,19 +177,19 @@ class UserController(wsgi.Controller):
                    "req : '%(req)s'\n\n") %
                  {"id": instance_id, "req": strutils.mask_password(req)})
         context = req.environ[wsgi.CONTEXT_KEY]
+        self.authorize_target_action(context, 'user:update_all', instance_id)
         context.notification = notification.DBaaSUserChangePassword(
             context, request=req)
         users = body['users']
+        model_users = []
         with StartNotification(context, instance_id=instance_id,
                                username=",".join([user['name']
                                                   for user in users])):
-            model_users = []
             for user in users:
                 try:
-                    mu = guest_models.MySQLUser()
-                    mu.name = user['name']
-                    mu.host = user.get('host')
-                    mu.password = user['password']
+                    mu = guest_models.MySQLUser(name=user['name'],
+                                                host=user.get('host'),
+                                                password=user['password'])
                     found_user = models.User.load(context, instance_id,
                                                   mu.name, mu.host)
                     if not found_user:
@@ -189,12 +199,18 @@ class UserController(wsgi.Controller):
                         raise exception.UserNotFound(uuid=user_and_host)
                     model_users.append(mu)
                 except (ValueError, AttributeError) as e:
-                    raise exception.BadRequest(msg=str(e))
-            models.User.change_password(context, instance_id, model_users)
+                    raise exception.BadRequest(_("Error loading user: %(e)s")
+                                               % {'e': e})
+            try:
+                models.User.change_password(context, instance_id, model_users)
+            except (ValueError, AttributeError) as e:
+                raise exception.BadRequest(_("User password update error: "
+                                             "%(e)s")
+                                           % {'e': e})
         return wsgi.Result(None, 202)
 
 
-class UserAccessController(wsgi.Controller):
+class UserAccessController(ExtensionController):
     """Controller for adding and removing database access for a user."""
     schemas = apischema.user
 
@@ -210,7 +226,8 @@ class UserAccessController(wsgi.Controller):
         try:
             user = models.User.load(context, instance_id, username, hostname)
         except (ValueError, AttributeError) as e:
-            raise exception.BadRequest(msg=str(e))
+            raise exception.BadRequest(_("Error loading user: %(e)s")
+                                       % {'e': e})
         if not user:
             raise exception.UserNotFound(uuid=user_id)
         return user
@@ -222,6 +239,8 @@ class UserAccessController(wsgi.Controller):
                  {"id": instance_id, "req": req})
 
         context = req.environ[wsgi.CONTEXT_KEY]
+        self.authorize_target_action(
+            context, 'user_access:index', instance_id)
         # Make sure this user exists.
         user_id = correct_id_with_req(user_id, req)
         user = self._get_user(context, instance_id, user_id)
@@ -239,6 +258,8 @@ class UserAccessController(wsgi.Controller):
                    "req : '%(req)s'\n\n") %
                  {"id": instance_id, "req": req})
         context = req.environ[wsgi.CONTEXT_KEY]
+        self.authorize_target_action(
+            context, 'user_access:update', instance_id)
         context.notification = notification.DBaaSUserGrant(
             context, request=req)
         user_id = correct_id_with_req(user_id, req)
@@ -260,6 +281,8 @@ class UserAccessController(wsgi.Controller):
                    "req : '%(req)s'\n\n") %
                  {"id": instance_id, "req": req})
         context = req.environ[wsgi.CONTEXT_KEY]
+        self.authorize_target_action(
+            context, 'user_access:delete', instance_id)
         context.notification = notification.DBaaSUserRevoke(
             context, request=req)
         user_id = correct_id_with_req(user_id, req)
@@ -278,7 +301,7 @@ class UserAccessController(wsgi.Controller):
         return wsgi.Result(None, 202)
 
 
-class SchemaController(wsgi.Controller):
+class SchemaController(ExtensionController):
     """Controller for instance functionality."""
     schemas = apischema.dbschema
 
@@ -289,6 +312,8 @@ class SchemaController(wsgi.Controller):
                  {"id": instance_id, "req": req})
 
         context = req.environ[wsgi.CONTEXT_KEY]
+        self.authorize_target_action(
+            context, 'database:index', instance_id)
         schemas, next_marker = models.Schemas.load(context, instance_id)
         view = views.SchemasView(schemas)
         paged = pagination.SimplePaginatedDataView(req.url, 'databases', view,
@@ -305,14 +330,20 @@ class SchemaController(wsgi.Controller):
                   "body": body})
 
         context = req.environ[wsgi.CONTEXT_KEY]
+        self.authorize_target_action(
+            context, 'database:create', instance_id)
         schemas = body['databases']
         context.notification = notification.DBaaSDatabaseCreate(context,
                                                                 request=req)
         with StartNotification(context, instance_id=instance_id,
                                dbname=".".join([db['name']
                                                 for db in schemas])):
-            model_schemas = populate_validated_databases(schemas)
-            models.Schema.create(context, instance_id, model_schemas)
+            try:
+                model_schemas = populate_validated_databases(schemas)
+                models.Schema.create(context, instance_id, model_schemas)
+            except (ValueError, AttributeError) as e:
+                raise exception.BadRequest(_("Database create error: %(e)s")
+                                           % {'e': e})
         return wsgi.Result(None, 202)
 
     def delete(self, req, tenant_id, instance_id, id):
@@ -320,25 +351,33 @@ class SchemaController(wsgi.Controller):
                    "req : '%(req)s'\n\n") %
                  {"id": instance_id, "req": req})
         context = req.environ[wsgi.CONTEXT_KEY]
+        self.authorize_target_action(
+            context, 'database:delete', instance_id)
         context.notification = notification.DBaaSDatabaseDelete(
             context, request=req)
         with StartNotification(context, instance_id=instance_id, dbname=id):
             try:
-                schema = guest_models.ValidatedMySQLDatabase()
-                schema.name = id
+                schema = guest_models.MySQLSchema(name=id)
+                schema.check_delete()
+                if not models.Schemas.find(context, instance_id, id):
+                    raise exception.DatabaseNotFound(uuid=id)
                 models.Schema.delete(context, instance_id, schema.serialize())
             except (ValueError, AttributeError) as e:
-                raise exception.BadRequest(msg=str(e))
+                raise exception.BadRequest(_("Database delete error: %(e)s")
+                                           % {'e': e})
         return wsgi.Result(None, 202)
 
     def show(self, req, tenant_id, instance_id, id):
+        context = req.environ[wsgi.CONTEXT_KEY]
+        self.authorize_target_action(
+            context, 'database:show', instance_id)
         raise webob.exc.HTTPNotImplemented()
 
 
 class MySQLRootController(DefaultRootController):
 
     def _find_root_user(self, context, instance_id):
-        user = guest_models.MySQLRootUser()
+        user = guest_models.MySQLUser.root()
         return models.User.load(context, instance_id,
                                 user.name, user.host,
                                 root_user=True)
