@@ -54,8 +54,10 @@ class BackupAgent(object):
         try:
             runner = get_restore_strategy(backup_type, RESTORE_NAMESPACE)
         except ImportError:
-            raise UnknownBackupType("Unknown Backup type: %s in namespace %s"
-                                    % (backup_type, RESTORE_NAMESPACE))
+            raise UnknownBackupType(_("Unknown Backup type: %(type)s in "
+                                      "namespace %(ns)s")
+                                    % {"type": backup_type,
+                                       "ns": RESTORE_NAMESPACE})
         return runner
 
     def stream_backup_to_storage(self, context, backup_info, runner, storage,
@@ -76,13 +78,17 @@ class BackupAgent(object):
                                 **backup_state)
         LOG.debug("Updated state for %s to %s.", backup_id, backup_state)
 
-        with runner(filename=backup_id, extra_opts=extra_opts,
-                    **parent_metadata) as bkup:
-            try:
+        try:
+            with runner(filename=backup_id, extra_opts=extra_opts,
+                        **parent_metadata) as bkup:
                 LOG.debug("Starting backup %s.", backup_id)
+                meta = {}
+                meta['datastore'] = backup_info['datastore']
+                meta['datastore_version'] = backup_info['datastore_version']
                 success, note, checksum, location = storage.save(
                     bkup.manifest,
-                    bkup)
+                    bkup,
+                    metadata=meta)
 
                 backup_state.update({
                     'checksum': checksum,
@@ -102,28 +108,22 @@ class BackupAgent(object):
                 if not success:
                     raise BackupError(note)
 
-                meta = bkup.metadata()
-                meta['datastore'] = backup_info['datastore']
-                meta['datastore_version'] = backup_info[
-                    'datastore_version']
-                storage.save_metadata(location, meta)
-
                 backup_state.update({'state': BackupState.COMPLETED})
 
                 return meta
 
-            except Exception:
-                LOG.exception(
-                    _("Error saving backup: %(backup_id)s.") % backup_state)
-                backup_state.update({'state': BackupState.FAILED})
-                raise
-            finally:
-                LOG.info(_("Completed backup %(backup_id)s.") % backup_state)
-                conductor.update_backup(CONF.guest_id,
-                                        sent=timeutils.float_utcnow(),
-                                        **backup_state)
-                LOG.debug("Updated state for %s to %s.",
-                          backup_id, backup_state)
+        except Exception:
+            LOG.exception(
+                _("Error saving backup: %(backup_id)s.") % backup_state)
+            backup_state.update({'state': BackupState.FAILED})
+            raise
+        finally:
+            LOG.info(_("Completed backup %(backup_id)s.") % backup_state)
+            conductor.update_backup(CONF.guest_id,
+                                    sent=timeutils.float_utcnow(),
+                                    **backup_state)
+            LOG.debug("Updated state for %s to %s.",
+                      backup_id, backup_state)
 
     def execute_backup(self, context, backup_info,
                        runner=RUNNER, extra_opts=EXTRA_OPTS,
