@@ -26,13 +26,14 @@ from oslo_log import log as logging
 from oslo_utils import netutils
 
 from trove.common import cfg
+from trove.common.db.cassandra import models
 from trove.common import exception
 from trove.common.i18n import _
 from trove.common import instance as rd_instance
-from trove.common import pagination
 from trove.common.stream_codecs import IniCodec
 from trove.common.stream_codecs import PropertiesCodec
 from trove.common.stream_codecs import SafeYamlCodec
+from trove.common.stream_codecs import XmlCodec
 from trove.common import utils
 from trove.guestagent.common.configuration import ConfigurationManager
 from trove.guestagent.common.configuration import OneFileOverrideStrategy
@@ -40,7 +41,6 @@ from trove.guestagent.common import guestagent_utils
 from trove.guestagent.common import operating_system
 from trove.guestagent.common.operating_system import FileMode
 from trove.guestagent.datastore import service
-from trove.guestagent.db import models
 from trove.guestagent import pkg
 
 
@@ -63,6 +63,7 @@ class CassandraApp(object):
 
     CASSANDRA_CONF_FILE = "cassandra.yaml"
     CASSANDRA_TOPOLOGY_FILE = 'cassandra-rackdc.properties'
+    CASSANDRA_LOGBACK_FILE = "logback.xml"
 
     _TOPOLOGY_CODEC = PropertiesCodec(
         delimiter='=', unpack_singletons=True, string_mappings={
@@ -83,6 +84,14 @@ class CassandraApp(object):
             SafeYamlCodec(default_flow_style=False), requires_root=True,
             override_strategy=OneFileOverrideStrategy(revision_dir))
 
+        lb_revision_dir = guestagent_utils.build_file_path(
+            os.path.dirname(self.cassandra_logback), 'logback-overrides')
+        self.logback_conf_manager = ConfigurationManager(
+            self.cassandra_logback,
+            self.cassandra_owner, self.cassandra_owner,
+            XmlCodec(), requires_root=True,
+            override_strategy=OneFileOverrideStrategy(lb_revision_dir))
+
     @property
     def service_candidates(self):
         return ['cassandra']
@@ -119,8 +128,18 @@ class CassandraApp(object):
         return "/var/lib/cassandra"
 
     @property
-    def default_superuser_name(self):
-        return "cassandra"
+    def cassandra_system_log_file(self):
+        return guestagent_utils.build_file_path(
+            self.cassandra_log_dir, 'system', 'log')
+
+    @property
+    def cassandra_log_dir(self):
+        return "/var/log/cassandra"
+
+    @property
+    def cassandra_logback(self):
+        return guestagent_utils.build_file_path(self.cassandra_conf_dir,
+                                                self.CASSANDRA_LOGBACK_FILE)
 
     @property
     def default_superuser_password(self):
@@ -288,7 +307,8 @@ class CassandraApp(object):
             CassandraAdmin(update_user).alter_user_password(os_admin)
         else:
             cassandra = models.CassandraUser(
-                self.default_superuser_name, self.default_superuser_password)
+                models.CassandraUser.root_username,
+                self.default_superuser_password)
             CassandraAdmin(cassandra)._create_superuser(os_admin)
             CassandraAdmin(os_admin).drop_user(cassandra)
 
@@ -305,7 +325,7 @@ class CassandraApp(object):
         self.status = CassandraAppStatus(user)
 
     def store_admin_credentials(self, admin_credentials):
-        user = models.CassandraUser.deserialize_user(admin_credentials)
+        user = models.CassandraUser.deserialize(admin_credentials)
         self._update_admin_credentials(user)
 
     def get_admin_credentials(self):
@@ -422,10 +442,11 @@ class CassandraApp(object):
         if self.has_user_config():
             return self._load_current_superuser()
 
-        LOG.warn(_("Trove administrative user has not been configured yet. "
-                   "Using the built-in default: %s")
-                 % self.default_superuser_name)
-        return models.CassandraUser(self.default_superuser_name,
+        LOG.warning(
+            _("Trove administrative user has not been configured yet. "
+              "Using the built-in default: %s")
+            % models.CassandraUser.root_username)
+        return models.CassandraUser(models.CassandraUser.root_username,
                                     self.default_superuser_password)
 
     def has_user_config(self):
@@ -467,7 +488,7 @@ class CassandraApp(object):
 
     def _make_host_reachable(self):
         """
-        Some of these settings may be overriden by user defined
+        Some of these settings may be overridden by user defined
         configuration groups.
 
         authenticator and authorizer
@@ -686,6 +707,16 @@ class CassandraApp(object):
         # <keyspace> ( <table> ... )
         self._run_nodetool_command('flush', keyspace, *tables)
 
+    def set_logging_level(self, log_level):
+        """Set the log Cassandra's system log verbosity level.
+        """
+        # Apply the change at runtime.
+        self._run_nodetool_command('setlogginglevel', 'root', log_level)
+
+        # Persist the change.
+        self.logback_conf_manager.apply_system_override(
+            {'configuration': {'root': {'@level': log_level}}})
+
     def _run_nodetool_command(self, cmd, *args, **kwargs):
         """Execute a nodetool command on this node.
         """
@@ -697,7 +728,7 @@ class CassandraApp(object):
         Create a new superuser if it does not exist and grant it full
         superuser-level access to all keyspaces.
         """
-        cassandra = models.CassandraRootUser(password=root_password)
+        cassandra = models.CassandraUser.root(password=root_password)
         admin = self.build_admin()
         if self.is_root_enabled():
             admin.alter_user_password(cassandra)
@@ -839,9 +870,9 @@ class CassandraAdmin(object):
         List all non-superuser accounts. Omit names on the ignored list.
         Return an empty set if None.
         """
-        users = [user.serialize() for user in
-                 self._get_listed_users(self.client)]
-        return pagination.paginate_list(users, limit, marker, include_marker)
+        return guestagent_utils.serialize_list(
+            self._get_listed_users(self.client),
+            limit=limit, marker=marker, include_marker=include_marker)
 
     def _get_listed_users(self, client):
         """
@@ -863,7 +894,7 @@ class CassandraAdmin(object):
 
     def _load_user(self, client, username, check_reserved=True):
         if check_reserved:
-            self._check_reserved_user_name(username)
+            models.CassandraUser(username).check_reserved()
 
         acl = self._get_acl(client, username=username)
         return self._build_user(username, acl)
@@ -968,8 +999,8 @@ class CassandraAdmin(object):
         Grant all non-superuser permissions on a keyspace to a given user.
         """
         if check_reserved:
-            self._check_reserved_user_name(user.name)
-            self._check_reserved_keyspace_name(keyspace.name)
+            user.check_reserved()
+            keyspace.check_reserved()
 
         for access in self.__NO_SUPERUSER_MODIFIERS:
             self._grant_permission_on_keyspace(client, access, keyspace, user)
@@ -992,8 +1023,8 @@ class CassandraAdmin(object):
     def _revoke_all_access_on_keyspace(self, client, keyspace, user,
                                        check_reserved=True):
         if check_reserved:
-            self._check_reserved_user_name(user.name)
-            self._check_reserved_keyspace_name(keyspace.name)
+            user.check_reserved()
+            keyspace.check_reserved()
 
         LOG.debug("Revoking all permissions on '%s' from user '%s'."
                   % (keyspace.name, user.name))
@@ -1012,7 +1043,7 @@ class CassandraAdmin(object):
         Updatable attributes include username and password.
         If a new username and password are given a new user with those
         attributes is created and all permissions from the original
-        user get transfered to it. The original user is then dropped
+        user get transferred to it. The original user is then dropped
         therefore revoking its permissions.
         If only new password is specified the existing user gets altered
         with that password.
@@ -1092,10 +1123,9 @@ class CassandraAdmin(object):
 
     def list_databases(self, context, limit=None, marker=None,
                        include_marker=False):
-        databases = [keyspace.serialize() for keyspace
-                     in self._get_available_keyspaces(self.client)]
-        return pagination.paginate_list(databases, limit, marker,
-                                        include_marker)
+        return guestagent_utils.serialize_list(
+            self._get_available_keyspaces(self.client),
+            limit=limit, marker=marker, include_marker=include_marker)
 
     def _get_available_keyspaces(self, client):
         """
@@ -1116,32 +1146,24 @@ class CassandraAdmin(object):
 
     def _deserialize_keyspace(self, keyspace_dict, check_reserved=True):
         if keyspace_dict:
-            db = models.CassandraSchema.deserialize_schema(keyspace_dict)
+            db = models.CassandraSchema.deserialize(keyspace_dict)
             if check_reserved:
-                self._check_reserved_keyspace_name(db.name)
+                db.check_reserved()
 
             return db
 
         return None
 
-    def _check_reserved_keyspace_name(self, name):
-        if name in self.ignore_dbs:
-            raise ValueError(_("This keyspace-name is reserved: %s") % name)
-
     def _deserialize_user(self, user_dict, check_reserved=True):
         if user_dict:
-            user = models.CassandraUser.deserialize_user(user_dict)
+            user = models.CassandraUser.deserialize(user_dict)
             if check_reserved:
-                self._check_reserved_user_name(user.name)
+                user.check_reserved()
 
             return user
 
         return None
 
-    def _check_reserved_user_name(self, name):
-        if name in self.ignore_users:
-            raise ValueError(_("This user-name is reserved: %s") % name)
-
     @property
     def ignore_users(self):
         return cfg.get_ignored_users()
