@@ -22,8 +22,10 @@ import tempfile
 from oslo_log import log as logging
 from oslo_utils import netutils
 import pexpect
+import six
 
 from trove.common import cfg
+from trove.common.db import models
 from trove.common import exception
 from trove.common.i18n import _
 from trove.common import instance as rd_instance
@@ -31,7 +33,6 @@ from trove.common import utils as utils
 from trove.guestagent.common import operating_system
 from trove.guestagent.datastore.experimental.couchbase import system
 from trove.guestagent.datastore import service
-from trove.guestagent.db import models
 from trove.guestagent import pkg
 
 
@@ -91,7 +92,7 @@ class CouchbaseApp(object):
             LOG.info(_('Couchbase Server initial setup finished.'))
         except exception.ProcessExecutionError:
             LOG.exception(_('Error performing initial Couchbase setup.'))
-            raise RuntimeError("Couchbase Server initial setup failed")
+            raise RuntimeError(_("Couchbase Server initial setup failed"))
 
     def _install_couchbase(self, packages):
         """
@@ -129,7 +130,7 @@ class CouchbaseApp(object):
         if self.status.is_running:
             LOG.error(_("Cannot start Couchbase with configuration changes. "
                         "Couchbase state == %s.") % self.status)
-            raise RuntimeError("Couchbase is not stopped.")
+            raise RuntimeError(_("Couchbase is not stopped."))
         self._write_config(config_contents)
         self.start_db(True)
 
@@ -209,10 +210,7 @@ class CouchbaseRootAccess(object):
 
     @classmethod
     def enable_root(cls, root_password=None):
-        user = models.RootUser()
-        user.name = "root"
-        user.host = "%"
-        user.password = root_password or utils.generate_random_password()
+        user = models.DatastoreUser.root(password=root_password)
 
         if root_password:
             CouchbaseRootAccess().write_password_to_file(root_password)
@@ -246,6 +244,8 @@ class CouchbaseRootAccess(object):
         try:
             tempfd, tempname = tempfile.mkstemp()
             os.fchmod(tempfd, stat.S_IRUSR | stat.S_IWUSR)
+            if isinstance(root_password, six.text_type):
+                root_password = root_password.encode('utf-8')
             os.write(tempfd, root_password)
             os.fchmod(tempfd, stat.S_IRUSR)
             os.close(tempfd)
