@@ -20,10 +20,10 @@ from oslo_utils import netutils
 import pymongo
 
 from trove.common import cfg
+from trove.common.db.mongodb import models
 from trove.common import exception
 from trove.common.i18n import _
 from trove.common import instance as ds_instance
-from trove.common import pagination
 from trove.common.stream_codecs import JsonCodec, SafeYamlCodec
 from trove.common import utils as utils
 from trove.guestagent.common.configuration import ConfigurationManager
@@ -32,7 +32,6 @@ from trove.guestagent.common import guestagent_utils
 from trove.guestagent.common import operating_system
 from trove.guestagent.datastore.experimental.mongodb import system
 from trove.guestagent.datastore import service
-from trove.guestagent.db import models
 
 
 LOG = logging.getLogger(__name__)
@@ -255,7 +254,7 @@ class MongoDBApp(object):
     def add_config_servers(self, config_server_hosts):
         """Set config servers on a query router (mongos) instance.
         """
-        config_servers_string = ','.join(['%s:27019' % host
+        config_servers_string = ','.join(['%s:%s' % (host, CONFIGSVR_PORT)
                                           for host in config_server_hosts])
         LOG.info(_("Setting config servers: %s") % config_servers_string)
         self.configuration_manager.apply_system_override(
@@ -312,12 +311,14 @@ class MongoDBApp(object):
 
         MongoDBAdmin().rs_initiate()
         # TODO(ramashri) see if hardcoded values can be removed
-        utils.poll_until(check_initiate_status, sleep_time=30, time_out=100)
+        utils.poll_until(check_initiate_status, sleep_time=30,
+                         time_out=CONF.mongodb.add_members_timeout)
 
         # add replica-set members
         MongoDBAdmin().rs_add_members(members)
         # TODO(ramashri) see if hardcoded values can be removed
-        utils.poll_until(check_rs_status, sleep_time=10, time_out=100)
+        utils.poll_until(check_rs_status, sleep_time=10,
+                         time_out=CONF.mongodb.add_members_timeout)
 
     def _set_localhost_auth_bypass(self, enabled):
         """When active, the localhost exception allows connections from the
@@ -479,12 +480,6 @@ class MongoDBAdmin(object):
             type(self).admin_user = user
         return type(self).admin_user
 
-    def _is_modifiable_user(self, name):
-        if ((name in cfg.get_ignored_users(manager=MANAGER)) or
-                name == system.MONGO_ADMIN_NAME):
-            return False
-        return True
-
     @property
     def cmd_admin_auth_params(self):
         """Returns a list of strings that constitute MongoDB command line
@@ -508,10 +503,6 @@ class MongoDBAdmin(object):
         """
         LOG.debug('Creating user %s on database %s with roles %s.'
                   % (user.username, user.database.name, str(user.roles)))
-
-        if not user.password:
-            raise exception.BadRequest(_("User's password is empty."))
-
         if client:
             self._create_user_with_client(user, client)
         else:
@@ -524,15 +515,19 @@ class MongoDBAdmin(object):
         """
         with MongoDBClient(self._admin_user()) as client:
             for item in users:
-                user = models.MongoDBUser.deserialize_user(item)
-                if not self._is_modifiable_user(user.name):
-                    LOG.warning('Skipping creation of user with reserved '
-                                'name %(user)s' % {'user': user.name})
-                elif self._get_user_record(user.name, client=client):
-                    LOG.warning('Skipping creation of user with pre-existing '
-                                'name %(user)s' % {'user': user.name})
-                else:
+                user = models.MongoDBUser.deserialize(item)
+                # this could be called to create multiple users at once;
+                # catch exceptions, log the message, and continue
+                try:
+                    user.check_create()
+                    if self._get_user_record(user.name, client=client):
+                        raise ValueError(_('User with name %(user)s already '
+                                           'exists.') % {'user': user.name})
                     self.create_validated_user(user, client=client)
+                except (ValueError, pymongo.errors.PyMongoError) as e:
+                    LOG.error(e)
+                    LOG.warning(_('Skipping creation of user with name '
+                                  '%(user)s') % {'user': user.name})
 
     def delete_validated_user(self, user):
         """Deletes a user from their database. The caller should ensure that
@@ -548,20 +543,16 @@ class MongoDBAdmin(object):
         """Delete the given user.
         :param user:   a serialized user object
         """
-        user = models.MongoDBUser.deserialize_user(user)
-        if not self._is_modifiable_user(user.name):
-            raise exception.BadRequest(_(
-                'Cannot delete user with reserved name %(user)s')
-                % {'user': user.name})
-        else:
-            self.delete_validated_user(user)
+        user = models.MongoDBUser.deserialize(user)
+        user.check_delete()
+        self.delete_validated_user(user)
 
     def _get_user_record(self, name, client=None):
         """Get the user's record."""
         user = models.MongoDBUser(name)
-        if not self._is_modifiable_user(user.name):
-            LOG.warning('Skipping retrieval of user with reserved '
-                        'name %(user)s' % {'user': user.name})
+        if user.is_ignored:
+            LOG.warning(_('Skipping retrieval of user with reserved '
+                          'name %(user)s') % {'user': user.name})
             return None
         if client:
             user_info = client.admin.system.users.find_one(
@@ -575,6 +566,14 @@ class MongoDBAdmin(object):
         user.roles = user_info['roles']
         return user
 
+    def get_existing_user(self, name):
+        """Check that a user exists."""
+        user = self._get_user_record(name)
+        if not user:
+            raise ValueError(_('User with name %(user)s does not'
+                               'exist.') % {'user': name})
+        return user
+
     def get_user(self, name):
         """Get information for the given user."""
         LOG.debug('Getting user %s.' % name)
@@ -590,48 +589,51 @@ class MongoDBAdmin(object):
             for user_info in admin_client.admin.system.users.find():
                 user = models.MongoDBUser(name=user_info['_id'])
                 user.roles = user_info['roles']
-                if self._is_modifiable_user(user.name):
-                    users.append(user.serialize())
+                if not user.is_ignored:
+                    users.append(user)
         LOG.debug('users = ' + str(users))
-        return pagination.paginate_list(users, limit, marker,
-                                        include_marker)
+        return guestagent_utils.serialize_list(
+            users,
+            limit=limit, marker=marker, include_marker=include_marker)
 
     def change_passwords(self, users):
         with MongoDBClient(self._admin_user()) as admin_client:
             for item in users:
-                user = models.MongoDBUser.deserialize_user(item)
-                if not self._is_modifiable_user(user.name):
-                    LOG.warning('Skipping password change for user with '
-                                'reserved name %(user)s.'
-                                % {'user': user.name})
-                    return None
-                LOG.debug('Changing password for user %(user)s'
-                          % {'user': user.name})
-                self._create_user_with_client(user, admin_client)
+                user = models.MongoDBUser.deserialize(item)
+                # this could be called to create multiple users at once;
+                # catch exceptions, log the message, and continue
+                try:
+                    user.check_create()
+                    self.get_existing_user(user.name)
+                    self.create_validated_user(user, admin_client)
+                    LOG.debug('Changing password for user %(user)s'
+                              % {'user': user.name})
+                    self._create_user_with_client(user, admin_client)
+                except (ValueError, pymongo.errors.PyMongoError) as e:
+                    LOG.error(e)
+                    LOG.warning(_('Skipping password change for user with '
+                                  'name %(user)s') % {'user': user.name})
 
     def update_attributes(self, name, user_attrs):
         """Update user attributes."""
-        user = self._get_user_record(name)
-        if not user:
-            raise exception.BadRequest(_(
-                'Cannot update attributes for user %(user)s as it either does '
-                'not exist or is a reserved user.') % {'user': name})
+        user = self.get_existing_user(name)
         password = user_attrs.get('password')
         if password:
             user.password = password
             self.change_passwords([user.serialize()])
         if user_attrs.get('name'):
-            LOG.warning('Changing user name is not supported.')
+            LOG.warning(_('Changing user name is not supported.'))
         if user_attrs.get('host'):
-            LOG.warning('Changing user host is not supported.')
+            LOG.warning(_('Changing user host is not supported.'))
 
     def enable_root(self, password=None):
         """Create a user 'root' with role 'root'."""
         if not password:
             LOG.debug('Generating root user password.')
             password = utils.generate_random_password()
-        root_user = models.MongoDBUser(name='admin.root', password=password)
+        root_user = models.MongoDBUser.root(password=password)
         root_user.roles = {'db': 'admin', 'role': 'root'}
+        root_user.check_create()
         self.create_validated_user(root_user)
         return root_user.serialize()
 
@@ -650,11 +652,7 @@ class MongoDBAdmin(object):
 
     def grant_access(self, username, databases):
         """Adds the RW role to the user for each specified database."""
-        user = self._get_user_record(username)
-        if not user:
-            raise exception.BadRequest(_(
-                'Cannot grant access for reserved or non-existant user '
-                '%(user)s') % {'user': username})
+        user = self.get_existing_user(username)
         for db_name in databases:
             # verify the database name
             models.MongoDBSchema(db_name)
@@ -671,11 +669,7 @@ class MongoDBAdmin(object):
 
     def revoke_access(self, username, database):
         """Removes the RW role from the user for the specified database."""
-        user = self._get_user_record(username)
-        if not user:
-            raise exception.BadRequest(_(
-                'Cannot revoke access for reserved or non-existant user '
-                '%(user)s') % {'user': username})
+        user = self.get_existing_user(username)
         # verify the database name
         models.MongoDBSchema(database)
         role = {'db': database, 'role': 'readWrite'}
@@ -688,11 +682,7 @@ class MongoDBAdmin(object):
     def list_access(self, username):
         """Returns a list of all databases for which the user has the RW role.
         """
-        user = self._get_user_record(username)
-        if not user:
-            raise exception.BadRequest(_(
-                'Cannot list access for reserved or non-existant user '
-                '%(user)s') % {'user': username})
+        user = self.get_existing_user(username)
         return user.databases
 
     def create_database(self, databases):
@@ -703,17 +693,19 @@ class MongoDBAdmin(object):
         tmp = 'dummy'
         with MongoDBClient(self._admin_user()) as admin_client:
             for item in databases:
-                db_name = models.MongoDBSchema.deserialize_schema(item).name
-                LOG.debug('Creating MongoDB database %s' % db_name)
-                db = admin_client[db_name]
+                schema = models.MongoDBSchema.deserialize(item)
+                schema.check_create()
+                LOG.debug('Creating MongoDB database %s' % schema.name)
+                db = admin_client[schema.name]
                 db[tmp].insert({'dummy': True})
                 db.drop_collection(tmp)
 
     def delete_database(self, database):
         """Deletes the database."""
         with MongoDBClient(self._admin_user()) as admin_client:
-            db_name = models.MongoDBSchema.deserialize_schema(database).name
-            admin_client.drop_database(db_name)
+            schema = models.MongoDBSchema.deserialize(database)
+            schema.check_delete()
+            admin_client.drop_database(schema.name)
 
     def list_database_names(self):
         """Get the list of database names."""
@@ -722,15 +714,15 @@ class MongoDBAdmin(object):
 
     def list_databases(self, limit=None, marker=None, include_marker=False):
         """Lists the databases."""
-        db_names = self.list_database_names()
-        for hidden in cfg.get_ignored_dbs(manager=MANAGER):
-            if hidden in db_names:
-                db_names.remove(hidden)
-        databases = [models.MongoDBSchema(db_name).serialize()
-                     for db_name in db_names]
+        databases = []
+        for db_name in self.list_database_names():
+            schema = models.MongoDBSchema(name=db_name)
+            if not schema.is_ignored():
+                databases.append(schema)
         LOG.debug('databases = ' + str(databases))
-        return pagination.paginate_list(databases, limit, marker,
-                                        include_marker)
+        return guestagent_utils.serialize_list(
+            databases,
+            limit=limit, marker=marker, include_marker=include_marker)
 
     def add_shard(self, url):
         """Runs the addShard command."""
@@ -764,7 +756,7 @@ class MongoDBAdmin(object):
     def db_stats(self, database, scale=1):
         """Gets the stats for the given database."""
         with MongoDBClient(self._admin_user()) as admin_client:
-            db_name = models.MongoDBSchema.deserialize_schema(database).name
+            db_name = models.MongoDBSchema.deserialize(database).name
             return admin_client[db_name].command('dbStats', scale=scale)
 
     def list_active_shards(self):
