@@ -15,10 +15,12 @@
 #
 
 import abc
+import operator
 
 from oslo_config import cfg as oslo_cfg
 from oslo_log import log as logging
 from oslo_service import periodic_task
+from oslo_utils import encodeutils
 
 from trove.common import cfg
 from trove.common import exception
@@ -61,6 +63,8 @@ class Manager(periodic_task.PeriodicTasks):
     GUEST_LOG_DEFS_ERROR_LABEL = 'error'
     GUEST_LOG_DEFS_SLOW_QUERY_LABEL = 'slow_query'
 
+    MODULE_APPLY_TO_ALL = module_manager.ModuleManager.MODULE_APPLY_TO_ALL
+
     def __init__(self, manager_name):
         super(Manager, self).__init__(CONF)
 
@@ -251,7 +255,7 @@ class Manager(periodic_task.PeriodicTasks):
         return True
 
     #################
-    # Prepare related
+    # Instance related
     #################
     def prepare(self, context, packages, databases, memory_mb, users,
                 device_path=None, mount_point=None, backup_info=None,
@@ -285,7 +289,7 @@ class Manager(periodic_task.PeriodicTasks):
         except Exception as ex:
             self.prepare_error = True
             LOG.exception(_("An error occurred preparing datastore: %s") %
-                          ex.message)
+                          encodeutils.exception_to_unicode(ex))
             raise
         finally:
             LOG.info(_("Ending datastore prepare for '%s'.") % self.manager)
@@ -318,12 +322,16 @@ class Manager(periodic_task.PeriodicTasks):
                     LOG.info(_("Creating databases (called from 'prepare')."))
                     self.create_database(context, databases)
                     LOG.info(_('Databases created successfully.'))
+            except Exception as ex:
+                LOG.exception(_("An error occurred creating databases: "
+                                "%s") % ex.message)
+            try:
                 if users:
                     LOG.info(_("Creating users (called from 'prepare')"))
                     self.create_user(context, users)
                     LOG.info(_('Users created successfully.'))
             except Exception as ex:
-                LOG.exception(_("An error occurred creating databases/users: "
+                LOG.exception(_("An error occurred creating users: "
                                 "%s") % ex.message)
 
             # We only enable-root automatically if not restoring a backup
@@ -388,6 +396,18 @@ class Manager(periodic_task.PeriodicTasks):
         LOG.info(_('No post_prepare work has been defined.'))
         pass
 
+    def pre_upgrade(self, context):
+        """Prepares the guest for upgrade, returning a dict to be passed
+        to post_upgrade
+        """
+        return {}
+
+    def post_upgrade(self, context, upgrade_info):
+        """Recovers the guest after the image is upgraded using information
+        from the pre_upgrade step
+        """
+        pass
+
     #################
     # Service related
     #################
@@ -406,11 +426,12 @@ class Manager(periodic_task.PeriodicTasks):
         LOG.debug("Getting file system stats for '%s'" % mount_point)
         return dbaas.get_filesystem_volume_stats(mount_point)
 
-    def mount_volume(self, context, device_path=None, mount_point=None):
+    def mount_volume(self, context, device_path=None, mount_point=None,
+                     write_to_fstab=False):
         LOG.debug("Mounting the device %s at the mount point %s." %
                   (device_path, mount_point))
         device = volume.VolumeDevice(device_path)
-        device.mount(mount_point, write_to_fstab=False)
+        device.mount(mount_point, write_to_fstab=write_to_fstab)
 
     def unmount_volume(self, context, device_path=None, mount_point=None):
         LOG.debug("Unmounting the device %s from the mount point %s." %
@@ -452,8 +473,8 @@ class Manager(periodic_task.PeriodicTasks):
         self.guest_log_context = context
         gl_cache = self.guest_log_cache
         result = filter(None, [gl_cache[log_name].show()
-                        if gl_cache[log_name].exposed else None
-                        for log_name in gl_cache.keys()])
+                               if gl_cache[log_name].exposed else None
+                               for log_name in gl_cache.keys()])
         LOG.info(_("Returning list of logs: %s") % result)
         return result
 
@@ -466,7 +487,7 @@ class Manager(periodic_task.PeriodicTasks):
         if publish and not disable:
             enable = True
         LOG.info(_("Processing guest log '%(log)s' "
-                 "(enable=%(en)s, disable=%(dis)s, "
+                   "(enable=%(en)s, disable=%(dis)s, "
                    "publish=%(pub)s, discard=%(disc)s).") %
                  {'log': log_name, 'en': enable, 'dis': disable,
                   'pub': publish, 'disc': discard})
@@ -540,7 +561,7 @@ class Manager(periodic_task.PeriodicTasks):
                      "no configuration manager defined!") %
                    {'verb': verb, 'log': log_name})
             LOG.error(msg)
-            raise exception.GuestError(msg)
+            raise exception.GuestError(original_message=msg)
 
         return restart_required
 
@@ -554,7 +575,7 @@ class Manager(periodic_task.PeriodicTasks):
             if section_label:
                 config_man_values = {section_label: cfg_values}
             self.configuration_manager.apply_system_override(
-                config_man_values, change_id=apply_label)
+                config_man_values, change_id=apply_label, pre_user=True)
         if restart_required:
             self.status.set_status(instance.ServiceStatuses.RESTART_REQUIRED)
         else:
@@ -626,18 +647,36 @@ class Manager(periodic_task.PeriodicTasks):
     def module_apply(self, context, modules=None):
         LOG.info(_("Applying modules."))
         results = []
-        for module_data in modules:
-            module = module_data['module']
+        modules = [data['module'] for data in modules]
+        try:
+            # make sure the modules are applied in the correct order
+            modules.sort(key=operator.itemgetter('apply_order'))
+            modules.sort(key=operator.itemgetter('priority_apply'),
+                         reverse=True)
+        except KeyError:
+            # If we don't have ordering info then maybe we're running
+            # a version of the module feature before ordering was
+            # introduced.  In that case, since we don't have any
+            # way to order the modules we should just continue.
+            pass
+        for module in modules:
             id = module.get('id', None)
             module_type = module.get('type', None)
             name = module.get('name', None)
-            tenant = module.get('tenant', None)
-            datastore = module.get('datastore', None)
-            ds_version = module.get('datastore_version', None)
+            tenant = module.get('tenant', self.MODULE_APPLY_TO_ALL)
+            datastore = module.get('datastore', self.MODULE_APPLY_TO_ALL)
+            ds_version = module.get('datastore_version',
+                                    self.MODULE_APPLY_TO_ALL)
             contents = module.get('contents', None)
             md5 = module.get('md5', None)
             auto_apply = module.get('auto_apply', True)
             visible = module.get('visible', True)
+            is_admin = module.get('is_admin', None)
+            if is_admin is None:
+                # fall back to the old method of checking for an admin option
+                is_admin = (tenant == self.MODULE_APPLY_TO_ALL or
+                            not visible or
+                            auto_apply)
             if not name:
                 raise AttributeError(_("Module name not specified"))
             if not contents:
@@ -647,9 +686,14 @@ class Manager(periodic_task.PeriodicTasks):
                 raise exception.ModuleTypeNotFound(
                     _("No driver implemented for module type '%s'") %
                     module_type)
+            if (datastore and datastore != self.MODULE_APPLY_TO_ALL and
+                    datastore != CONF.datastore_manager):
+                reason = (_("Module not valid for datastore %s") %
+                          CONF.datastore_manager)
+                raise exception.ModuleInvalid(reason=reason)
             result = module_manager.ModuleManager.apply_module(
                 driver, module_type, name, tenant, datastore, ds_version,
-                contents, id, md5, auto_apply, visible)
+                contents, id, md5, auto_apply, visible, is_admin)
             results.append(result)
         LOG.info(_("Returning list of modules: %s") % results)
         return results
