@@ -15,11 +15,11 @@
 #
 
 import datetime
+import operator
 import os
 
 from oslo_log import log as logging
 
-from trove.common import cfg
 from trove.common import exception
 from trove.common.i18n import _
 from trove.common import stream_codecs
@@ -28,7 +28,6 @@ from trove.guestagent.common import operating_system
 
 
 LOG = logging.getLogger(__name__)
-CONF = cfg.CONF
 
 
 class ModuleManager(object):
@@ -43,12 +42,12 @@ class ModuleManager(object):
 
     @classmethod
     def get_current_timestamp(cls):
-        return datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
+        return datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")[0:22]
 
     @classmethod
     def apply_module(cls, driver, module_type, name, tenant,
                      datastore, ds_version, contents, module_id, md5,
-                     auto_apply, visible):
+                     auto_apply, visible, admin_module):
         tenant = tenant or cls.MODULE_APPLY_TO_ALL
         datastore = datastore or cls.MODULE_APPLY_TO_ALL
         ds_version = ds_version or cls.MODULE_APPLY_TO_ALL
@@ -59,29 +58,33 @@ class ModuleManager(object):
         now = cls.get_current_timestamp()
         default_result = cls.build_default_result(
             module_type, name, tenant, datastore,
-            ds_version, module_id, md5, auto_apply, visible, now)
+            ds_version, module_id, md5,
+            auto_apply, visible, now, admin_module)
         result = cls.read_module_result(module_dir, default_result)
         try:
+            driver.configure(name, datastore, ds_version, data_file)
             applied, message = driver.apply(
-                name, datastore, ds_version, data_file)
+                name, datastore, ds_version, data_file, admin_module)
         except Exception as ex:
             LOG.exception(_("Could not apply module '%s'") % name)
             applied = False
             message = ex.message
         finally:
             status = 'OK' if applied else 'ERROR'
-            admin_only = (not visible or tenant == cls.MODULE_APPLY_TO_ALL or
-                          auto_apply)
             result['removed'] = None
             result['status'] = status
             result['message'] = message
             result['updated'] = now
             result['id'] = module_id
             result['md5'] = md5
+            result['type'] = module_type
+            result['name'] = name
+            result['datastore'] = datastore
+            result['datastore_version'] = ds_version
             result['tenant'] = tenant
             result['auto_apply'] = auto_apply
             result['visible'] = visible
-            result['admin_only'] = admin_only
+            result['is_admin'] = admin_module
             cls.write_module_result(module_dir, result)
         return result
 
@@ -111,9 +114,7 @@ class ModuleManager(object):
     @classmethod
     def build_default_result(cls, module_type, name, tenant,
                              datastore, ds_version, module_id, md5,
-                             auto_apply, visible, now):
-        admin_only = (not visible or tenant == cls.MODULE_APPLY_TO_ALL or
-                      auto_apply)
+                             auto_apply, visible, now, admin_module):
         result = {
             'type': module_type,
             'name': name,
@@ -129,11 +130,16 @@ class ModuleManager(object):
             'removed': None,
             'auto_apply': auto_apply,
             'visible': visible,
-            'admin_only': admin_only,
+            'is_admin': admin_module,
             'contents': None,
         }
         return result
 
+    @classmethod
+    def is_admin_module(cls, tenant, auto_apply, visible):
+        return (not visible or tenant == cls.MODULE_APPLY_TO_ALL or
+                auto_apply)
+
     @classmethod
     def read_module_result(cls, result_file, default=None):
         result_file = cls.get_result_filename(result_file)
@@ -177,7 +183,9 @@ class ModuleManager(object):
                     (is_admin or result.get('visible'))):
                 if include_contents:
                     codec = stream_codecs.Base64Codec()
-                    if not is_admin and result.get('admin_only'):
+                    # keep admin_only for backwards compatibility
+                    if not is_admin and (result.get('is_admin') or
+                                         result.get('admin_only')):
                         contents = (
                             "Must be admin to retrieve contents for module %s"
                             % result.get('name', 'Unknown'))
@@ -189,6 +197,7 @@ class ModuleManager(object):
                         result['contents'] = operating_system.read_file(
                             contents_file, codec=codec, decode=False)
                 results.append(result)
+        results.sort(key=operator.itemgetter('updated'), reverse=True)
         return results
 
     @classmethod
@@ -203,6 +212,7 @@ class ModuleManager(object):
             raise exception.NotFound(
                 _("Module '%s' has not been applied") % name)
         try:
+            driver.configure(name, datastore, ds_version, contents_file)
             removed, message = driver.remove(
                 name, datastore, ds_version, contents_file)
             cls.remove_module_result(module_dir)
