@@ -49,11 +49,18 @@ class InnoBackupEx(base.BackupRunner):
     """Implementation of Backup Strategy for InnoBackupEx."""
     __strategy_name__ = 'innobackupex'
 
+    @property
+    def user_and_pass(self):
+        return (' --user=%(user)s --password=%(password)s ' %
+                {'user': ADMIN_USER_NAME,
+                 'password': MySqlApp.get_auth_password()})
+
     @property
     def cmd(self):
         cmd = ('sudo innobackupex'
                ' --stream=xbstream'
                ' %(extra_opts)s ' +
+               self.user_and_pass +
                MySqlApp.get_data_dir() +
                ' 2>/tmp/innobackupex.log'
                )
@@ -97,7 +104,7 @@ class InnoBackupExIncremental(InnoBackupEx):
 
     def __init__(self, *args, **kwargs):
         if not kwargs.get('lsn'):
-            raise AttributeError('lsn attribute missing, bad parent?')
+            raise AttributeError(_('lsn attribute missing, bad parent?'))
         super(InnoBackupExIncremental, self).__init__(*args, **kwargs)
         self.parent_location = kwargs.get('parent_location')
         self.parent_checksum = kwargs.get('parent_checksum')
@@ -109,6 +116,7 @@ class InnoBackupExIncremental(InnoBackupEx):
                ' --incremental'
                ' --incremental-lsn=%(lsn)s'
                ' %(extra_opts)s ' +
+               self.user_and_pass +
                MySqlApp.get_data_dir() +
                ' 2>/tmp/innobackupex.log')
         return cmd + self.zip_cmd + self.encrypt_cmd
