@@ -18,7 +18,6 @@ import csv
 
 from oslo_log import log as logging
 
-from trove.common import cfg
 from trove.common import exception
 from trove.common.i18n import _
 from trove.guestagent.backup.backupagent import BackupAgent
@@ -28,7 +27,6 @@ from trove.guestagent.datastore.mysql.service import MySqlApp
 from trove.guestagent.strategies.replication import mysql_base
 
 AGENT = BackupAgent()
-CONF = cfg.CONF
 
 LOG = logging.getLogger(__name__)
 
@@ -69,8 +67,8 @@ class MysqlBinlogReplication(mysql_base.MysqlReplicationBase):
         LOG.info(_("Reading log position from %s") % INFO_FILE)
         try:
             with open(INFO_FILE, 'rb') as f:
-                row = csv.reader(f, delimiter='\t',
-                                 skipinitialspace=True).next()
+                row = next(csv.reader(f, delimiter='\t',
+                                      skipinitialspace=True))
                 return {
                     'log_file': row[0],
                     'log_position': int(row[1])
