@@ -13,11 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-import csv
-
 from oslo_log import log as logging
 
-from trove.common import cfg
 from trove.common import exception
 from trove.common.i18n import _
 from trove.guestagent.backup.backupagent import BackupAgent
@@ -27,7 +24,6 @@ from trove.guestagent.datastore.mysql.service import MySqlApp
 from trove.guestagent.strategies.replication import mysql_base
 
 AGENT = BackupAgent()
-CONF = cfg.CONF
 
 LOG = logging.getLogger(__name__)
 
@@ -75,8 +71,7 @@ class MysqlGTIDReplication(mysql_base.MysqlReplicationBase):
         LOG.info(_("Reading last master GTID from %s") % INFO_FILE)
         try:
             with open(INFO_FILE, 'rb') as f:
-                row = csv.reader(f, delimiter='\t',
-                                 skipinitialspace=True).next()
+                row = f.read().split('\t')
                 return row[2]
         except (IOError, IndexError) as ex:
             LOG.exception(ex)
