@@ -27,6 +27,7 @@ from trove.common.i18n import _LI
 from trove.common import notification
 from trove.common.notification import StartNotification
 from trove.common import pagination
+from trove.common import policy
 from trove.common.remote import create_guest_client
 from trove.common import utils
 from trove.common import wsgi
@@ -47,6 +48,11 @@ class InstanceController(wsgi.Controller):
     """Controller for instance functionality."""
     schemas = apischema.instance.copy()
 
+    @classmethod
+    def authorize_instance_action(cls, context, instance_rule_name, instance):
+        policy.authorize_on_target(context, 'instance:%s' % instance_rule_name,
+                                   {'tenant': instance.tenant_id})
+
     @classmethod
     def get_action_schema(cls, body, action_schema):
         action_type = list(body.keys())[0]
@@ -78,7 +84,6 @@ class InstanceController(wsgi.Controller):
         if not body:
             raise exception.BadRequest(_("Invalid request body."))
         context = req.environ[wsgi.CONTEXT_KEY]
-        instance = models.Instance.load(context, id)
         _actions = {
             'restart': self._action_restart,
             'resize': self._action_resize,
@@ -86,6 +91,7 @@ class InstanceController(wsgi.Controller):
             'promote_to_replica_source':
             self._action_promote_to_replica_source,
             'eject_replica_source': self._action_eject_replica_source,
+            'reset_status': self._action_reset_status,
         }
         selected_action = None
         action_name = None
@@ -97,11 +103,16 @@ class InstanceController(wsgi.Controller):
                      "instance %(instance_id)s for tenant '%(tenant_id)s'"),
                  {'action_name': action_name, 'instance_id': id,
                   'tenant_id': tenant_id})
+        needs_server = True
+        if action_name in ['reset_status']:
+            needs_server = False
+        instance = models.Instance.load(context, id, needs_server=needs_server)
         return selected_action(context, req, instance, body)
 
     def _action_restart(self, context, req, instance, body):
         context.notification = notification.DBaaSInstanceRestart(context,
                                                                  request=req)
+        self.authorize_instance_action(context, 'restart', instance)
         with StartNotification(context, instance_id=instance.id):
             instance.restart()
         return wsgi.Result(None, 202)
@@ -132,6 +143,8 @@ class InstanceController(wsgi.Controller):
     def _action_resize_volume(self, context, req, instance, volume):
         context.notification = notification.DBaaSInstanceResizeVolume(
             context, request=req)
+        self.authorize_instance_action(context, 'resize_volume', instance)
+
         with StartNotification(context, instance_id=instance.id,
                                new_size=volume['size']):
             instance.resize_volume(volume['size'])
@@ -140,6 +153,8 @@ class InstanceController(wsgi.Controller):
     def _action_resize_flavor(self, context, req, instance, flavorRef):
         context.notification = notification.DBaaSInstanceResizeInstance(
             context, request=req)
+        self.authorize_instance_action(context, 'resize_flavor', instance)
+
         new_flavor_id = utils.get_id_from_href(flavorRef)
         with StartNotification(context, instance_id=instance.id,
                                new_flavor_id=new_flavor_id):
@@ -150,6 +165,8 @@ class InstanceController(wsgi.Controller):
         raise webob.exc.HTTPNotImplemented()
 
     def _action_promote_to_replica_source(self, context, req, instance, body):
+        self.authorize_instance_action(
+            context, 'promote_to_replica_source', instance)
         context.notification = notification.DBaaSInstanceEject(context,
                                                                request=req)
         with StartNotification(context, instance_id=instance.id):
@@ -157,17 +174,36 @@ class InstanceController(wsgi.Controller):
         return wsgi.Result(None, 202)
 
     def _action_eject_replica_source(self, context, req, instance, body):
+        self.authorize_instance_action(
+            context, 'eject_replica_source', instance)
         context.notification = notification.DBaaSInstancePromote(context,
                                                                  request=req)
         with StartNotification(context, instance_id=instance.id):
             instance.eject_replica_source()
         return wsgi.Result(None, 202)
 
+    def _action_reset_status(self, context, req, instance, body):
+        if 'force_delete' in body['reset_status']:
+            self.authorize_instance_action(context, 'force_delete', instance)
+        else:
+            self.authorize_instance_action(
+                context, 'reset_status', instance)
+        context.notification = notification.DBaaSInstanceResetStatus(
+            context, request=req)
+        with StartNotification(context, instance_id=instance.id):
+            instance.reset_status()
+
+            LOG.debug("Failing backups for instance %s." % instance.id)
+            backup_model.fail_for_instance(instance.id)
+
+        return wsgi.Result(None, 202)
+
     def index(self, req, tenant_id):
         """Return all instances."""
         LOG.info(_LI("Listing database instances for tenant '%s'"), tenant_id)
         LOG.debug("req : '%s'\n\n", req)
         context = req.environ[wsgi.CONTEXT_KEY]
+        policy.authorize_on_tenant(context, 'instance:index')
         clustered_q = req.GET.get('include_clustered', '').lower()
         include_clustered = clustered_q == 'true'
         servers, marker = models.Instances.load(context, include_clustered)
@@ -182,6 +218,10 @@ class InstanceController(wsgi.Controller):
                  id)
         LOG.debug("req : '%s'\n\n", req)
         context = req.environ[wsgi.CONTEXT_KEY]
+
+        instance = models.Instance.load(context, id)
+        self.authorize_instance_action(context, 'backups', instance)
+
         backups, marker = backup_model.list_for_instance(context, id)
         view = backup_views.BackupViews(backups)
         paged = pagination.SimplePaginatedDataView(req.url, 'backups', view,
@@ -196,8 +236,9 @@ class InstanceController(wsgi.Controller):
         LOG.debug("req : '%s'\n\n", req)
 
         context = req.environ[wsgi.CONTEXT_KEY]
-        server = models.load_instance_with_guest(models.DetailInstance,
-                                                 context, id)
+        server = models.load_instance_with_info(models.DetailInstance,
+                                                context, id)
+        self.authorize_instance_action(context, 'show', server)
         return wsgi.Result(views.InstanceDetailView(server,
                                                     req=req).data(), 200)
 
@@ -209,6 +250,7 @@ class InstanceController(wsgi.Controller):
         LOG.debug("req : '%s'\n\n", req)
         context = req.environ[wsgi.CONTEXT_KEY]
         instance = models.load_any_instance(context, id)
+        self.authorize_instance_action(context, 'delete', instance)
         context.notification = notification.DBaaSInstanceDelete(
             context, request=req)
         with StartNotification(context, instance_id=instance.id):
@@ -232,6 +274,7 @@ class InstanceController(wsgi.Controller):
         LOG.debug("req : '%s'\n\n", strutils.mask_password(req))
         LOG.debug("body : '%s'\n\n", strutils.mask_password(body))
         context = req.environ[wsgi.CONTEXT_KEY]
+        policy.authorize_on_tenant(context, 'instance:create')
         context.notification = notification.DBaaSInstanceCreate(context,
                                                                 request=req)
         datastore_args = body['instance'].get('datastore', {})
@@ -253,6 +296,25 @@ class InstanceController(wsgi.Controller):
         except ValueError as ve:
             raise exception.BadRequest(msg=ve)
 
+        modules = body['instance'].get('modules')
+
+        # The following operations have their own API calls.
+        # We need to make sure the same policies are enforced when
+        # creating an instance.
+        # i.e. if attaching configuration group to an existing instance is not
+        # allowed, it should not be possible to create a new instance with the
+        # group attached either
+        if configuration:
+            policy.authorize_on_tenant(context, 'instance:update')
+        if modules:
+            policy.authorize_on_tenant(context, 'instance:module_apply')
+        if users:
+            policy.authorize_on_tenant(
+                context, 'instance:extension:user:create')
+        if databases:
+            policy.authorize_on_tenant(
+                context, 'instance:extension:database:create')
+
         if 'volume' in body['instance']:
             volume_info = body['instance']['volume']
             volume_size = int(volume_info['size'])
@@ -274,7 +336,22 @@ class InstanceController(wsgi.Controller):
                                            # also check for older name
                                            body['instance'].get('slave_of'))
         replica_count = body['instance'].get('replica_count')
-        modules = body['instance'].get('modules')
+        locality = body['instance'].get('locality')
+        if locality:
+            locality_domain = ['affinity', 'anti-affinity']
+            locality_domain_msg = ("Invalid locality '%s'. "
+                                   "Must be one of ['%s']" %
+                                   (locality,
+                                    "', '".join(locality_domain)))
+            if locality not in locality_domain:
+                raise exception.BadRequest(msg=locality_domain_msg)
+            if slave_of_id:
+                dupe_locality_msg = (
+                    'Cannot specify locality when adding replicas to existing '
+                    'master.')
+                raise exception.BadRequest(msg=dupe_locality_msg)
+        region_name = body['instance'].get('region_name', CONF.os_region_name)
+
         instance = models.Instance.create(context, name, flavor_id,
                                           image_id, databases, users,
                                           datastore, datastore_version,
@@ -283,7 +360,9 @@ class InstanceController(wsgi.Controller):
                                           configuration, slave_of_id,
                                           replica_count=replica_count,
                                           volume_type=volume_type,
-                                          modules=modules)
+                                          modules=modules,
+                                          locality=locality,
+                                          region_name=region_name)
 
         view = views.InstanceDetailView(instance, req=req)
         return wsgi.Result(view.data(), 200)
@@ -296,15 +375,6 @@ class InstanceController(wsgi.Controller):
                 return configuration_id
 
     def _modify_instance(self, context, req, instance, **kwargs):
-        """Modifies the instance using the specified keyword arguments
-        'detach_replica': ignored if not present or False, if True,
-        specifies the instance is a replica that will be detached from
-        its master
-        'configuration_id': Ignored if not present, if None, detaches an
-        an attached configuration group, if not None, attaches the
-        specified configuration group
-        """
-
         if 'detach_replica' in kwargs and kwargs['detach_replica']:
             LOG.debug("Detaching replica from source.")
             context.notification = notification.DBaaSInstanceDetach(
@@ -319,13 +389,21 @@ class InstanceController(wsgi.Controller):
                 configuration_id = kwargs['configuration_id']
                 with StartNotification(context, instance_id=instance.id,
                                        configuration_id=configuration_id):
-                    instance.assign_configuration(configuration_id)
+                    instance.attach_configuration(configuration_id)
             else:
                 context.notification = (
                     notification.DBaaSInstanceDetachConfiguration(context,
                                                                   request=req))
                 with StartNotification(context, instance_id=instance.id):
-                    instance.unassign_configuration()
+                    instance.detach_configuration()
+        if 'datastore_version' in kwargs:
+            datastore_version = datastore_models.DatastoreVersion.load(
+                instance.datastore, kwargs['datastore_version'])
+            context.notification = (
+                notification.DBaaSInstanceUpgrade(context, request=req))
+            with StartNotification(context, instance_id=instance.id,
+                                   datastore_version_id=datastore_version.id):
+                instance.upgrade(datastore_version)
         if kwargs:
             instance.update_db(**kwargs)
 
@@ -339,6 +417,7 @@ class InstanceController(wsgi.Controller):
         context = req.environ[wsgi.CONTEXT_KEY]
 
         instance = models.Instance.load(context, id)
+        self.authorize_instance_action(context, 'update', instance)
 
         # Make sure args contains a 'configuration_id' argument,
         args = {}
@@ -356,6 +435,7 @@ class InstanceController(wsgi.Controller):
         context = req.environ[wsgi.CONTEXT_KEY]
 
         instance = models.Instance.load(context, id)
+        self.authorize_instance_action(context, 'edit', instance)
 
         args = {}
         args['detach_replica'] = ('replica_of' in body['instance'] or
@@ -365,6 +445,10 @@ class InstanceController(wsgi.Controller):
             args['name'] = body['instance']['name']
         if 'configuration' in body['instance']:
             args['configuration_id'] = self._configuration_parse(context, body)
+        if 'datastore_version' in body['instance']:
+            args['datastore_version'] = body['instance'].get(
+                'datastore_version')
+
         self._modify_instance(context, req, instance, **args)
         return wsgi.Result(None, 202)
 
@@ -375,6 +459,8 @@ class InstanceController(wsgi.Controller):
         LOG.info(_LI("Getting default configuration for instance %s"), id)
         context = req.environ[wsgi.CONTEXT_KEY]
         instance = models.Instance.load(context, id)
+        self.authorize_instance_action(context, 'configuration', instance)
+
         LOG.debug("Server: %s", instance)
         config = instance.get_default_configuration_template()
         LOG.debug("Default config for instance %(instance_id)s is %(config)s",
@@ -389,6 +475,7 @@ class InstanceController(wsgi.Controller):
         instance = models.Instance.load(context, id)
         if not instance:
             raise exception.NotFound(uuid=id)
+        self.authorize_instance_action(context, 'guest_log_list', instance)
         client = create_guest_client(context, id)
         guest_log_list = client.guest_log_list()
         return wsgi.Result({'logs': guest_log_list}, 200)
@@ -418,6 +505,7 @@ class InstanceController(wsgi.Controller):
         instance = models.Instance.load(context, id)
         if not instance:
             raise exception.NotFound(uuid=id)
+        self.authorize_instance_action(context, 'module_list', instance)
         from_guest = bool(req.GET.get('from_guest', '').lower())
         include_contents = bool(req.GET.get('include_contents', '').lower())
         if from_guest:
@@ -434,7 +522,7 @@ class InstanceController(wsgi.Controller):
         return wsgi.Result({'modules': result_list}, 200)
 
     def _module_list(self, context, id, include_contents):
-        """Return information about instnace modules."""
+        """Return information about instance modules."""
         client = create_guest_client(context, id)
         result_list = client.module_list(include_contents)
         return wsgi.Result({'modules': result_list}, 200)
@@ -445,15 +533,12 @@ class InstanceController(wsgi.Controller):
         instance = models.Instance.load(context, id)
         if not instance:
             raise exception.NotFound(uuid=id)
+        self.authorize_instance_action(context, 'module_apply', instance)
         module_ids = [mod['id'] for mod in body.get('modules', [])]
         modules = module_models.Modules.load_by_ids(context, module_ids)
-        module_list = []
-        for module in modules:
-            module.contents = module_models.Module.deprocess_contents(
-                module.contents)
-            module_info = module_views.DetailedModuleView(module).data(
-                include_contents=True)
-            module_list.append(module_info)
+        models.validate_modules_for_apply(
+            modules, instance.datastore.id, instance.datastore_version.id)
+        module_list = module_views.get_module_list(modules)
         client = create_guest_client(context, id)
         result_list = client.module_apply(module_list)
         models.Instance.add_instance_modules(context, id, modules)
@@ -465,12 +550,15 @@ class InstanceController(wsgi.Controller):
         instance = models.Instance.load(context, id)
         if not instance:
             raise exception.NotFound(uuid=id)
+        self.authorize_instance_action(context, 'module_remove', instance)
         module = module_models.Module.load(context, module_id)
         module_info = module_views.DetailedModuleView(module).data()
         client = create_guest_client(context, id)
         client.module_remove(module_info)
-        instance_module = module_models.InstanceModule.load(
+        instance_modules = module_models.InstanceModules.load_all(
             context, instance_id=id, module_id=module_id)
-        if instance_module:
+        for instance_module in instance_modules:
             module_models.InstanceModule.delete(context, instance_module)
+            LOG.debug("Deleted IM record %s (instance %s, module %s)." %
+                      (instance_module.id, id, module_id))
         return wsgi.Result(None, 200)
