@@ -15,12 +15,11 @@
 
 from oslo_log import log as logging
 
-from trove.common import cfg
 from trove.common.views import create_links
+from trove.common import wsgi
 from trove.instance import models
 
 LOG = logging.getLogger(__name__)
-CONF = cfg.CONF
 
 
 class InstanceView(object):
@@ -29,6 +28,7 @@ class InstanceView(object):
     def __init__(self, instance, req=None):
         self.instance = instance
         self.req = req
+        self.context = req.environ[wsgi.CONTEXT_KEY]
 
     def data(self):
         instance_dict = {
@@ -39,6 +39,7 @@ class InstanceView(object):
             "flavor": self._build_flavor_info(),
             "datastore": {"type": self.instance.datastore.name,
                           "version": self.instance.datastore_version.name},
+            "region": self.instance.region_name
         }
         if self.instance.volume_support:
             instance_dict['volume'] = {'size': self.instance.volume_size}
@@ -123,6 +124,12 @@ class InstanceDetailView(InstanceView):
         if self.instance.shard_id:
             result['instance']['shard_id'] = self.instance.shard_id
 
+        if self.context.is_admin:
+            result['instance']['server_id'] = self.instance.server_id
+            result['instance']['volume_id'] = self.instance.volume_id
+            result['instance']['encrypted_rpc_messaging'] = (
+                self.instance.encrypted_rpc_messaging)
+
         return result
 
     def _build_fault_info(self):
