@@ -13,10 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from sets import Set
-
 from oslo_log import log as logging
-import oslo_messaging as messaging
 from oslo_service import periodic_task
 from oslo_utils import importutils
 
@@ -29,8 +26,9 @@ from trove.common.exception import TroveError
 from trove.common.i18n import _
 from trove.common.notification import DBaaSQuotas, EndNotification
 from trove.common import remote
-import trove.common.rpc.version as rpc_version
+from trove.common import server_group as srv_grp
 from trove.common.strategies.cluster import strategy
+from trove.datastore.models import DatastoreVersion
 import trove.extensions.mgmt.instances.models as mgmtmodels
 from trove.instance.tasks import InstanceTasks
 from trove.taskmanager import models
@@ -43,8 +41,6 @@ CONF = cfg.CONF
 
 class Manager(periodic_task.PeriodicTasks):
 
-    target = messaging.Target(version=rpc_version.RPC_API_VERSION)
-
     def __init__(self):
         super(Manager, self).__init__(CONF)
         self.admin_context = TroveContext(
@@ -93,6 +89,11 @@ class Manager(periodic_task.PeriodicTasks):
             instance.db_info.save()
 
     def promote_to_replica_source(self, context, instance_id):
+        # TODO(atomic77) Promote and eject need to be able to handle the case
+        # where a datastore like Postgresql needs to treat the slave to be
+        # promoted differently from the old master and the slaves which will
+        # be simply reassigned to a new master. See:
+        # https://bugs.launchpad.net/trove/+bug/1553339
 
         def _promote_to_replica_source(old_master, master_candidate,
                                        replica_models):
@@ -115,41 +116,44 @@ class Manager(periodic_task.PeriodicTasks):
             # and possibly some number of "orphaned" slaves
 
             exception_replicas = []
+            error_messages = ""
             for replica in replica_models:
                 try:
                     if replica.id != master_candidate.id:
                         replica.detach_replica(old_master, for_failover=True)
                         replica.attach_replica(master_candidate)
-                except exception.TroveError:
-                    msg = _("promote-to-replica-source: Unable to migrate "
-                            "replica %(slave)s from old replica source "
-                            "%(old_master)s to new source %(new_master)s.")
-                    msg_values = {
-                        "slave": replica.id,
-                        "old_master": old_master.id,
-                        "new_master": master_candidate.id
-                    }
-                    LOG.exception(msg % msg_values)
+                except exception.TroveError as ex:
+                    msg = (_("Unable to migrate replica %(slave)s from "
+                             "old replica source %(old_master)s to "
+                             "new source %(new_master)s on promote.") %
+                           {"slave": replica.id,
+                            "old_master": old_master.id,
+                            "new_master": master_candidate.id})
+                    LOG.exception(msg)
                     exception_replicas.append(replica)
+                    error_messages += "%s (%s)\n" % (msg, ex)
 
             try:
                 old_master.demote_replication_master()
-            except Exception:
-                LOG.exception(_("Exception demoting old replica source"))
+            except Exception as ex:
+                msg = (_("Exception demoting old replica source %s.") %
+                       old_master.id)
+                LOG.exception(msg)
                 exception_replicas.append(old_master)
+                error_messages += "%s (%s)\n" % (msg, ex)
 
             self._set_task_status([old_master] + replica_models,
                                   InstanceTasks.NONE)
             if exception_replicas:
                 self._set_task_status(exception_replicas,
                                       InstanceTasks.PROMOTION_ERROR)
-                msg = _("promote-to-replica-source %(id)s: The following "
-                        "replicas may not have been switched: %(replicas)s")
-                msg_values = {
-                    "id": master_candidate.id,
-                    "replicas": exception_replicas
-                }
-                raise ReplicationSlaveAttachError(msg % msg_values)
+                msg = (_("promote-to-replica-source %(id)s: The following "
+                         "replicas may not have been switched: %(replicas)s:"
+                         "\n%(err)s") %
+                       {"id": master_candidate.id,
+                        "replicas": [repl.id for repl in exception_replicas],
+                        "err": error_messages})
+                raise ReplicationSlaveAttachError(msg)
 
         with EndNotification(context):
             master_candidate = BuiltInstanceTasks.load(context, instance_id)
@@ -181,7 +185,7 @@ class Manager(periodic_task.PeriodicTasks):
     def _most_current_replica(self, old_master, replica_models):
         last_txns = self._get_replica_txns(replica_models)
         master_ids = [txn[1] for txn in last_txns if txn[1]]
-        if len(Set(master_ids)) > 1:
+        if len(set(master_ids)) > 1:
             raise TroveError(_("Replicas of %s not all replicating"
                                " from same master") % old_master.id)
         return sorted(last_txns, key=lambda x: x[2], reverse=True)[0][0]
@@ -202,35 +206,35 @@ class Manager(periodic_task.PeriodicTasks):
             old_master.attach_public_ips(slave_ips)
 
             exception_replicas = []
+            error_messages = ""
             for replica in replica_models:
                 try:
                     if replica.id != master_candidate.id:
                         replica.detach_replica(old_master, for_failover=True)
                         replica.attach_replica(master_candidate)
-                except exception.TroveError:
-                    msg = _("eject-replica-source: Unable to migrate "
-                            "replica %(slave)s from old replica source "
-                            "%(old_master)s to new source %(new_master)s.")
-                    msg_values = {
-                        "slave": replica.id,
-                        "old_master": old_master.id,
-                        "new_master": master_candidate.id
-                    }
-                    LOG.exception(msg % msg_values)
-                    exception_replicas.append(replica.id)
+                except exception.TroveError as ex:
+                    msg = (_("Unable to migrate replica %(slave)s from "
+                             "old replica source %(old_master)s to "
+                             "new source %(new_master)s on eject.") %
+                           {"slave": replica.id,
+                            "old_master": old_master.id,
+                            "new_master": master_candidate.id})
+                    LOG.exception(msg)
+                    exception_replicas.append(replica)
+                    error_messages += "%s (%s)\n" % (msg, ex)
 
             self._set_task_status([old_master] + replica_models,
                                   InstanceTasks.NONE)
             if exception_replicas:
                 self._set_task_status(exception_replicas,
                                       InstanceTasks.EJECTION_ERROR)
-                msg = _("eject-replica-source %(id)s: The following "
-                        "replicas may not have been switched: %(replicas)s")
-                msg_values = {
-                    "id": master_candidate.id,
-                    "replicas": exception_replicas
-                }
-                raise ReplicationSlaveAttachError(msg % msg_values)
+                msg = (_("eject-replica-source %(id)s: The following "
+                         "replicas may not have been switched: %(replicas)s:"
+                         "\n%(err)s") %
+                       {"id": master_candidate.id,
+                        "replicas": [repl.id for repl in exception_replicas],
+                        "err": error_messages})
+                raise ReplicationSlaveAttachError(msg)
 
         with EndNotification(context):
             master = BuiltInstanceTasks.load(context, instance_id)
@@ -290,6 +294,11 @@ class Manager(periodic_task.PeriodicTasks):
         replica_backup_created = False
         replicas = []
 
+        master_instance_tasks = BuiltInstanceTasks.load(context, slave_of_id)
+        server_group = master_instance_tasks.server_group
+        scheduler_hints = srv_grp.ServerGroup.convert_to_hint(server_group)
+        LOG.debug("Using scheduler hints for locality: %s" % scheduler_hints)
+
         try:
             for replica_index in range(0, len(ids)):
                 try:
@@ -308,7 +317,7 @@ class Manager(periodic_task.PeriodicTasks):
                         packages, volume_size, replica_backup_id,
                         availability_zone, root_passwords[replica_index],
                         nics, overrides, None, snapshot, volume_type,
-                        modules)
+                        modules, scheduler_hints)
                     replicas.append(instance_tasks)
                 except Exception:
                     # if it's the first replica, then we shouldn't continue
@@ -329,7 +338,7 @@ class Manager(periodic_task.PeriodicTasks):
                          image_id, databases, users, datastore_manager,
                          packages, volume_size, backup_id, availability_zone,
                          root_password, nics, overrides, slave_of_id,
-                         cluster_config, volume_type, modules):
+                         cluster_config, volume_type, modules, locality):
         if slave_of_id:
             self._create_replication_slave(context, instance_id, name,
                                            flavor, image_id, databases, users,
@@ -343,12 +352,16 @@ class Manager(periodic_task.PeriodicTasks):
                 raise AttributeError(_(
                     "Cannot create multiple non-replica instances."))
             instance_tasks = FreshInstanceTasks.load(context, instance_id)
+
+            scheduler_hints = srv_grp.ServerGroup.build_scheduler_hint(
+                context, locality, instance_id)
             instance_tasks.create_instance(flavor, image_id, databases, users,
                                            datastore_manager, packages,
                                            volume_size, backup_id,
                                            availability_zone, root_password,
                                            nics, overrides, cluster_config,
-                                           None, volume_type, modules)
+                                           None, volume_type, modules,
+                                           scheduler_hints)
             timeout = (CONF.restore_usage_timeout if backup_id
                        else CONF.usage_timeout)
             instance_tasks.wait_for_instance(timeout, flavor)
@@ -357,26 +370,24 @@ class Manager(periodic_task.PeriodicTasks):
                         image_id, databases, users, datastore_manager,
                         packages, volume_size, backup_id, availability_zone,
                         root_password, nics, overrides, slave_of_id,
-                        cluster_config, volume_type, modules):
+                        cluster_config, volume_type, modules, locality):
         with EndNotification(context,
                              instance_id=(instance_id[0]
-                                          if type(instance_id) is list
+                                          if isinstance(instance_id, list)
                                           else instance_id)):
             self._create_instance(context, instance_id, name, flavor,
                                   image_id, databases, users,
                                   datastore_manager, packages, volume_size,
                                   backup_id, availability_zone,
                                   root_password, nics, overrides, slave_of_id,
-                                  cluster_config, volume_type, modules)
+                                  cluster_config, volume_type, modules,
+                                  locality)
 
-    def update_overrides(self, context, instance_id, overrides):
+    def upgrade(self, context, instance_id, datastore_version_id):
         instance_tasks = models.BuiltInstanceTasks.load(context, instance_id)
-        instance_tasks.update_overrides(overrides)
-
-    def unassign_configuration(self, context, instance_id, flavor,
-                               configuration_id):
-        instance_tasks = models.BuiltInstanceTasks.load(context, instance_id)
-        instance_tasks.unassign_configuration(flavor, configuration_id)
+        datastore_version = DatastoreVersion.load_by_uuid(datastore_version_id)
+        with EndNotification(context):
+            instance_tasks.upgrade(datastore_version)
 
     def create_cluster(self, context, cluster_id):
         with EndNotification(context, cluster_id=cluster_id):
@@ -391,6 +402,15 @@ class Manager(periodic_task.PeriodicTasks):
         cluster_tasks = models.load_cluster_tasks(context, cluster_id)
         cluster_tasks.shrink_cluster(context, cluster_id, instance_ids)
 
+    def restart_cluster(self, context, cluster_id):
+        cluster_tasks = models.load_cluster_tasks(context, cluster_id)
+        cluster_tasks.restart_cluster(context, cluster_id)
+
+    def upgrade_cluster(self, context, cluster_id, datastore_version_id):
+        datastore_version = DatastoreVersion.load_by_uuid(datastore_version_id)
+        cluster_tasks = models.load_cluster_tasks(context, cluster_id)
+        cluster_tasks.upgrade_cluster(context, cluster_id, datastore_version)
+
     def delete_cluster(self, context, cluster_id):
         with EndNotification(context):
             cluster_tasks = models.load_cluster_tasks(context, cluster_id)
@@ -406,13 +426,14 @@ class Manager(periodic_task.PeriodicTasks):
             mgmtmodels.publish_exist_events(self.exists_transformer,
                                             self.admin_context)
 
-    @periodic_task.periodic_task(spacing=CONF.quota_notification_interval)
-    def publish_quota_notifications(self, context):
-        nova_client = remote.create_nova_client(self.admin_context)
-        for tenant in nova_client.tenants.list():
-            for quota in QUOTAS.get_all_quotas_by_tenant(tenant.id):
-                usage = QUOTAS.get_quota_usage(quota)
-                DBaaSQuotas(self.admin_context, quota, usage).notify()
+    if CONF.quota_notification_interval:
+        @periodic_task.periodic_task(spacing=CONF.quota_notification_interval)
+        def publish_quota_notifications(self, context):
+            nova_client = remote.create_nova_client(self.admin_context)
+            for tenant in nova_client.tenants.list():
+                for quota in QUOTAS.get_all_quotas_by_tenant(tenant.id):
+                    usage = QUOTAS.get_quota_usage(quota)
+                    DBaaSQuotas(self.admin_context, quota, usage).notify()
 
     def __getattr__(self, name):
         """
