@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import netaddr
 import os
 import re
 import time
@@ -33,6 +34,7 @@ from proboscis import before_class
 from proboscis.decorators import time_out
 from proboscis import SkipTest
 from proboscis import test
+import six
 from troveclient.compat import exceptions
 
 from trove.common import exception as rd_exceptions
@@ -85,7 +87,10 @@ class InstanceTestInfo(object):
         self.id = None  # The ID of the instance in the database.
         self.local_id = None
         self.address = None
-        self.nics = None  # The dict of type/id for nics used on the intance.
+        self.nics = None  # The dict of type/id for nics used on the instance.
+        shared_network = CONFIG.get('shared_network', None)
+        if shared_network:
+            self.nics = [{'net-id': shared_network}]
         self.initial_result = None  # The initial result from the create call.
         self.user_ip = None  # The IP address of the instance, given to user.
         self.infra_ip = None  # The infrastructure network IP address.
@@ -125,9 +130,14 @@ class InstanceTestInfo(object):
     def get_address(self):
         result = self.dbaas_admin.mgmt.instances.show(self.id)
         if not hasattr(result, 'hostname'):
-            return result.ip[0]
+            try:
+                return next(str(ip) for ip in result.ip
+                            if netaddr.valid_ipv4(ip))
+            except StopIteration:
+                fail("No IPV4 ip found")
         else:
-            return result.server['addresses']
+            return [str(ip) for ip in result.server['addresses']
+                    if netaddr.valid_ipv4(ip)]
 
     def get_local_id(self):
         mgmt_instance = self.dbaas_admin.management.show(self.id)
@@ -256,7 +266,8 @@ class CreateInstanceQuotaTest(unittest.TestCase):
                           dbaas.instances.create,
                           self.test_info.name,
                           self.test_info.dbaas_flavor_href,
-                          self.test_info.volume)
+                          self.test_info.volume,
+                          nics=instance_info.nics)
 
     def test_update_quota_invalid_resource_should_fail(self):
         quota_dict = {'invalid_resource': 100}
@@ -271,12 +282,15 @@ class CreateInstanceQuotaTest(unittest.TestCase):
                       self.test_info.user.tenant_id, quota_dict)
 
     def test_create_too_many_instances(self):
+        raise SkipTest(
+            "Skipping until https://review.openstack.org/#/c/331323 merges")
         instance_quota = 0
         quota_dict = {'instances': instance_quota}
         new_quotas = dbaas_admin.quota.update(self.test_info.user.tenant_id,
                                               quota_dict)
 
-        verify_quota = dbaas_admin.quota.show(self.test_info.user.tenant_id)
+        set_quota = dbaas_admin.quota.show(self.test_info.user.tenant_id)
+        verify_quota = {q.resource: q.limit for q in set_quota}
 
         assert_equal(new_quotas['instances'], quota_dict['instances'])
         assert_equal(0, verify_quota['instances'])
@@ -293,7 +307,8 @@ class CreateInstanceQuotaTest(unittest.TestCase):
                       dbaas.instances.create,
                       self.test_info.name,
                       self.test_info.dbaas_flavor_href,
-                      self.test_info.volume)
+                      self.test_info.volume,
+                      nics=instance_info.nics)
 
         assert_equal(413, dbaas.last_http_code)
 
@@ -312,7 +327,8 @@ class CreateInstanceQuotaTest(unittest.TestCase):
                       dbaas.instances.create,
                       self.test_info.name,
                       self.test_info.dbaas_flavor_href,
-                      self.test_info.volume)
+                      self.test_info.volume,
+                      nics=instance_info.nics)
 
         assert_equal(413, dbaas.last_http_code)
 
@@ -354,7 +370,8 @@ class CreateInstanceFail(object):
         result = dbaas.instances.create(instance_name,
                                         instance_info.dbaas_flavor_href,
                                         volume, databases,
-                                        availability_zone="BAD_ZONE")
+                                        availability_zone="BAD_ZONE",
+                                        nics=instance_info.nics)
 
         poll_until(self.instance_in_error(result.id))
         instance = dbaas.instances.get(result.id)
@@ -391,7 +408,8 @@ class CreateInstanceFail(object):
             volume = None
         assert_raises(exceptions.BadRequest, dbaas.instances.create,
                       instance_name, '',
-                      volume, databases)
+                      volume, databases,
+                      nics=instance_info.nics)
         assert_equal(400, dbaas.last_http_code)
 
     @test(enabled=VOLUME_SUPPORT)
@@ -401,7 +419,8 @@ class CreateInstanceFail(object):
         volume = {}
         assert_raises(exceptions.BadRequest, dbaas.instances.create,
                       instance_name, instance_info.dbaas_flavor_href,
-                      volume, databases)
+                      volume, databases,
+                      nics=instance_info.nics)
         assert_equal(400, dbaas.last_http_code)
 
     @test(enabled=VOLUME_SUPPORT)
@@ -411,7 +430,8 @@ class CreateInstanceFail(object):
         volume = {'size': None}
         assert_raises(exceptions.BadRequest, dbaas.instances.create,
                       instance_name, instance_info.dbaas_flavor_href,
-                      volume, databases)
+                      volume, databases,
+                      nics=instance_info.nics)
         assert_equal(400, dbaas.last_http_code)
 
     @test(enabled=not VOLUME_SUPPORT)
@@ -421,7 +441,8 @@ class CreateInstanceFail(object):
         volume = {'size': 2}
         assert_raises(exceptions.HTTPNotImplemented, dbaas.instances.create,
                       instance_name, instance_info.dbaas_flavor_href,
-                      volume, databases)
+                      volume, databases,
+                      nics=instance_info.nics)
         assert_equal(501, dbaas.last_http_code)
 
     def test_create_failure_with_volume_size_and_disabled_for_datastore(self):
@@ -432,7 +453,8 @@ class CreateInstanceFail(object):
         volume = {'size': 2}
         assert_raises(exceptions.HTTPNotImplemented, dbaas.instances.create,
                       instance_name, instance_info.dbaas_flavor_href,
-                      volume, databases, datastore=datastore)
+                      volume, databases, datastore=datastore,
+                      nics=instance_info.nics)
         assert_equal(501, dbaas.last_http_code)
 
     @test(enabled=EPHEMERAL_SUPPORT)
@@ -442,7 +464,8 @@ class CreateInstanceFail(object):
         flavor_name = CONFIG.values.get('instance_flavor_name', 'm1.tiny')
         flavors = dbaas.find_flavors_by_name(flavor_name)
         assert_raises(exceptions.BadRequest, dbaas.instances.create,
-                      instance_name, flavors[0].id, None, databases)
+                      instance_name, flavors[0].id, None, databases,
+                      nics=instance_info.nics)
         assert_equal(400, dbaas.last_http_code)
 
     @test
@@ -455,7 +478,8 @@ class CreateInstanceFail(object):
         databases = []
         assert_raises(exceptions.BadRequest, dbaas.instances.create,
                       instance_name, instance_info.dbaas_flavor_href,
-                      volume, databases)
+                      volume, databases,
+                      nics=instance_info.nics)
         assert_equal(400, dbaas.last_http_code)
 
     @test
@@ -468,7 +492,8 @@ class CreateInstanceFail(object):
         databases = []
         assert_raises(exceptions.BadRequest, dbaas.instances.create,
                       instance_name, instance_info.dbaas_flavor_href,
-                      volume, databases)
+                      volume, databases,
+                      nics=instance_info.nics)
         assert_equal(400, dbaas.last_http_code)
 
     @test
@@ -480,7 +505,7 @@ class CreateInstanceFail(object):
                              'hostname', 'id', 'name', 'datastore',
                              'server_state_description', 'status', 'updated',
                              'users', 'volume', 'root_enabled_at',
-                             'root_enabled_by']
+                             'root_enabled_by', 'fault']
             with CheckInstance(result._info) as check:
                 check.contains_allowed_attrs(
                     result._info, allowed_attrs,
@@ -490,7 +515,7 @@ class CreateInstanceFail(object):
                 check.guest_status()
 
     @test
-    def test_create_failure_with_datastore_default_notfound(self):
+    def test_create_failure_with_datastore_default_not_defined(self):
         if not FAKE:
             raise SkipTest("This test only for fake mode.")
         if VOLUME_SUPPORT:
@@ -507,11 +532,12 @@ class CreateInstanceFail(object):
             assert_raises(exceptions.NotFound,
                           dbaas.instances.create, instance_name,
                           instance_info.dbaas_flavor_href,
-                          volume, databases, users)
+                          volume, databases, users,
+                          nics=instance_info.nics)
         except exceptions.BadRequest as e:
             assert_equal(e.message,
-                         "Please specify datastore. Default datastore "
-                         "cannot be found.")
+                         "Please specify datastore. No default datastore "
+                         "is defined.")
         datastore_models.CONF.default_datastore = \
             origin_default_datastore
 
@@ -530,7 +556,8 @@ class CreateInstanceFail(object):
                           dbaas.instances.create, instance_name,
                           instance_info.dbaas_flavor_href,
                           volume, databases, users,
-                          datastore=datastore)
+                          datastore=datastore,
+                          nics=instance_info.nics)
         except exceptions.BadRequest as e:
             assert_equal(e.message,
                          "Default version for datastore '%s' not found." %
@@ -551,7 +578,8 @@ class CreateInstanceFail(object):
                           dbaas.instances.create, instance_name,
                           instance_info.dbaas_flavor_href,
                           volume, databases, users,
-                          datastore=datastore)
+                          datastore=datastore,
+                          nics=instance_info.nics)
         except exceptions.BadRequest as e:
             assert_equal(e.message,
                          "Datastore '%s' cannot be found." %
@@ -574,7 +602,8 @@ class CreateInstanceFail(object):
                           instance_info.dbaas_flavor_href,
                           volume, databases, users,
                           datastore=datastore,
-                          datastore_version=datastore_version)
+                          datastore_version=datastore_version,
+                          nics=instance_info.nics)
         except exceptions.BadRequest as e:
             assert_equal(e.message,
                          "Datastore version '%s' cannot be found." %
@@ -597,7 +626,8 @@ class CreateInstanceFail(object):
                           instance_info.dbaas_flavor_href,
                           volume, databases, users,
                           datastore=datastore,
-                          datastore_version=datastore_version)
+                          datastore_version=datastore_version,
+                          nics=instance_info.nics)
         except exceptions.BadRequest as e:
             assert_equal(e.message,
                          "Datastore version '%s' is not active." %
@@ -657,10 +687,6 @@ class CreateInstance(object):
         else:
             instance_info.volume = None
 
-        shared_network = CONFIG.get('shared_network', None)
-        if shared_network:
-            instance_info.nics = [{'net-id': shared_network}]
-
         if create_new_instance():
             instance_info.initial_result = dbaas.instances.create(
                 instance_info.name,
@@ -692,7 +718,8 @@ class CreateInstance(object):
 
         # Check these attrs only are returned in create response
         allowed_attrs = ['created', 'flavor', 'addresses', 'id', 'links',
-                         'name', 'status', 'updated', 'datastore']
+                         'name', 'status', 'updated', 'datastore', 'fault',
+                         'region']
         if ROOT_ON_CREATE:
             allowed_attrs.append('password')
         if VOLUME_SUPPORT:
@@ -749,7 +776,8 @@ class CreateInstanceFlavors(object):
         else:
             volume = None
         self.result = dbaas.instances.create(instance_name, flavor_id, volume,
-                                             databases)
+                                             databases,
+                                             nics=instance_info.nics)
         poll_until(self._result_is_active)
         self._delete_async(self.result.id)
 
@@ -788,7 +816,8 @@ class CreateInstanceWithNeutron(unittest.TestCase):
         self.result = self.dbaas_client.instances.create(
             self.instance_name,
             instance_info.dbaas_flavor_href,
-            volume, databases)
+            volume, databases,
+            nics=instance_info.nics)
         self.instance_id = self.result.id
 
         def verify_instance_is_active():
@@ -952,11 +981,11 @@ class SecurityGroupsTest(object):
     def test_created_security_group(self):
         assert_is_not_none(self.testSecurityGroup)
         with TypeCheck('SecurityGroup', self.testSecurityGroup) as secGrp:
-            secGrp.has_field('id', basestring)
-            secGrp.has_field('name', basestring)
-            secGrp.has_field('description', basestring)
-            secGrp.has_field('created', basestring)
-            secGrp.has_field('updated', basestring)
+            secGrp.has_field('id', six.string_types)
+            secGrp.has_field('name', six.string_types)
+            secGrp.has_field('description', six.string_types)
+            secGrp.has_field('created', six.string_types)
+            secGrp.has_field('updated', six.string_types)
         assert_equal(self.testSecurityGroup.name, self.secGroupName)
         assert_equal(self.testSecurityGroup.description,
                      self.secGroupDescription)
@@ -1134,7 +1163,8 @@ class TestInstanceListing(object):
     @test
     def test_index_list(self):
         allowed_attrs = ['id', 'links', 'name', 'status', 'flavor',
-                         'datastore', 'ip', 'hostname', 'replica_of']
+                         'datastore', 'ip', 'hostname', 'replica_of',
+                         'region']
         if VOLUME_SUPPORT:
             allowed_attrs.append('volume')
         instances = dbaas.instances.list()
@@ -1155,7 +1185,7 @@ class TestInstanceListing(object):
     def test_get_instance(self):
         allowed_attrs = ['created', 'databases', 'flavor', 'hostname', 'id',
                          'links', 'name', 'status', 'updated', 'ip',
-                         'datastore']
+                         'datastore', 'fault', 'region']
         if VOLUME_SUPPORT:
             allowed_attrs.append('volume')
         else:
@@ -1243,7 +1273,7 @@ class TestInstanceListing(object):
                          'flavor', 'guest_status', 'host', 'hostname', 'id',
                          'name', 'root_enabled_at', 'root_enabled_by',
                          'server_state_description', 'status', 'datastore',
-                         'updated', 'users', 'volume']
+                         'updated', 'users', 'volume', 'fault', 'region']
         with CheckInstance(result._info) as check:
             check.contains_allowed_attrs(
                 result._info, allowed_attrs,
@@ -1583,7 +1613,7 @@ class CheckInstance(AttrCheck):
             self.instance['volume'], allowed_attrs,
             msg="Volume")
 
-    def slave_of(self):
+    def replica_of(self):
         if 'replica_of' not in self.instance:
             self.fail("'replica_of' not found in instance.")
         else:
@@ -1606,7 +1636,7 @@ class CheckInstance(AttrCheck):
 
 
 @test(groups=[GROUP])
-class BadInstanceStatusBug():
+class BadInstanceStatusBug(object):
 
     @before_class()
     def setUp(self):
@@ -1638,7 +1668,8 @@ class BadInstanceStatusBug():
 
         result = self.client.instances.create('testbox',
                                               instance_info.dbaas_flavor_href,
-                                              size)
+                                              size,
+                                              nics=instance_info.nics)
         id = result.id
         self.instances.append(id)
 
