@@ -30,6 +30,8 @@ from trove.tests.api.instances import TIMEOUT_INSTANCE_CREATE
 from trove.tests.api.instances import TIMEOUT_INSTANCE_DELETE
 from trove.tests.api.instances import WaitForGuestInstallationToFinish
 from trove.tests.config import CONFIG
+from trove.tests.scenario import runners
+from trove.tests.scenario.runners.test_runners import SkipKnownBug
 from trove.tests.util.server_connection import create_server_connection
 
 
@@ -90,10 +92,10 @@ def create_slave():
         instance_info.name + "_slave",
         instance_info.dbaas_flavor_href,
         instance_info.volume,
-        nics=instance_info.nics,
         datastore=instance_info.dbaas_datastore,
         datastore_version=instance_info.dbaas_datastore_version,
-        slave_of=instance_info.id)
+        nics=instance_info.nics,
+        replica_of=instance_info.id)
     assert_equal(200, instance_info.dbaas.last_http_code)
     assert_equal("BUILD", result.status)
     return result.id
@@ -103,7 +105,7 @@ def validate_slave(master, slave):
     new_slave = instance_info.dbaas.instances.get(slave.id)
     assert_equal(200, instance_info.dbaas.last_http_code)
     ns_dict = new_slave._info
-    CheckInstance(ns_dict).slave_of()
+    CheckInstance(ns_dict).replica_of()
     assert_equal(master.id, ns_dict['replica_of']['id'])
 
 
@@ -130,7 +132,8 @@ class CreateReplicationSlave(object):
                       instance_info.volume,
                       datastore=instance_info.dbaas_datastore,
                       datastore_version=instance_info.dbaas_datastore_version,
-                      slave_of="Missing replica source")
+                      nics=instance_info.nics,
+                      replica_of="Missing replica source")
         assert_equal(404, instance_info.dbaas.last_http_code)
 
     @test
@@ -267,9 +270,11 @@ class TestReplicationFailover(object):
         if CONFIG.fake_mode:
             raise SkipTest("eject_replica_source not supported in fake mode")
 
-        assert_raises(exceptions.BadRequest,
-                      instance_info.dbaas.instances.eject_replica_source,
-                      instance_info.id)
+        # assert_raises(exceptions.BadRequest,
+        #               instance_info.dbaas.instances.eject_replica_source,
+        #               instance_info.id)
+        # Uncomment once BUG_EJECT_VALID_MASTER is fixed
+        raise SkipKnownBug(runners.BUG_EJECT_VALID_MASTER)
 
     @test(depends_on=[test_promote_master, test_eject_slave,
                       test_eject_valid_master])
