@@ -216,7 +216,8 @@ class BackupCreateTest(trove_testtools.TestCase):
                               self.context, self.instance_id,
                               BACKUP_NAME, BACKUP_DESC)
 
-    def test_create_backup_creation_error(self):
+    @patch('trove.backup.models.LOG')
+    def test_create_backup_creation_error(self, mock_logging):
         instance = MagicMock()
         instance.cluster_id = None
         with patch.object(instance_models.BuiltInstance, 'load',
