@@ -16,7 +16,7 @@ import hashlib
 import mock
 import os
 
-from mock import Mock, MagicMock, patch, ANY, DEFAULT
+from mock import Mock, MagicMock, patch, ANY, DEFAULT, call
 from oslo_utils import netutils
 from webob.exc import HTTPNotFound
 
@@ -106,7 +106,7 @@ class MockSwift(object):
         self.etag.update(self.store)
         return self.etag.hexdigest()
 
-    def save(self, filename, stream):
+    def save(self, filename, stream, metadata=None):
         location = '%s/%s/%s' % (self.url, self.container, filename)
         return True, 'w00t', 'fake-checksum', location
 
@@ -128,7 +128,7 @@ class MockStorage(Storage):
     def load(self, location, backup_checksum):
         pass
 
-    def save(self, filename, stream):
+    def save(self, filename, stream, metadata=None):
         pass
 
     def load_metadata(self, location, checksum):
@@ -159,7 +159,7 @@ class MockRestoreRunner(RestoreRunner):
         return False
 
 
-class MockStats:
+class MockStats(object):
     f_blocks = 1024 ** 2
     f_bsize = 4096
     f_bfree = 512 * 1024
@@ -230,7 +230,8 @@ class BackupAgentTest(trove_testtools.TestCase):
         self.assertIsNotNone(inno_backup_ex.cmd)
         str_innobackup_cmd = ('sudo innobackupex'
                               ' --stream=xbstream'
-                              ' %(extra_opts)s'
+                              ' %(extra_opts)s '
+                              ' --user=os_admin --password=123'
                               ' /var/lib/mysql/data 2>/tmp/innobackupex.log'
                               ' | gzip |'
                               ' openssl enc -aes-256-cbc -salt '
@@ -252,7 +253,9 @@ class BackupAgentTest(trove_testtools.TestCase):
         self.assertIsNotNone(cbbackup.manifest)
         self.assertIn('gz.enc', cbbackup.manifest)
 
-    def test_backup_impl_DB2Backup(self):
+    @mock.patch.object(db2_impl.DB2Backup, 'list_dbnames',
+                       return_value=['testdb1', 'testdb2'])
+    def test_backup_impl_DB2Backup(self, _):
         netutils.get_my_ipv4 = Mock(return_value="1.1.1.1")
         db2_backup = db2_impl.DB2Backup('db2backup', extra_opts='')
         self.assertIsNotNone(db2_backup)
@@ -329,33 +332,33 @@ class BackupAgentTest(trove_testtools.TestCase):
         agent.execute_backup(context=None, backup_info=backup_info,
                              runner=MockBackup)
 
-        self.assertTrue(
-            conductor_api.API.update_backup.called_once_with(
-                ANY,
-                backup_id=backup_info['id'],
-                state=BackupState.NEW))
-
-        self.assertTrue(
-            conductor_api.API.update_backup.called_once_with(
+        conductor_api.API.update_backup.assert_has_calls([
+            call(
                 ANY,
                 backup_id=backup_info['id'],
+                sent=ANY,
                 size=ANY,
-                state=BackupState.BUILDING))
-
-        self.assertTrue(
-            conductor_api.API.update_backup.called_once_with(
+                state=BackupState.BUILDING
+            ),
+            call(
                 ANY,
                 backup_id=backup_info['id'],
-                checksum=ANY,
+                checksum='fake-checksum',
                 location=ANY,
-                note=ANY,
-                backup_type=backup_info['type'],
-                state=BackupState.COMPLETED))
+                note='w00t',
+                sent=ANY,
+                size=ANY,
+                backup_type=MockBackup.backup_type,
+                state=BackupState.COMPLETED,
+                success=True
+            )
+        ])
 
     @patch.object(conductor_api.API, 'get_client', Mock(return_value=Mock()))
     @patch.object(conductor_api.API, 'update_backup',
                   Mock(return_value=Mock()))
-    def test_execute_bad_process_backup(self):
+    @patch('trove.guestagent.backup.backupagent.LOG')
+    def test_execute_bad_process_backup(self, mock_logging):
         agent = backupagent.BackupAgent()
         backup_info = {'id': '123',
                        'location': 'fake-location',
@@ -369,28 +372,27 @@ class BackupAgentTest(trove_testtools.TestCase):
                           context=None, backup_info=backup_info,
                           runner=MockCheckProcessBackup)
 
-        self.assertTrue(
-            conductor_api.API.update_backup.called_once_with(
-                ANY,
-                backup_id=backup_info['id'],
-                state=BackupState.NEW))
-
-        self.assertTrue(
-            conductor_api.API.update_backup.called_once_with(
+        conductor_api.API.update_backup.assert_has_calls([
+            call(
                 ANY,
                 backup_id=backup_info['id'],
+                sent=ANY,
                 size=ANY,
-                state=BackupState.BUILDING))
-
-        self.assertTrue(
-            conductor_api.API.update_backup.called_once_with(
+                state=BackupState.BUILDING
+            ),
+            call(
                 ANY,
                 backup_id=backup_info['id'],
-                checksum=ANY,
+                checksum='fake-checksum',
                 location=ANY,
-                note=ANY,
-                backup_type=backup_info['type'],
-                state=BackupState.FAILED))
+                note='w00t',
+                sent=ANY,
+                size=ANY,
+                backup_type=MockCheckProcessBackup.backup_type,
+                state=BackupState.FAILED,
+                success=True
+            )
+        ])
 
     @patch.object(conductor_api.API, 'get_client', Mock(return_value=Mock()))
     @patch.object(conductor_api.API, 'update_backup',
@@ -407,17 +409,34 @@ class BackupAgentTest(trove_testtools.TestCase):
                            'location': 'fake-location',
                            'type': 'InnoBackupEx',
                            'checksum': 'fake-checksum',
+                           'datastore': 'mysql',
+                           'datastore_version': '5.5'
                            }
 
             self.assertRaises(backupagent.BackupError, agent.execute_backup,
                               context=None, backup_info=backup_info,
                               runner=MockLossyBackup)
 
-            self.assertTrue(
-                conductor_api.API.update_backup.called_once_with(
+            conductor_api.API.update_backup.assert_has_calls([
+                call(ANY,
+                     backup_id=backup_info['id'],
+                     sent=ANY,
+                     size=ANY,
+                     state=BackupState.BUILDING
+                     ),
+                call(
                     ANY,
                     backup_id=backup_info['id'],
-                    state=BackupState.FAILED))
+                    checksum='y',
+                    location='z',
+                    note='Error',
+                    sent=ANY,
+                    size=ANY,
+                    backup_type=MockLossyBackup.backup_type,
+                    state=BackupState.FAILED,
+                    success=False
+                )]
+            )
 
     def test_execute_restore(self):
         """This test should ensure backup agent
@@ -463,12 +482,12 @@ class BackupAgentTest(trove_testtools.TestCase):
     @patch.object(MySqlApp, 'get_data_dir', return_value='/var/lib/mysql/data')
     @patch.object(conductor_api.API, 'get_client', Mock(return_value=Mock()))
     @patch.object(MockSwift, 'load_metadata', return_value={'lsn': '54321'})
-    @patch.object(MockStorage, 'save_metadata')
+    @patch.object(MockSwift, 'save')
     @patch.object(backupagent, 'get_storage_strategy', return_value=MockSwift)
     @patch('trove.guestagent.backup.backupagent.LOG')
     def test_backup_incremental_metadata(self, mock_logging,
                                          get_storage_strategy_mock,
-                                         save_metadata_mock,
+                                         save_mock,
                                          load_metadata_mock,
                                          get_datadir_mock):
         meta = {
@@ -482,25 +501,25 @@ class BackupAgentTest(trove_testtools.TestCase):
                             __exit__=MagicMock(return_value=True)):
             agent = backupagent.BackupAgent()
 
+            expected_metadata = {'datastore': 'mysql',
+                                 'datastore_version': 'bo.gus'}
             bkup_info = {'id': '123',
                          'location': 'fake-location',
                          'type': 'InnoBackupEx',
                          'checksum': 'fake-checksum',
-                         'parent': {'location': 'fake', 'checksum': 'md5'},
-                         'datastore': 'mysql',
-                         'datastore_version': 'bo.gus'
-                         }
+                         'parent': {'location': 'fake', 'checksum': 'md5'}}
+            bkup_info.update(expected_metadata)
 
             agent.execute_backup(TroveContext(),
                                  bkup_info,
                                  '/var/lib/mysql/data')
 
-            self.assertTrue(MockStorage.save_metadata.called_once_with(
-                            ANY,
-                            meta))
+            save_mock.assert_called_once_with(
+                ANY, ANY, metadata=expected_metadata)
 
     @patch.object(conductor_api.API, 'get_client', Mock(return_value=Mock()))
-    def test_backup_incremental_bad_metadata(self):
+    @patch('trove.guestagent.backup.backupagent.LOG')
+    def test_backup_incremental_bad_metadata(self, mock_logging):
         with patch.object(backupagent, 'get_storage_strategy',
                           return_value=MockSwift):
 
