@@ -18,7 +18,6 @@ import jsonschema
 from mock import MagicMock
 from mock import Mock
 from mock import patch
-from testtools import TestCase
 from testtools.matchers import Is, Equals
 from trove.cluster import models
 from trove.cluster.models import Cluster, DBCluster
@@ -33,10 +32,12 @@ from trove.datastore import models as datastore_models
 from trove.tests.unittests import trove_testtools
 
 
-class TestClusterController(TestCase):
+class TestClusterController(trove_testtools.TestCase):
+
     def setUp(self):
         super(TestClusterController, self).setUp()
         self.controller = ClusterController()
+        self.locality = 'anti-affinity'
         instances = [
             {
                 "flavorRef": "7",
@@ -57,7 +58,8 @@ class TestClusterController(TestCase):
                     "type": "mongodb",
                     "version": "2.4.10"
                 },
-                "instances": instances
+                "instances": instances,
+                "locality": self.locality,
             }
         }
         self.add_shard = {
@@ -67,13 +69,13 @@ class TestClusterController(TestCase):
     def test_get_schema_create(self):
         schema = self.controller.get_schema('create', self.cluster)
         self.assertIsNotNone(schema)
-        self.assertTrue('cluster' in schema['properties'])
+        self.assertIn('cluster', schema['properties'])
         self.assertTrue('cluster')
 
     def test_get_schema_action_add_shard(self):
         schema = self.controller.get_schema('add_shard', self.add_shard)
         self.assertIsNotNone(schema)
-        self.assertTrue('add_shard' in schema['properties'])
+        self.assertIn('add_shard', schema['properties'])
 
     def test_validate_create(self):
         body = self.cluster
@@ -112,6 +114,20 @@ class TestClusterController(TestCase):
         self.assertIn("'' does not match '^.*[0-9a-zA-Z]+.*$'", error_messages)
         self.assertIn("type", error_paths)
 
+    def test_validate_create_bad_locality(self):
+        body = self.cluster
+        body['cluster']['locality'] = "$%^&"
+        schema = self.controller.get_schema('create', body)
+        validator = jsonschema.Draft4Validator(schema)
+        self.assertFalse(validator.is_valid(body))
+        errors = sorted(validator.iter_errors(body), key=lambda e: e.path)
+        error_messages = [error.message for error in errors]
+        error_paths = [error.path.pop() for error in errors]
+        self.assertThat(len(errors), Is(1))
+        self.assertIn("'$%^&' does not match '^.*[0-9a-zA-Z]+.*$'",
+                      error_messages)
+        self.assertIn("locality", error_paths)
+
     @patch.object(Cluster, 'create')
     @patch.object(datastore_models, 'get_datastore_version')
     def test_create_clusters_disabled(self,
@@ -160,6 +176,8 @@ class TestClusterController(TestCase):
                 'volume_type': None,
                 'flavor_id': '1234',
                 'availability_zone': 'az',
+                'modules': None,
+                'region_name': None,
                 'nics': [
                     {'net-id': 'e89aa5fd-6b0a-436d-a75c-1545d34d5331'}
                 ]
@@ -176,7 +194,8 @@ class TestClusterController(TestCase):
         self.controller.create(req, body, tenant_id)
         mock_cluster_create.assert_called_with(context, 'products',
                                                datastore, datastore_version,
-                                               instances, {})
+                                               instances, {},
+                                               self.locality, None)
 
     @patch.object(Cluster, 'load')
     def test_show_cluster(self,
@@ -193,6 +212,7 @@ class TestClusterController(TestCase):
         mock_cluster.instances_without_server = []
         mock_cluster.datastore_version.manager = 'mongodb'
         mock_cluster_load.return_value = mock_cluster
+        mock_cluster.locality = self.locality
 
         self.controller.show(req, tenant_id, id)
         mock_cluster_load.assert_called_with(context, id)
@@ -228,7 +248,8 @@ class TestClusterController(TestCase):
         cluster.delete.assert_called_with()
 
 
-class TestClusterControllerWithStrategy(TestCase):
+class TestClusterControllerWithStrategy(trove_testtools.TestCase):
+
     def setUp(self):
         super(TestClusterControllerWithStrategy, self).setUp()
         self.controller = ClusterController()
