@@ -73,7 +73,7 @@ class TestClusterController(trove_testtools.TestCase):
     def test_get_schema_create(self):
         schema = self.controller.get_schema('create', self.cluster)
         self.assertIsNotNone(schema)
-        self.assertTrue('cluster' in schema['properties'])
+        self.assertIn('cluster', schema['properties'])
         self.assertTrue('cluster')
 
     def test_validate_create(self):
@@ -157,6 +157,7 @@ class TestClusterController(trove_testtools.TestCase):
                 "flavor_id": "1234",
                 "availability_zone": "az",
                 'modules': None,
+                'region_name': None,
                 "nics": [
                     {"net-id": "e89aa5fd-6b0a-436d-a75c-1545d34d5331"}
                 ]
@@ -167,6 +168,7 @@ class TestClusterController(trove_testtools.TestCase):
                 "flavor_id": "1234",
                 "availability_zone": "az",
                 'modules': None,
+                'region_name': None,
                 "nics": [
                     {"net-id": "e89aa5fd-6b0a-436d-a75c-1545d34d5331"}
                 ]
@@ -177,6 +179,7 @@ class TestClusterController(trove_testtools.TestCase):
                 "flavor_id": "1234",
                 "availability_zone": "az",
                 'modules': None,
+                'region_name': None,
                 "nics": [
                     {"net-id": "e89aa5fd-6b0a-436d-a75c-1545d34d5331"}
                 ]
@@ -193,7 +196,7 @@ class TestClusterController(trove_testtools.TestCase):
         self.controller.create(req, body, tenant_id)
         mock_cluster_create.assert_called_with(context, 'products',
                                                datastore, datastore_version,
-                                               instances, {}, None)
+                                               instances, {}, None, None)
 
     @patch.object(Cluster, 'load')
     def test_show_cluster(self,
