@@ -11,7 +11,9 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+
 import mock
+import os
 from mock import ANY, DEFAULT, Mock, patch, PropertyMock
 from testtools.testcase import ExpectedException
 from trove.common import exception
@@ -22,7 +24,12 @@ from trove.guestagent.common import operating_system
 from trove.guestagent.datastore.experimental.cassandra import (
     service as cass_service
 )
+from trove.guestagent.datastore.experimental.db2 import (
+    service as db2_service)
 from trove.guestagent.strategies.backup import base as backupBase
+from trove.guestagent.strategies.backup.experimental import db2_impl
+from trove.guestagent.strategies.backup.experimental.postgresql_impl \
+    import PgBaseBackupUtil
 from trove.guestagent.strategies.backup.mysql_impl import MySqlApp
 from trove.guestagent.strategies.restore import base as restoreBase
 from trove.guestagent.strategies.restore.mysql_impl import MySQLRestoreMixin
@@ -70,13 +77,16 @@ ZIP = "gzip"
 UNZIP = "gzip -d -c"
 ENCRYPT = "openssl enc -aes-256-cbc -salt -pass pass:default_aes_cbc_key"
 DECRYPT = "openssl enc -d -aes-256-cbc -salt -pass pass:default_aes_cbc_key"
-XTRA_BACKUP_RAW = ("sudo innobackupex --stream=xbstream %(extra_opts)s"
+XTRA_BACKUP_RAW = ("sudo innobackupex --stream=xbstream %(extra_opts)s "
+                   " --user=os_admin --password=password"
                    " /var/lib/mysql/data 2>/tmp/innobackupex.log")
 XTRA_BACKUP = XTRA_BACKUP_RAW % {'extra_opts': ''}
 XTRA_BACKUP_EXTRA_OPTS = XTRA_BACKUP_RAW % {'extra_opts': '--no-lock'}
 XTRA_BACKUP_INCR = ('sudo innobackupex --stream=xbstream'
                     ' --incremental --incremental-lsn=%(lsn)s'
-                    ' %(extra_opts)s /var/lib/mysql/data'
+                    ' %(extra_opts)s '
+                    ' --user=os_admin --password=password'
+                    ' /var/lib/mysql/data'
                     ' 2>/tmp/innobackupex.log')
 SQLDUMP_BACKUP_RAW = ("mysqldump --all-databases %(extra_opts)s "
                       "--opt --password=password -u os_admin"
@@ -439,7 +449,13 @@ class GuestAgentBackupTest(trove_testtools.TestCase):
                          DECRYPT + PIPE + UNZIP + PIPE + REDISBACKUP_RESTORE)
 
     @patch.object(utils, 'execute_with_timeout')
-    def test_backup_encrypted_db2backup_command(self, *mock):
+    @patch.object(ImportOverrideStrategy, '_initialize_import_directory')
+    @patch.multiple(operating_system, exists=DEFAULT, write_file=DEFAULT,
+                    chown=DEFAULT, chmod=DEFAULT)
+    @patch.object(db2_service, 'run_command')
+    @patch.object(db2_service.DB2App, 'process_default_dbm_config')
+    @patch.object(db2_impl.DB2Backup, 'list_dbnames')
+    def test_backup_encrypted_db2backup_command(self, *mock, **kwargs):
         backupBase.BackupRunner.is_encrypted = True
         backupBase.BackupRunner.encrypt_key = CRYPTO_KEY
         RunnerClass = utils.import_class(BACKUP_DB2_CLS)
@@ -450,7 +466,13 @@ class GuestAgentBackupTest(trove_testtools.TestCase):
         self.assertIn("gz.enc", bkp.manifest)
 
     @patch.object(utils, 'execute_with_timeout')
-    def test_backup_not_encrypted_db2backup_command(self, *mock):
+    @patch.object(ImportOverrideStrategy, '_initialize_import_directory')
+    @patch.multiple(operating_system, exists=DEFAULT, write_file=DEFAULT,
+                    chown=DEFAULT, chmod=DEFAULT)
+    @patch.object(db2_service, 'run_command')
+    @patch.object(db2_service.DB2App, 'process_default_dbm_config')
+    @patch.object(db2_impl.DB2Backup, 'list_dbnames')
+    def test_backup_not_encrypted_db2backup_command(self, *mock, **kwargs):
         backupBase.BackupRunner.is_encrypted = False
         backupBase.BackupRunner.encrypt_key = CRYPTO_KEY
         RunnerClass = utils.import_class(BACKUP_DB2_CLS)
@@ -459,7 +481,12 @@ class GuestAgentBackupTest(trove_testtools.TestCase):
         self.assertEqual(DB2BACKUP_CMD + PIPE + ZIP, bkp.command)
         self.assertIn("gz", bkp.manifest)
 
-    def test_restore_decrypted_db2backup_command(self):
+    @patch.object(ImportOverrideStrategy, '_initialize_import_directory')
+    @patch.multiple(operating_system, exists=DEFAULT, write_file=DEFAULT,
+                    chown=DEFAULT, chmod=DEFAULT)
+    @patch.object(db2_service, 'run_command')
+    @patch.object(db2_service.DB2App, 'process_default_dbm_config')
+    def test_restore_decrypted_db2backup_command(self, *args, **kwargs):
         restoreBase.RestoreRunner.is_zipped = True
         restoreBase.RestoreRunner.is_encrypted = False
         RunnerClass = utils.import_class(RESTORE_DB2_CLS)
@@ -467,7 +494,12 @@ class GuestAgentBackupTest(trove_testtools.TestCase):
                             location="filename", checksum="md5")
         self.assertEqual(restr.restore_cmd, UNZIP + PIPE + DB2BACKUP_RESTORE)
 
-    def test_restore_encrypted_db2backup_command(self):
+    @patch.object(ImportOverrideStrategy, '_initialize_import_directory')
+    @patch.multiple(operating_system, exists=DEFAULT, write_file=DEFAULT,
+                    chown=DEFAULT, chmod=DEFAULT)
+    @patch.object(db2_service, 'run_command')
+    @patch.object(db2_service.DB2App, 'process_default_dbm_config')
+    def test_restore_encrypted_db2backup_command(self, *args, **kwargs):
         restoreBase.RestoreRunner.is_zipped = True
         restoreBase.RestoreRunner.is_encrypted = True
         restoreBase.RestoreRunner.encrypt_key = CRYPTO_KEY
@@ -890,12 +922,70 @@ class RedisRestoreTests(trove_testtools.TestCase):
                           self.restore_runner.restore)
 
 
-class DB2BackupTests(trove_testtools.TestCase):
+class PostgresqlBackupTests(trove_testtools.TestCase):
 
     def setUp(self):
+        super(PostgresqlBackupTests, self).setUp()
+        self.bkutil = PgBaseBackupUtil()
+
+        self.b1 = ['000000010000000000000003',
+                   '000000010000000000000004',
+                   '000000010000000000000005',
+                   '000000010000000000000006',
+                   '000000010000000000000006.00000168.backup']
+
+        self.b2 = ['000000010000000000000007',
+                   '000000010000000000000008',
+                   '000000010000000000000009',
+                   '000000010000000000000010',
+                   '000000010000000000000009.0008A168.backup']
+
+    def tearDown(self):
+        super(PostgresqlBackupTests, self).tearDown()
+
+    def test_check_most_recent_backup(self):
+
+        with patch.object(os, 'listdir', return_value=self.b1):
+            mrb = self.bkutil.most_recent_backup_file()
+            self.assertEqual(mrb, self.b1[4])
+            mrbfile = self.bkutil.most_recent_backup_wal()
+            self.assertEqual(mrbfile, self.b1[3])
+
+        with patch.object(os, 'listdir', return_value=self.b1 + self.b2):
+            mrb = self.bkutil.most_recent_backup_file()
+            self.assertEqual(mrb, self.b2[4])
+            mrbfile = self.bkutil.most_recent_backup_wal()
+            self.assertEqual(mrbfile, self.b2[2])
+
+    def test_check_most_recent_wal_list(self):
+
+        with patch.object(os, 'listdir', return_value=self.b1):
+            logs = self.bkutil.log_files_since_last_backup()
+            self.assertEqual(logs, [self.b1[3]])
+
+        with patch.object(os, 'listdir', return_value=self.b2):
+            logs = self.bkutil.log_files_since_last_backup()
+            self.assertEqual(logs, [self.b2[2], self.b2[3]])
+
+        with patch.object(os, 'listdir', return_value=self.b1 + self.b2):
+            logs = self.bkutil.log_files_since_last_backup()
+            self.assertEqual(logs, [self.b2[2], self.b2[3]])
+
+
+class DB2BackupTests(trove_testtools.TestCase):
+
+    @patch.object(ImportOverrideStrategy, '_initialize_import_directory')
+    @patch.multiple(operating_system, exists=DEFAULT, write_file=DEFAULT,
+                    chown=DEFAULT, chmod=DEFAULT)
+    @patch.object(db2_service, 'run_command')
+    @patch.object(db2_service.DB2App, 'process_default_dbm_config')
+    def setUp(self, *args, **kwargs):
         super(DB2BackupTests, self).setUp()
         self.exec_timeout_patch = patch.object(utils, 'execute_with_timeout')
         self.exec_timeout_patch.start()
+        self.exec_list_database = patch.object(db2_impl.DB2Backup,
+                                               'list_dbnames')
+        self.exec_list_database.start()
         self.backup_runner = utils.import_class(BACKUP_DB2_CLS)
         self.backup_runner_patch = patch.multiple(
             self.backup_runner, _run=DEFAULT,
@@ -904,6 +994,7 @@ class DB2BackupTests(trove_testtools.TestCase):
     def tearDown(self):
         super(DB2BackupTests, self).tearDown()
         self.backup_runner_patch.stop()
+        self.exec_list_database.stop()
         self.exec_timeout_patch.stop()
 
     def test_backup_success(self):
@@ -929,7 +1020,12 @@ class DB2BackupTests(trove_testtools.TestCase):
 
 class DB2RestoreTests(trove_testtools.TestCase):
 
-    def setUp(self):
+    @patch.object(ImportOverrideStrategy, '_initialize_import_directory')
+    @patch.multiple(operating_system, exists=DEFAULT, write_file=DEFAULT,
+                    chown=DEFAULT, chmod=DEFAULT)
+    @patch.object(db2_service, 'run_command')
+    @patch.object(db2_service.DB2App, 'process_default_dbm_config')
+    def setUp(self, *args, **kwargs):
         super(DB2RestoreTests, self).setUp()
 
         self.restore_runner = utils.import_class(
