@@ -26,6 +26,7 @@ from mock import patch
 from oslo_utils import netutils
 from testtools import ExpectedException
 
+from trove.common.db.cassandra import models
 from trove.common import exception
 from trove.common.instance import ServiceStatuses
 from trove.guestagent import backup
@@ -35,13 +36,14 @@ from trove.guestagent.datastore.experimental.cassandra import (
     manager as cass_manager)
 from trove.guestagent.datastore.experimental.cassandra import (
     service as cass_service)
-from trove.guestagent.db import models
 from trove.guestagent import pkg as pkg
 from trove.guestagent import volume
+from trove.tests.unittests.guestagent.test_datastore_manager import \
+    DatastoreManagerTest
 from trove.tests.unittests import trove_testtools
 
 
-class GuestAgentCassandraDBManagerTest(trove_testtools.TestCase):
+class GuestAgentCassandraDBManagerTest(DatastoreManagerTest):
 
     __MOUNT_POINT = '/var/lib/cassandra'
 
@@ -50,7 +52,7 @@ class GuestAgentCassandraDBManagerTest(trove_testtools.TestCase):
     __N_BU = '_build_user'
     __N_RU = '_rename_user'
     __N_AUP = '_alter_user_password'
-    __N_CAU = 'trove.guestagent.db.models.CassandraUser'
+    __N_CAU = 'trove.common.db.cassandra.models.CassandraUser'
     __N_CU = '_create_user'
     __N_GFA = '_grant_full_access_on_keyspace'
     __N_DU = '_drop_user'
@@ -74,7 +76,7 @@ class GuestAgentCassandraDBManagerTest(trove_testtools.TestCase):
     @patch.object(ImportOverrideStrategy, '_initialize_import_directory')
     @patch('trove.guestagent.datastore.experimental.cassandra.service.LOG')
     def setUp(self, *args, **kwargs):
-        super(GuestAgentCassandraDBManagerTest, self).setUp()
+        super(GuestAgentCassandraDBManagerTest, self).setUp('cassandra')
 
         conn_patcher = patch.multiple(cass_service.CassandraConnection,
                                       _connect=DEFAULT,
@@ -334,7 +336,8 @@ class GuestAgentCassandraDBManagerTest(trove_testtools.TestCase):
         return [item.serialize() for item in collection]
 
     @classmethod
-    def _get_random_name(self, size, chars=string.letters + string.digits):
+    def _get_random_name(self, size,
+                         chars=string.ascii_letters + string.digits):
         return ''.join(random.choice(chars) for _ in range(size))
 
     def test_create_database(self):
@@ -452,7 +455,7 @@ class GuestAgentCassandraDBManagerTest(trove_testtools.TestCase):
             found = self.manager.list_databases(self.context)
             self.assertEqual(2, len(found))
             self.assertEqual(3, len(found[0]))
-            self.assertEqual(None, found[1])
+            self.assertIsNone(found[1])
             self.assertIn(db1.serialize(), found[0])
             self.assertIn(db2.serialize(), found[0])
             self.assertIn(db3.serialize(), found[0])
@@ -603,7 +606,7 @@ class GuestAgentCassandraDBManagerTest(trove_testtools.TestCase):
             found = self.manager.list_users(self.context)
             self.assertEqual(2, len(found))
             self.assertEqual(3, len(found[0]))
-            self.assertEqual(None, found[1])
+            self.assertIsNone(found[1])
             self.assertIn(usr1.serialize(), found[0])
             self.assertIn(usr2.serialize(), found[0])
             self.assertIn(usr3.serialize(), found[0])
@@ -733,7 +736,8 @@ class GuestAgentCassandraDBManagerTest(trove_testtools.TestCase):
         self.assertIsNone(
             self.manager.apply_overrides(Mock(), NonCallableMagicMock()))
 
-    def test_enable_root(self):
+    @patch('trove.guestagent.datastore.experimental.cassandra.service.LOG')
+    def test_enable_root(self, _):
         with patch.object(self.manager._app, 'is_root_enabled',
                           return_value=False):
             with patch.object(cass_service.CassandraAdmin,
@@ -748,7 +752,8 @@ class GuestAgentCassandraDBManagerTest(trove_testtools.TestCase):
                 self.manager.enable_root(self.context)
                 alter_mock.assert_called_once_with(ANY)
 
-    def test_is_root_enabled(self):
+    @patch('trove.guestagent.datastore.experimental.cassandra.service.LOG')
+    def test_is_root_enabled(self, _):
         trove_admin = Mock()
         trove_admin.configure_mock(name=self.manager._app._ADMIN_USER)
         other_admin = Mock()
@@ -770,3 +775,22 @@ class GuestAgentCassandraDBManagerTest(trove_testtools.TestCase):
                           'list_superusers',
                           return_value=[trove_admin, other_admin]):
             self.assertTrue(self.manager.is_root_enabled(self.context))
+
+    def test_guest_log_enable(self):
+        self._assert_guest_log_enable(False, 'INFO')
+        self._assert_guest_log_enable(True, 'OFF')
+
+    def _assert_guest_log_enable(self, disable, expected_level):
+        with patch.multiple(
+                self.manager._app,
+                logback_conf_manager=DEFAULT,
+                _run_nodetool_command=DEFAULT
+        ) as app_mocks:
+            self.assertFalse(self.manager.guest_log_enable(
+                Mock(), Mock(), disable))
+
+            (app_mocks['logback_conf_manager'].apply_system_override.
+             assert_called_once_with(
+                {'configuration': {'root': {'@level': expected_level}}}))
+            app_mocks['_run_nodetool_command'].assert_called_once_with(
+                'setlogginglevel', 'root', expected_level)
