@@ -120,7 +120,7 @@ class QuotaControllerTest(trove_testtools.TestCase):
             result = self.controller.update(self.req, body, FAKE_TENANT1,
                                             FAKE_TENANT2)
             self.assertEqual(1, instance_quota.save.call_count)
-            self.assertTrue('instances' in result._data['quotas'])
+            self.assertIn('instances', result._data['quotas'])
             self.assertEqual(200, result.status)
             self.assertEqual(2, result._data['quotas']['instances'])
 
@@ -139,7 +139,7 @@ class QuotaControllerTest(trove_testtools.TestCase):
             result = self.controller.update(self.req, body, FAKE_TENANT1,
                                             FAKE_TENANT2)
             self.assertEqual(0, instance_quota.save.call_count)
-            self.assertFalse('instances' in result._data['quotas'])
+            self.assertNotIn('instances', result._data['quotas'])
             self.assertEqual(1, volume_quota.save.call_count)
             self.assertEqual(200, result.status)
             self.assertEqual(10, result._data['quotas']['volumes'])
@@ -461,7 +461,8 @@ class DbQuotaDriverTest(trove_testtools.TestCase):
         self.mock_quota_result.all = Mock(return_value=[])
         self.mock_usage_result.all = Mock(return_value=FAKE_QUOTAS)
 
-        delta = {'instances': 5, 'volumes': 3}
+        max_inst = CONF.max_instances_per_tenant
+        delta = {'instances': max_inst, 'volumes': 3}
         self.assertRaises(exception.QuotaExceeded,
                           self.driver.reserve,
                           FAKE_TENANT1,
@@ -484,7 +485,8 @@ class DbQuotaDriverTest(trove_testtools.TestCase):
         self.mock_quota_result.all = Mock(return_value=[])
         self.mock_usage_result.all = Mock(return_value=FAKE_QUOTAS)
 
-        delta = {'instances': 4, 'volumes': 2}
+        max_inst = CONF.max_instances_per_tenant
+        delta = {'instances': max_inst - 1, 'volumes': 2}
         self.assertRaises(exception.QuotaExceeded,
                           self.driver.reserve,
                           FAKE_TENANT1,
