@@ -18,7 +18,8 @@ import uuid
 
 from cinderclient import exceptions as cinder_exceptions
 import cinderclient.v2.client as cinderclient
-from mock import Mock, MagicMock, patch, PropertyMock
+from cinderclient.v2 import volumes as cinderclient_volumes
+from mock import Mock, MagicMock, patch, PropertyMock, call
 from novaclient import exceptions as nova_exceptions
 import novaclient.v2.flavors
 import novaclient.v2.servers
@@ -35,11 +36,13 @@ from trove.common.exception import MalformedSecurityGroupRuleError
 from trove.common.exception import PollTimeOut
 from trove.common.exception import TroveError
 from trove.common.instance import ServiceStatuses
+from trove.common.notification import TroveInstanceModifyVolume
 from trove.common import remote
 import trove.common.template as template
 from trove.common import utils
 from trove.datastore import models as datastore_models
 import trove.db.models
+from trove.extensions.common import models as common_models
 from trove.extensions.mysql import models as mysql_models
 import trove.guestagent.api
 from trove.instance.models import BaseInstance
@@ -58,12 +61,13 @@ VOLUME_ID = 'volume-id-1'
 
 class FakeOptGroup(object):
     def __init__(self, tcp_ports=['3306', '3301-3307'],
-                 udp_ports=[]):
+                 udp_ports=[], icmp=False):
         self.tcp_ports = tcp_ports
         self.udp_ports = udp_ports
+        self.icmp = icmp
 
 
-class fake_Server:
+class fake_Server(object):
     def __init__(self):
         self.id = None
         self.name = None
@@ -76,10 +80,11 @@ class fake_Server:
         self.status = 'ACTIVE'
 
 
-class fake_ServerManager:
+class fake_ServerManager(object):
     def create(self, name, image_id, flavor_id, files, userdata,
                security_groups, block_device_mapping, availability_zone=None,
-               nics=None, config_drive=False):
+               nics=None, config_drive=False,
+               scheduler_hints=None):
         server = fake_Server()
         server.id = "server_id"
         server.name = name
@@ -94,7 +99,7 @@ class fake_ServerManager:
         return server
 
 
-class fake_nova_client:
+class fake_nova_client(object):
     def __init__(self):
         self.servers = fake_ServerManager()
 
@@ -191,10 +196,11 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
         self.orig_DBI_find_by = DBInstance.find_by
         self.userdata = "hello moto"
         self.guestconfig_content = "guest config"
-        with NamedTemporaryFile(suffix=".cloudinit", delete=False) as f:
+        with NamedTemporaryFile(mode="w", suffix=".cloudinit",
+                                delete=False) as f:
             self.cloudinit = f.name
             f.write(self.userdata)
-        with NamedTemporaryFile(delete=False) as f:
+        with NamedTemporaryFile(mode="w", delete=False) as f:
             self.guestconfig = f.name
             f.write(self.guestconfig_content)
         self.freshinstancetasks = taskmanager_models.FreshInstanceTasks(
@@ -212,6 +218,9 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
         self.task_models_conf_patch = patch('trove.taskmanager.models.CONF')
         self.task_models_conf_mock = self.task_models_conf_patch.start()
         self.addCleanup(self.task_models_conf_patch.stop)
+        self.inst_models_conf_patch = patch('trove.instance.models.CONF')
+        self.inst_models_conf_mock = self.inst_models_conf_patch.start()
+        self.addCleanup(self.inst_models_conf_patch.stop)
 
     def tearDown(self):
         super(FreshInstanceTasksTest, self).tearDown()
@@ -236,7 +245,8 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
             None, None, None, datastore_manager, None, None, None)
         self.assertEqual(server.userdata, self.userdata)
 
-    def test_create_instance_guestconfig(self):
+    @patch.object(DBInstance, 'get_by')
+    def test_create_instance_guestconfig(self, patch_get_by):
         def fake_conf_getter(*args, **kwargs):
             if args[0] == 'guest_config':
                 return self.guestconfig
@@ -247,9 +257,9 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
             else:
                 return ''
 
-        self.task_models_conf_mock.get.side_effect = fake_conf_getter
+        self.inst_models_conf_mock.get.side_effect = fake_conf_getter
         # execute
-        files = self.freshinstancetasks._get_injected_files("test")
+        files = self.freshinstancetasks.get_injected_files("test")
         # verify
         self.assertTrue(
             '/etc/trove/conf.d/guest_info.conf' in files)
@@ -259,7 +269,8 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
             self.guestconfig_content,
             files['/etc/trove/conf.d/trove-guestagent.conf'])
 
-    def test_create_instance_guestconfig_compat(self):
+    @patch.object(DBInstance, 'get_by')
+    def test_create_instance_guestconfig_compat(self, patch_get_by):
         def fake_conf_getter(*args, **kwargs):
             if args[0] == 'guest_config':
                 return self.guestconfig
@@ -270,9 +281,9 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
             else:
                 return ''
 
-        self.task_models_conf_mock.get.side_effect = fake_conf_getter
+        self.inst_models_conf_mock.get.side_effect = fake_conf_getter
         # execute
-        files = self.freshinstancetasks._get_injected_files("test")
+        files = self.freshinstancetasks.get_injected_files("test")
         # verify
         self.assertTrue(
             '/etc/guest_info' in files)
@@ -310,8 +321,9 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
                   return_value=fake_InstanceServiceStatus.find_by())
     @patch.object(DBInstance, 'find_by',
                   return_value=fake_DBInstance.find_by())
+    @patch('trove.taskmanager.models.LOG')
     def test_update_status_of_instance_failure(
-            self, dbi_find_by_mock, iss_find_by_mock):
+            self, mock_logging, dbi_find_by_mock, iss_find_by_mock):
         self.task_models_conf_mock.get.return_value = ''
         self.freshinstancetasks.update_statuses_on_time_out()
         self.assertEqual(ServiceStatuses.FAILED_TIMEOUT_GUESTAGENT,
@@ -363,9 +375,20 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
                           self.freshinstancetasks._create_secgroup,
                           datastore_manager)
 
+    def test_create_sg_rules_icmp(self):
+        datastore_manager = 'mysql'
+        self.task_models_conf_mock.get = Mock(
+            return_value=FakeOptGroup(icmp=True))
+        self.freshinstancetasks.update_db = Mock()
+        self.freshinstancetasks._create_secgroup(datastore_manager)
+        self.assertEqual(3, taskmanager_models.SecurityGroupRule.
+                         create_sec_group_rule.call_count)
+
     @patch.object(BaseInstance, 'update_db')
     @patch('trove.taskmanager.models.CONF')
-    def test_error_sec_group_create_instance(self, mock_conf, mock_update_db):
+    @patch('trove.taskmanager.models.LOG')
+    def test_error_sec_group_create_instance(self, mock_logging,
+                                             mock_conf, mock_update_db):
         mock_conf.get = Mock(
             return_value=FakeOptGroup(tcp_ports=['3306', '-3306']))
         mock_flavor = {'id': 7, 'ram': 256, 'name': 'smaller_flavor'}
@@ -374,20 +397,20 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
             'Error creating security group for instance',
             self.freshinstancetasks.create_instance, mock_flavor,
             'mysql-image-id', None, None, 'mysql', 'mysql-server', 2,
-            None, None, None, None, Mock(), None)
+            None, None, None, None, Mock(), None, None, None, None, None)
 
     @patch.object(BaseInstance, 'update_db')
     @patch.object(backup_models.Backup, 'get_by_id')
     @patch.object(taskmanager_models.FreshInstanceTasks, 'report_root_enabled')
-    @patch.object(taskmanager_models.FreshInstanceTasks, '_get_injected_files')
+    @patch.object(taskmanager_models.FreshInstanceTasks, 'get_injected_files')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_create_secgroup')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_build_volume_info')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_create_server')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_guest_prepare')
     @patch.object(template, 'SingleInstanceConfigTemplate')
-    @patch.object(template, 'OverrideConfigTemplate')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_create_dns_entry',
                   side_effect=TroveError)
+    @patch('trove.taskmanager.models.LOG')
     def test_error_create_dns_entry_create_instance(self, *args):
         mock_flavor = {'id': 6, 'ram': 512, 'name': 'big_flavor'}
         self.assertRaisesRegexp(
@@ -395,11 +418,12 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
             'Error creating DNS entry for instance',
             self.freshinstancetasks.create_instance, mock_flavor,
             'mysql-image-id', None, None, 'mysql', 'mysql-server',
-            2, Mock(), None, 'root_password', None, Mock(), None)
+            2, Mock(), None, 'root_password', None, Mock(), None, None, None,
+            None, None)
 
     @patch.object(BaseInstance, 'update_db')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_create_dns_entry')
-    @patch.object(taskmanager_models.FreshInstanceTasks, '_get_injected_files')
+    @patch.object(taskmanager_models.FreshInstanceTasks, 'get_injected_files')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_create_server')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_create_secgroup')
     @patch.object(taskmanager_models.FreshInstanceTasks, '_build_volume_info')
@@ -410,6 +434,8 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
                              mock_guest_prepare,
                              mock_build_volume_info,
                              mock_create_secgroup,
+                             mock_create_server,
+                             mock_get_injected_files,
                              *args):
         mock_flavor = {'id': 8, 'ram': 768, 'name': 'bigger_flavor'}
         config_content = {'config_contents': 'some junk'}
@@ -420,16 +446,24 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
                                                 None, None, 'mysql',
                                                 'mysql-server', 2,
                                                 None, None, None, None,
-                                                overrides, None)
+                                                overrides, None, None,
+                                                'volume_type', None,
+                                                {'group': 'sg-id'})
         mock_create_secgroup.assert_called_with('mysql')
-        mock_build_volume_info.assert_called_with('mysql', volume_size=2)
+        mock_build_volume_info.assert_called_with('mysql', volume_size=2,
+                                                  volume_type='volume_type')
         mock_guest_prepare.assert_called_with(
             768, mock_build_volume_info(), 'mysql-server', None, None, None,
-            config_content, None, overrides, None, None)
+            config_content, None, overrides, None, None, None)
+        mock_create_server.assert_called_with(
+            8, 'mysql-image-id', mock_create_secgroup(),
+            'mysql', mock_build_volume_info()['block_device'], None,
+            None, mock_get_injected_files(), {'group': 'sg-id'})
 
     @patch.object(trove.guestagent.api.API, 'attach_replication_slave')
     @patch.object(rpc, 'get_client')
-    def test_attach_replication_slave(self, mock_get_client,
+    @patch.object(DBInstance, 'get_by')
+    def test_attach_replication_slave(self, mock_get_by, mock_get_client,
                                       mock_attach_replication_slave):
         mock_flavor = {'id': 8, 'ram': 768, 'name': 'bigger_flavor'}
         snapshot = {'replication_strategy': 'MysqlGTIDReplication',
@@ -451,6 +485,8 @@ class FreshInstanceTasksTest(trove_testtools.TestCase):
                   '_render_replica_config')
     @patch.object(trove.guestagent.api.API, 'attach_replication_slave',
                   side_effect=GuestError)
+    @patch('trove.taskmanager.models.LOG')
+    @patch.object(DBInstance, 'get_by')
     def test_error_attach_replication_slave(self, *args):
         mock_flavor = {'id': 8, 'ram': 768, 'name': 'bigger_flavor'}
         snapshot = {'replication_strategy': 'MysqlGTIDReplication',
@@ -478,7 +514,7 @@ class ResizeVolumeTest(trove_testtools.TestCase):
                                                             self.old_vol_size,
                                                             self.new_vol_size)
 
-        class FakeGroup():
+        class FakeGroup(object):
             def __init__(self):
                 self.mount_point = 'var/lib/mysql'
                 self.device_path = '/dev/vdb'
@@ -491,7 +527,8 @@ class ResizeVolumeTest(trove_testtools.TestCase):
     def tearDown(self):
         super(ResizeVolumeTest, self).tearDown()
 
-    def test_resize_volume_unmount_exception(self):
+    @patch('trove.taskmanager.models.LOG')
+    def test_resize_volume_unmount_exception(self, mock_logging):
         self.instance.guest.unmount_volume = Mock(
             side_effect=GuestError("test exception"))
         self.assertRaises(GuestError,
@@ -501,7 +538,8 @@ class ResizeVolumeTest(trove_testtools.TestCase):
         self.instance.guest.unmount_volume.side_effect = None
         self.instance.reset_mock()
 
-    def test_resize_volume_detach_exception(self):
+    @patch('trove.taskmanager.models.LOG')
+    def test_resize_volume_detach_exception(self, mock_logging):
         self.instance.nova_client.volumes.delete_server_volume = Mock(
             side_effect=nova_exceptions.ClientException("test exception"))
         self.assertRaises(nova_exceptions.ClientException,
@@ -513,7 +551,8 @@ class ResizeVolumeTest(trove_testtools.TestCase):
             None)
         self.instance.reset_mock()
 
-    def test_resize_volume_extend_exception(self):
+    @patch('trove.taskmanager.models.LOG')
+    def test_resize_volume_extend_exception(self, mock_logging):
         self.instance.volume_client.volumes.extend = Mock(
             side_effect=cinder_exceptions.ClientException("test exception"))
         self.assertRaises(cinder_exceptions.ClientException,
@@ -527,21 +566,24 @@ class ResizeVolumeTest(trove_testtools.TestCase):
         self.instance.volume_client.volumes.extend.side_effect = None
         self.instance.reset_mock()
 
-    def test_resize_volume_verify_extend_no_volume(self):
+    @patch('trove.taskmanager.models.LOG')
+    def test_resize_volume_verify_extend_no_volume(self, mock_logging):
         self.instance.volume_client.volumes.get = Mock(
             return_value=None)
         self.assertRaises(cinder_exceptions.ClientException,
                           self.action._verify_extend)
         self.instance.reset_mock()
 
-    def test_resize_volume_poll_timeout(self):
+    @patch('trove.taskmanager.models.LOG')
+    def test_resize_volume_poll_timeout(self, mock_logging):
         utils.poll_until = Mock(side_effect=PollTimeOut)
         self.assertRaises(PollTimeOut, self.action._verify_extend)
         self.assertEqual(2, self.instance.volume_client.volumes.get.call_count)
         utils.poll_until.side_effect = None
         self.instance.reset_mock()
 
-    def test_resize_volume_active_server_succeeds(self):
+    @patch.object(TroveInstanceModifyVolume, 'notify')
+    def test_resize_volume_active_server_succeeds(self, *args):
         server = Mock(status=InstanceStatus.ACTIVE)
         self.instance.attach_mock(server, 'server')
         self.action.execute()
@@ -634,7 +676,7 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
 
         self.instance_task._guest = MagicMock(spec=trove.guestagent.api.API)
         self.instance_task._nova_client = MagicMock(
-            spec=novaclient.v2.Client)
+            spec=novaclient.client)
         self.stub_server_mgr = MagicMock(
             spec=novaclient.v2.servers.ServerManager)
         self.stub_running_server = MagicMock(
@@ -657,6 +699,10 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
                                                    True)
         stub_flavor_manager.get = MagicMock(return_value=nova_flavor)
 
+        self.instance_task._volume_client = MagicMock(spec=cinderclient)
+        self.instance_task._volume_client.volumes = Mock(
+            spec=cinderclient_volumes.VolumeManager)
+
         answers = (status for status in
                    self.get_inst_service_status('inst_stat-id',
                                                 [ServiceStatuses.SHUTDOWN,
@@ -666,7 +712,7 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
 
         def side_effect_func(*args, **kwargs):
             if 'instance_id' in kwargs:
-                return answers.next()
+                return next(answers)
             elif ('id' in kwargs and 'deleted' in kwargs
                   and not kwargs['deleted']):
                 return db_instance
@@ -718,7 +764,8 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
         self.assertEqual(1, self.stub_server_mgr.get.call_count)
         self.assertThat(self.db_instance.flavor_id, Is(self.new_flavor['id']))
 
-    def test_resize_flavor_resize_failure(self):
+    @patch('trove.taskmanager.models.LOG')
+    def test_resize_flavor_resize_failure(self, mock_logging):
         orig_server = self.instance_task.server
         self.stub_verifying_server.status = 'ERROR'
         with patch.object(self.instance_task._nova_client.servers, 'get',
@@ -750,7 +797,8 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
         self.instance_task.set_datastore_status_to_paused.assert_any_call()
 
     @patch.object(utils, 'poll_until')
-    def test_reboot_datastore_not_ready(self, mock_poll):
+    @patch('trove.taskmanager.models.LOG')
+    def test_reboot_datastore_not_ready(self, mock_logging, mock_poll):
         self.instance_task.datastore_status_matches = Mock(return_value=False)
         self.instance_task._refresh_datastore_status = Mock()
         self.instance_task.server.reboot = Mock()
@@ -763,15 +811,38 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
 
     @patch.object(BaseInstance, 'update_db')
     def test_detach_replica(self, mock_update_db):
-        self.instance_task.detach_replica(Mock(), True)
-        self.instance_task._guest.detach_replica.assert_called_with(True)
-        mock_update_db.assert_called_with(slave_of_id=None)
+        with patch.object(self.instance_task, 'reset_task_status') as tr_mock:
+            self.instance_task.detach_replica(Mock(), True)
+            self.instance_task._guest.detach_replica.assert_called_with(True)
+            mock_update_db.assert_called_with(slave_of_id=None)
+            tr_mock.assert_not_called()
 
-    def test_error_detach_replica(self):
-        with patch.object(self.instance_task._guest, 'detach_replica',
-                          side_effect=GuestError):
-            self.assertRaises(GuestError, self.instance_task.detach_replica,
-                              Mock(), True)
+        with patch.object(self.instance_task, 'reset_task_status') as tr_mock:
+            self.instance_task.detach_replica(Mock(), False)
+            self.instance_task._guest.detach_replica.assert_called_with(False)
+            mock_update_db.assert_called_with(slave_of_id=None)
+            tr_mock.assert_called_once_with()
+
+    @patch.object(BaseInstance, 'update_db')
+    @patch('trove.taskmanager.models.LOG')
+    def test_error_detach_replica(self, mock_logging, mock_update_db):
+        with patch.object(self.instance_task, 'reset_task_status') as tr_mock:
+            with patch.object(self.instance_task._guest, 'detach_replica',
+                              side_effect=GuestError):
+                self.assertRaises(
+                    GuestError, self.instance_task.detach_replica,
+                    Mock(), True)
+                mock_update_db.assert_not_called()
+                tr_mock.assert_not_called()
+
+        with patch.object(self.instance_task, 'reset_task_status') as tr_mock:
+            with patch.object(self.instance_task._guest, 'detach_replica',
+                              side_effect=GuestError):
+                self.assertRaises(
+                    GuestError, self.instance_task.detach_replica,
+                    Mock(), False)
+                mock_update_db.assert_not_called()
+                tr_mock.assert_called_once_with()
 
     @patch.object(BaseInstance, 'update_db')
     def test_make_read_only(self, mock_update_db):
@@ -782,7 +853,7 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
     @patch.object(BaseInstance, 'update_db')
     def test_attach_replica(self, mock_update_db):
         master = MagicMock()
-        replica_context = Mock()
+        replica_context = trove_testtools.TroveTestContext(self)
         mock_guest = MagicMock()
         mock_guest.get_replica_context = Mock(return_value=replica_context)
         type(master).guest = PropertyMock(return_value=mock_guest)
@@ -799,7 +870,8 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
             replica_context, config_content)
         mock_update_db.assert_called_with(slave_of_id=master.id)
 
-    def test_error_attach_replica(self):
+    @patch('trove.taskmanager.models.LOG')
+    def test_error_attach_replica(self, mock_logging):
         with patch.object(self.instance_task._guest, 'attach_replica',
                           side_effect=GuestError):
             self.assertRaises(GuestError, self.instance_task.attach_replica,
@@ -857,6 +929,36 @@ class BuiltInstanceTasksTest(trove_testtools.TestCase):
         self.instance_task.demote_replication_master()
         self.instance_task._guest.demote_replication_master.assert_any_call()
 
+    @patch.multiple(taskmanager_models.BuiltInstanceTasks,
+                    get_injected_files=Mock(return_value="the-files"))
+    def test_upgrade(self, *args):
+        pre_rebuild_server = self.instance_task.server
+        dsv = Mock(image_id='foo_image')
+        mock_volume = Mock(attachments=[{'device': '/dev/mock_dev'}])
+        with patch.object(self.instance_task._volume_client.volumes, "get",
+                          Mock(return_value=mock_volume)):
+            mock_server = Mock(status='ACTIVE')
+            with patch.object(self.instance_task._nova_client.servers,
+                              'get', Mock(return_value=mock_server)):
+                with patch.multiple(self.instance_task._guest,
+                                    pre_upgrade=Mock(return_value={}),
+                                    post_upgrade=Mock()):
+                    self.instance_task.upgrade(dsv)
+
+                    self.instance_task._guest.pre_upgrade.assert_called_with()
+                    pre_rebuild_server.rebuild.assert_called_with(
+                        dsv.image_id, files="the-files")
+                    self.instance_task._guest.post_upgrade.assert_called_with(
+                        mock_volume.attachments[0])
+
+    def test_fix_device_path(self):
+        self.assertEqual("/dev/vdb", self.instance_task.
+                         _fix_device_path("vdb"))
+        self.assertEqual("/dev/dev", self.instance_task.
+                         _fix_device_path("dev"))
+        self.assertEqual("/dev/vdb/dev", self.instance_task.
+                         _fix_device_path("vdb/dev"))
+
 
 class BackupTasksTest(trove_testtools.TestCase):
 
@@ -913,7 +1015,8 @@ class BackupTasksTest(trove_testtools.TestCase):
                                                      self.backup.id)
         self.backup.delete.assert_any_call()
 
-    def test_delete_backup_fail_delete_manifest(self):
+    @patch('trove.taskmanager.models.LOG')
+    def test_delete_backup_fail_delete_manifest(self, mock_logging):
         with patch.object(self.swift_client, 'delete_object',
                           side_effect=ClientException("foo")):
             with patch.object(self.swift_client, 'head_object',
@@ -928,7 +1031,8 @@ class BackupTasksTest(trove_testtools.TestCase):
                     self.backup.state,
                     "backup should be in DELETE_FAILED status")
 
-    def test_delete_backup_fail_delete_segment(self):
+    @patch('trove.taskmanager.models.LOG')
+    def test_delete_backup_fail_delete_segment(self, mock_logging):
         with patch.object(self.swift_client, 'delete_object',
                           side_effect=ClientException("foo")):
             self.assertRaises(
@@ -975,7 +1079,8 @@ class NotifyMixinTest(trove_testtools.TestCase):
         mixin = taskmanager_models.NotifyMixin()
         self.assertThat(mixin._get_service_id('mysql', id_map), Equals('123'))
 
-    def test_get_service_id_unknown(self):
+    @patch('trove.taskmanager.models.LOG')
+    def test_get_service_id_unknown(self, mock_logging):
         id_map = {
             'mysql': '123',
             'percona': 'abc'
@@ -1009,3 +1114,25 @@ class RootReportTest(trove_testtools.TestCase):
             self.assertTrue(mysql_models.RootHistory.load.called)
             self.assertEqual(history.user, report.user)
             self.assertEqual(history.id, report.id)
+
+
+class ClusterRootTest(trove_testtools.TestCase):
+
+    @patch.object(common_models.RootHistory, "create")
+    @patch.object(common_models.Root, "create")
+    def test_cluster_root_create(self, root_create, root_history_create):
+        context = Mock()
+        uuid = utils.generate_uuid()
+        user = "root"
+        password = "rootpassword"
+        cluster_instances = [utils.generate_uuid(), utils.generate_uuid()]
+        common_models.ClusterRoot.create(context, uuid, user, password,
+                                         cluster_instances)
+        root_create.assert_called_with(context, uuid, user, password,
+                                       cluster_instances_list=None)
+        self.assertEqual(2, root_history_create.call_count)
+        calls = [
+            call(context, cluster_instances[0], user),
+            call(context, cluster_instances[1], user)
+        ]
+        root_history_create.assert_has_calls(calls)
